@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -858,7 +858,7 @@ typedef struct VBOXVHWACMD_HH_SAVESTATE_LOADPERFORM
     struct SSMHANDLE * pSSM;
 } VBOXVHWACMD_HH_SAVESTATE_LOADPERFORM;
 
-typedef DECLCALLBACK(void) FNVBOXVHWA_HH_CALLBACK(void*);
+typedef DECLCALLBACKTYPE(void, FNVBOXVHWA_HH_CALLBACK,(void *));
 typedef FNVBOXVHWA_HH_CALLBACK *PFNVBOXVHWA_HH_CALLBACK;
 
 #define VBOXVHWA_HH_CALLBACK_SET(_pCmd, _pfn, _parg) \
