@@ -3,24 +3,34 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef VBOX_INCLUDED_RemoteDesktop_VRDE_h
@@ -1569,14 +1579,14 @@ typedef VRDECALLBACKS_3 VRDECALLBACKS_4;
  * @return IPRT status code.
  */
 DECLEXPORT(int) VRDECreateServer (const VRDEINTERFACEHDR *pCallbacks,
-                                    void *pvCallback,
-                                    VRDEINTERFACEHDR **ppEntryPoints,
-                                    HVRDESERVER *phServer);
+                                  void *pvCallback,
+                                  VRDEINTERFACEHDR **ppEntryPoints,
+                                  HVRDESERVER *phServer);
 
-typedef DECLCALLBACK(int) FNVRDECREATESERVER (const VRDEINTERFACEHDR *pCallbacks,
-                                              void *pvCallback,
-                                              VRDEINTERFACEHDR **ppEntryPoints,
-                                              HVRDESERVER *phServer);
+typedef DECLCALLBACKTYPE(int, FNVRDECREATESERVER,(const VRDEINTERFACEHDR *pCallbacks,
+                                                  void *pvCallback,
+                                                  VRDEINTERFACEHDR **ppEntryPoints,
+                                                  HVRDESERVER *phServer));
 typedef FNVRDECREATESERVER *PFNVRDECREATESERVER;
 
 /**
@@ -1595,7 +1605,7 @@ typedef FNVRDECREATESERVER *PFNVRDECREATESERVER;
  */
 DECLEXPORT(const char * const *) VRDESupportedProperties (void);
 
-typedef DECLCALLBACK(const char * const *) FNVRDESUPPORTEDPROPERTIES (void);
+typedef DECLCALLBACKTYPE(const char * const *, FNVRDESUPPORTEDPROPERTIES,(void));
 typedef FNVRDESUPPORTEDPROPERTIES *PFNVRDESUPPORTEDPROPERTIES;
 
 RT_C_DECLS_END
