@@ -3,28 +3,41 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-#ifndef ___VBox_disopcode_h
-#define ___VBox_disopcode_h
+#ifndef VBOX_INCLUDED_disopcode_h
+#define VBOX_INCLUDED_disopcode_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/assert.h>
 
@@ -129,12 +142,14 @@ enum OPCODES
     OP_ENTER,
     OP_LEAVE,
     OP_RETF,
+    OP_INT1,
     OP_INT3,
     OP_INT,
     OP_INTO,
     OP_IRET,
     OP_AAM,
     OP_AAD,
+    OP_SALC,
     OP_XLAT,
     OP_ESCF0,
     OP_ESCF1,
@@ -195,6 +210,7 @@ enum OPCODES
     OP_COMISS,
     OP_WRMSR,
     OP_RDTSC,
+    OP_RDTSCP,
     OP_RDMSR,
     OP_RDPMC,
     OP_SYSENTER,
@@ -373,6 +389,7 @@ enum OPCODES
     OP_GRP12,
     OP_GRP13,
     OP_GRP14,
+    OP_GRP17,
     OP_EMMS,
     OP_MMX_UD78,
     OP_MMX_UD79,
@@ -423,8 +440,18 @@ enum OPCODES
     OP_PABSB,
     OP_PABSW,
     OP_PABSD,
-    OP_PMOVSX,
-    OP_PMOVZX,
+    OP_PMOVSXBW,
+    OP_PMOVSXBD,
+    OP_PMOVSXBQ,
+    OP_PMOVSXWD,
+    OP_PMOVSXWQ,
+    OP_PMOVSXDQ,
+    OP_PMOVZXBW,
+    OP_PMOVZXBD,
+    OP_PMOVZXBQ,
+    OP_PMOVZXWD,
+    OP_PMOVZXWQ,
+    OP_PMOVZXDQ,
     OP_PMULDQ,
     OP_PMINSB,
     OP_PMINSD,
@@ -483,8 +510,7 @@ enum OPCODES
     OP_AESDECLAST,
     OP_MOVBEGM,
     OP_MOVBEMG,
-    OP_CRC32GDEB,
-    OP_CRC32GDEY,
+    OP_CRC32,
     OP_POPCNT,
     OP_TZCNT,
     OP_LZCNT,
@@ -493,10 +519,14 @@ enum OPCODES
     OP_ANDN,
     OP_BZHI,
     OP_BEXTR,
+    OP_BLSR,
+    OP_BLSMSK,
+    OP_BLSI,
     OP_PEXT,
-    OP_SARX,
     OP_PDEP,
+    OP_SHLX,
     OP_SHRX,
+    OP_SARX,
     OP_MULX,
     OP_MASKMOVDQU,
     OP_MASKMOVPS,
@@ -554,12 +584,14 @@ enum OPCODES
     OP_PALIGNR,
     OP_PEXTRB,
     OP_PEXTRD,
+    OP_PEXTRQ,
     OP_EXTRACTPS,
     OP_INSERTF128,
     OP_EXTRACTF128,
     OP_CVTPS2PH,
     OP_PINSRB,
     OP_PINSRD,
+    OP_PINSRQ,
     OP_INSERTPS,
     OP_INSERTI128,
     OP_EXTRACTI128,
@@ -574,6 +606,9 @@ enum OPCODES
     OP_PCMPISTRI,
     OP_AESKEYGEN,
     OP_RORX,
+    OP_RDRAND,
+    OP_RDSEED,
+    OP_MOVBE,
     OP_VEX3B,
     OP_VEX2B,
 /** @} */
@@ -727,6 +762,8 @@ enum OPCODES
     OP_XSAVE,
     OP_XSAVEOPT,
     OP_XRSTOR,
+    OP_XGETBV,
+    OP_XSETBV,
     OP_RDFSBASE,
     OP_RDGSBASE,
     OP_WRFSBASE,
@@ -778,31 +815,319 @@ enum OPCODES
 /** @} */
 /** @name AVX instructions
  * @{ */
-    OP_VLDMXCSR,
+    /* Manual */
     OP_VSTMXCSR,
-    OP_VMOVUPS,
-    OP_VMOVUPD,
-    OP_VMOVSS,
-    OP_VMOVSD,
-    OP_VMOVHLPS,
-    OP_VMOVLPS,
-    OP_VMOVLPD,
-    OP_VMOVSLDUP,
-    OP_VMOVDDUP,
-    OP_VMOVAPS,
+    OP_VLDMXCSR,
+    OP_VPACKUSDW,
+
+    /* Generated from tables: */
+    OP_VADDPD,
+    OP_VADDPS,
+    OP_VADDSD,
+    OP_VADDSS,
+    OP_VADDSUBPD,
+    OP_VADDSUBPS,
+    OP_VAESDEC,
+    OP_VAESDECLAST,
+    OP_VAESENC,
+    OP_VAESENCLAST,
+    OP_VAESIMC,
+    OP_VAESKEYGEN,
+    OP_VANDNPD,
+    OP_VANDNPS,
+    OP_VANDPD,
+    OP_VANDPS,
+    OP_VBLENDPD,
+    OP_VBLENDPS,
+    OP_VBLENDVPD,
+    OP_VBLENDVPS,
+    OP_VBROADCASTF128,
+    OP_VBROADCASTSD,
+    OP_VBROADCASTSS,
+    OP_VCMPSD,
+    OP_VCMPSS,
+    OP_VCOMISD,
+    OP_VCOMISS,
+    OP_VCVTDQ2PD,
+    OP_VCVTDQ2PS,
+    OP_VCVTPD2DQ,
+    OP_VCVTPD2PS,
+    OP_VCVTPH2PS,
+    OP_VCVTPS2DQ,
+    OP_VCVTPS2PD,
+    OP_VCVTPS2PH,
+    OP_VCVTSD2SS,
+    OP_VCVTSI2SS,
+    OP_VCVTSS2SD,
+    OP_VCVTSS2SI,
+    OP_VCVTTPD2DQ,
+    OP_VCVTTPS2DQ,
+    OP_VCVTTSS2SI,
+    OP_VDIVPD,
+    OP_VDIVPS,
+    OP_VDIVSD,
+    OP_VDIVSS,
+    OP_VDPPD,
+    OP_VDPPS,
+    OP_VEXTRACTF128,
+    OP_VEXTRACTI128,
+    OP_VEXTRACTPS,
+    OP_VFMADD132PS,
+    OP_VFMADD132SS,
+    OP_VFMADD213PS,
+    OP_VFMADD213SS,
+    OP_VFMADD231PS,
+    OP_VFMADD231SS,
+    OP_VFMADDSUB132PS,
+    OP_VFMADDSUB213PS,
+    OP_VFMADDSUB231PS,
+    OP_VFMSUB132PS,
+    OP_VFMSUB132SS,
+    OP_VFMSUB213PS,
+    OP_VFMSUB213SS,
+    OP_VFMSUB231PS,
+    OP_VFMSUB231SS,
+    OP_VFMSUBADD132PS,
+    OP_VFMSUBADD213PS,
+    OP_VFMSUBADD231PS,
+    OP_VFNMADD132PS,
+    OP_VFNMADD132SS,
+    OP_VFNMADD213PS,
+    OP_VFNMADD213SS,
+    OP_VFNMADD231PS,
+    OP_VFNMADD231SS,
+    OP_VFNMSUB132PS,
+    OP_VFNMSUB132SS,
+    OP_VFNMSUB213PS,
+    OP_VFNMSUB213SS,
+    OP_VFNMSUB231PS,
+    OP_VFNMSUB231SS,
+    OP_VGATHER,
+    OP_VHADDPD,
+    OP_VHADDPS,
+    OP_VHSUBPD,
+    OP_VHSUBPS,
+    OP_VINSERTF128,
+    OP_VINSERTI128,
+    OP_VINSERTPS,
+    OP_VLDDQU,
+    OP_VMASKMOVDQU,
+    OP_VMASKMOVPD,
+    OP_VMASKMOVPS,
+    OP_VMAXPD,
+    OP_VMAXPS,
+    OP_VMAXSD,
+    OP_VMAXSS,
+    OP_VMINPD,
+    OP_VMINPS,
+    OP_VMINSD,
+    OP_VMINSS,
     OP_VMOVAPD,
-    OP_VMOVNTPS,
-    OP_VMOVNTPD,
+    OP_VMOVAPS,
     OP_VMOVD,
-    OP_VMOVQ,
+    OP_VMOVDDUP,
     OP_VMOVDQA,
     OP_VMOVDQU,
+    OP_VMOVHPD,
+    OP_VMOVHPS,
+    OP_VMOVLHPS = OP_VMOVHPS, /**< @todo OP_VMOVHPS */
+    OP_VMOVLPD,
+    OP_VMOVLPS,
+    OP_VMOVHLPS = OP_VMOVLPS, /**< @todo OP_VMOVLPS */
+    OP_VMOVMSKPD,
+    OP_VMOVMSKPS,
     OP_VMOVNTDQ,
     OP_VMOVNTDQA,
+    OP_VMOVNTPD,
+    OP_VMOVNTPS,
+    OP_VMOVQ,
+    OP_VMOVSD,
+    OP_VMOVSHDUP,
+    OP_VMOVSLDUP,
+    OP_VMOVSS,
+    OP_VMOVUPD,
+    OP_VMOVUPS,
+    OP_VMPSADBW,
+    OP_VMULPD,
+    OP_VMULPS,
+    OP_VMULSD,
+    OP_VMULSS,
+    OP_VORPD,
+    OP_VORPS,
+    OP_VPABSB,
+    OP_VPABSD,
+    OP_VPABSW,
+    OP_VPACKSSDW,
+    OP_VPACKSSWB,
+    OP_VPACKUSWB,
+    OP_VPADDB,
+    OP_VPADDD,
+    OP_VPADDQ,
+    OP_VPADDSB,
+    OP_VPADDSW,
+    OP_VPADDUSB,
+    OP_VPADDUSW,
+    OP_VPADDW,
+    OP_VPALIGNR,
+    OP_VPAND,
+    OP_VPANDN,
+    OP_VPAVGB,
+    OP_VPAVGW,
+    OP_VPBLENDD,
+    OP_VPBLENDVB,
+    OP_VPBLENDW,
+    OP_VPBROADCASTB,
+    OP_VPBROADCASTD,
+    OP_VPBROADCASTI128,
+    OP_VPBROADCASTQ,
+    OP_VPBROADCASTW,
+    OP_VPCLMULQDQ,
+    OP_VPCMPEQB,
+    OP_VPCMPEQD,
+    OP_VPCMPEQQ,
+    OP_VPCMPEQW,
+    OP_VPCMPESTRI,
+    OP_VPCMPESTRM,
+    OP_VPCMPGTB,
+    OP_VPCMPGTD,
+    OP_VPCMPGTQ,
+    OP_VPCMPGTW,
+    OP_VPCMPISTRI,
+    OP_VPCMPISTRM,
+    OP_VPERM2F128,
+    OP_VPERM2I128,
+    OP_VPERMD,
+    OP_VPERMILPD,
+    OP_VPERMILPS,
+    OP_VPERMPD,
+    OP_VPERMPS,
+    OP_VPERMQ,
+    OP_VPEXTRB,
+    OP_VPEXTRD,
+    OP_VPEXTRW,
+    OP_VPEXTRQ,
+    OP_VPHADDD,
+    OP_VPHADDSW,
+    OP_VPHADDW,
+    OP_VPHMINPOSUW,
+    OP_VPHSUBD,
+    OP_VPHSUBSW,
+    OP_VPHSUBW,
+    OP_VPINSRB,
+    OP_VPINSRD,
+    OP_VPINSRW,
+    OP_VPINSRQ,
+    OP_VPMADDUBSW,
+    OP_VPMADDWD,
+    OP_VPMASKMOVD,
+    OP_VPMAXSB,
+    OP_VPMAXSD,
+    OP_VPMAXSW,
+    OP_VPMAXUB,
+    OP_VPMAXUD,
+    OP_VPMAXUW,
+    OP_VPMINSB,
+    OP_VPMINSD,
+    OP_VPMINSW,
+    OP_VPMINUB,
+    OP_VPMINUD,
+    OP_VPMINUW,
+    OP_VPMOVMSKB,
+    OP_VPMOVSXBW,
+    OP_VPMOVSXBD,
+    OP_VPMOVSXBQ,
+    OP_VPMOVSXWD,
+    OP_VPMOVSXWQ,
+    OP_VPMOVSXDQ,
+    OP_VPMOVZXBW,
+    OP_VPMOVZXBD,
+    OP_VPMOVZXBQ,
+    OP_VPMOVZXWD,
+    OP_VPMOVZXWQ,
+    OP_VPMOVZXDQ,
+    OP_VPMULDQ,
+    OP_VPMULHRSW,
+    OP_VPMULHUW,
+    OP_VPMULHW,
+    OP_VPMULLD,
+    OP_VPMULLW,
+    OP_VPMULUDQ,
+    OP_VPOR,
+    OP_VPSADBW,
+    OP_VPSHUFB,
+    OP_VPSHUFD,
+    OP_VPSHUFHW,
+    OP_VPSHUFLW,
+    OP_VPSIGNB,
+    OP_VPSIGND,
+    OP_VPSIGNW,
+    OP_VPSLLD,
+    OP_VPSLLQ,
+    OP_VPSLLVD,
+    OP_VPSLLW,
+    OP_VPSRAD,
+    OP_VPSRAVD,
+    OP_VPSRAW,
+    OP_VPSRLD,
+    OP_VPSRLQ,
+    OP_VPSRLVD,
+    OP_VPSRLW,
+    OP_VPSUBB,
+    OP_VPSUBD,
+    OP_VPSUBQ,
+    OP_VPSUBSB,
+    OP_VPSUBSW,
+    OP_VPSUBUSB,
+    OP_VPSUBUSW,
+    OP_VPSUBW,
+    OP_VPTEST,
+    OP_VPUNPCKHBW,
+    OP_VPUNPCKHDQ,
+    OP_VPUNPCKHQDQ,
+    OP_VPUNPCKHWD,
+    OP_VPUNPCKLBW,
+    OP_VPUNPCKLDQ,
+    OP_VPUNPCKLQDQ,
+    OP_VPUNPCKLWD,
+    OP_VPXOR,
+    OP_VRCPPS,
+    OP_VRCPSS,
+    OP_VROUNDPD,
+    OP_VROUNDPS,
+    OP_VROUNDSD,
+    OP_VROUNDSS,
+    OP_VRSQRTPS,
+    OP_VRSQRTSS,
+    OP_VSHUFPD,
+    OP_VSHUFPS,
+    OP_VSQRTPD,
+    OP_VSQRTPS,
+    OP_VSQRTSD,
+    OP_VSQRTSS,
+    OP_VSUBPD,
+    OP_VSUBPS,
+    OP_VSUBSD,
+    OP_VSUBSS,
+    OP_VTESTPD,
+    OP_VTESTPS,
+    OP_VUCOMISD,
+    OP_VUCOMISS,
+    OP_VUNPCKHPD,
+    OP_VUNPCKHPS,
+    OP_VUNPCKLPD,
+    OP_VUNPCKLPS,
+    OP_VVPACKUSDW,
+    OP_VXORPD,
+    OP_VXORPS,
+    OP_VZEROALL,
+
 /** @} */
     OP_END_OF_OPCODES
 };
 AssertCompile(OP_LOCK == 7);
+#if 0
+AssertCompile(OP_END_OF_OPCODES < 1024 /* see 15 byte DISOPCODE variant */);
+#endif
 /** @} */
 
 
@@ -1056,7 +1381,7 @@ enum OP_PARM
 #define OP_PARM_Vx              (OP_PARM_V+OP_PARM_x)
 #define OP_PARM_Vy              (OP_PARM_V+OP_PARM_y)
 #define OP_PARM_Wq              (OP_PARM_W+OP_PARM_q)
-//#define OP_PARM_Ws              (OP_PARM_W+OP_PARM_s) - wtf? Same as lgdt (OP_PARM_Ms)?
+/*#define OP_PARM_Ws              (OP_PARM_W+OP_PARM_s) - wtf? Same as lgdt (OP_PARM_Ms)?*/
 #define OP_PARM_Wx              (OP_PARM_W+OP_PARM_x)
 #define OP_PARM_Xb              (OP_PARM_X+OP_PARM_b)
 #define OP_PARM_Xv              (OP_PARM_X+OP_PARM_v)
@@ -1140,5 +1465,5 @@ enum OP_PARM
 
 /** @} */
 
-#endif
+#endif /* !VBOX_INCLUDED_disopcode_h */
 
