@@ -3,24 +3,34 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef IPRT_INCLUDED_crypto_pkix_h
@@ -49,7 +59,8 @@ struct RTCRX509SUBJECTPUBLICKEYINFO;
  * @returns IPRT status code.
  * @param   pAlgorithm      The signature algorithm (digest w/ cipher).
  * @param   hPublicKey      The public key.
- * @param   pParameters     Parameter to the public key algorithm. Optional.
+ * @param   pParameters     The signature parameters (not key, those are already
+ *                          kept by hPublicKey).
  * @param   pSignatureValue The signature value.
  * @param   pvData          The signed data.
  * @param   cbData          The amount of signed data.
@@ -70,7 +81,8 @@ RTDECL(int) RTCrPkixPubKeyVerifySignature(PCRTASN1OBJID pAlgorithm, RTCRKEY hPub
  * @returns IPRT status code.
  * @param   pAlgorithm      The signature algorithm (digest w/ cipher).
  * @param   hPublicKey      The public key.
- * @param   pParameters     Parameter to the public key algorithm. Optional.
+ * @param   pParameters     The signature parameters (not key, those are already
+ *                          kept by hPublicKey).
  * @param   pvSignedDigest  The signed digest.
  * @param   cbSignedDigest  The signed digest size.
  * @param   hDigest         The digest of the data to compare @a pvSignedDigest
@@ -135,10 +147,22 @@ RTDECL(int) RTCrPkixPubKeySignDigest(PCRTASN1OBJID pAlgorithm, RTCRKEY hPrivateK
  * Gets the cipher OID matching the given signature algorithm.
  *
  * @returns Cipher OID string on success, NULL on failure.
- * @param   pAlgorithm      The signature algorithm (digest w/ cipher).
+ * @param   pAlgorithm          The signature algorithm (hash function w/ cipher).
+ * @sa      RTCrX509AlgorithmIdentifier_GetEncryptionOid,
+ *          RTCrX509AlgorithmIdentifier_GetEncryptionOidFromOid
  */
 RTDECL(const char *) RTCrPkixGetCiperOidFromSignatureAlgorithm(PCRTASN1OBJID pAlgorithm);
 
+/**
+ * Gets the cipher OID matching the given signature algorithm OID.
+ *
+ * @returns Cipher OID string on success, NULL on failure.
+ * @param   pszSignatureOid     The signature algorithm ID (hash function w/ cipher).
+ * @sa      RTCrX509AlgorithmIdentifier_GetEncryptionOid,
+ *          RTCrX509AlgorithmIdentifier_GetEncryptionOidFromOid
+ */
+RTDECL(const char *) RTCrPkixGetCiperOidFromSignatureAlgorithmOid(const char *pszSignatureOid);
+
 
 /** @name PKCS-1 Object Identifiers (OIDs)
  * @{ */
@@ -159,6 +183,32 @@ RTDECL(const char *) RTCrPkixGetCiperOidFromSignatureAlgorithm(PCRTASN1OBJID pAl
 #define RTCR_PKCS1_SHA224_WITH_RSA_OID              "1.2.840.113549.1.1.14"
 #define RTCR_PKCS1_SHA512T224_WITH_RSA_OID          "1.2.840.113549.1.1.15"
 #define RTCR_PKCS1_SHA512T256_WITH_RSA_OID          "1.2.840.113549.1.1.16"
+/** @} */
+
+/** @name ANSI X9.62 Object Identifiers (OIDs)
+ * @{ */
+#define RTCR_X962_ECDSA_OID                         "1.2.840.10045.2.1"
+#define RTCR_X962_ECDSA_WITH_SHA1_OID               "1.2.840.10045.4.1"
+#define RTCR_X962_ECDSA_WITH_SHA2_OID               "1.2.840.10045.4.3"
+#define RTCR_X962_ECDSA_WITH_SHA224_OID             "1.2.840.10045.4.3.1"
+#define RTCR_X962_ECDSA_WITH_SHA256_OID             "1.2.840.10045.4.3.2"
+#define RTCR_X962_ECDSA_WITH_SHA384_OID             "1.2.840.10045.4.3.3"
+#define RTCR_X962_ECDSA_WITH_SHA512_OID             "1.2.840.10045.4.3.4"
+/** @}  */
+
+/** @name NIST Object Identifiers (OIDs)
+ * @{ */
+#define RTCR_NIST_ALGORITHM_OID                     "2.16.840.1.101.3.4"
+#define RTCR_NIST_HASH_ALGS_OID                     "2.16.840.1.101.3.4.2"
+#define RTCR_NIST_SIG_ALGS_OID                      "2.16.840.1.101.3.4.3"
+#define RTCR_NIST_SHA3_224_WITH_ECDSA_OID           "2.16.840.1.101.3.4.3.9"
+#define RTCR_NIST_SHA3_256_WITH_ECDSA_OID           "2.16.840.1.101.3.4.3.10"
+#define RTCR_NIST_SHA3_384_WITH_ECDSA_OID           "2.16.840.1.101.3.4.3.11"
+#define RTCR_NIST_SHA3_512_WITH_ECDSA_OID           "2.16.840.1.101.3.4.3.12"
+#define RTCR_NIST_SHA3_224_WITH_RSA_OID             "2.16.840.1.101.3.4.3.13"
+#define RTCR_NIST_SHA3_256_WITH_RSA_OID             "2.16.840.1.101.3.4.3.14"
+#define RTCR_NIST_SHA3_384_WITH_RSA_OID             "2.16.840.1.101.3.4.3.15"
+#define RTCR_NIST_SHA3_512_WITH_RSA_OID             "2.16.840.1.101.3.4.3.16"
 /** @}  */
 
 
@@ -199,8 +249,8 @@ typedef struct RTCRPKIXSIGNATUREDESC
      * @param   pParams         Algorithm/key parameters, optional.  Will be NULL if
      *                          none.
      */
-    DECLCALLBACKMEMBER(int, pfnInit)(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, void *pvOpaque, bool fSigning,
-                                     RTCRKEY hKey, PCRTASN1DYNTYPE pParams);
+    DECLCALLBACKMEMBER(int, pfnInit,(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, void *pvOpaque, bool fSigning,
+                                     RTCRKEY hKey, PCRTASN1DYNTYPE pParams));
 
     /**
      * Resets the state before performing another signing or verification.
@@ -213,7 +263,7 @@ typedef struct RTCRPKIXSIGNATUREDESC
      * @param   pvState         The opaque provider state.
      * @param   fSigning        Exactly the same value as the init call.
      */
-    DECLCALLBACKMEMBER(int, pfnReset)(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, bool fSigning);
+    DECLCALLBACKMEMBER(int, pfnReset,(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, bool fSigning));
 
     /**
      * Deletes the provider state. Optional.
@@ -225,7 +275,7 @@ typedef struct RTCRPKIXSIGNATUREDESC
      * @param   pvState         The opaque provider state.
      * @param   fSigning        Exactly the same value as the init call.
      */
-    DECLCALLBACKMEMBER(void, pfnDelete)(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, bool fSigning);
+    DECLCALLBACKMEMBER(void, pfnDelete,(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, bool fSigning));
 
     /**
      * Verifies a signed message digest (fSigning = false).
@@ -248,8 +298,8 @@ typedef struct RTCRPKIXSIGNATUREDESC
      * @param   pvSignature     The signature to validate.
      * @param   cbSignature     The size of the signature (in bytes).
      */
-    DECLCALLBACKMEMBER(int, pfnVerify)(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, RTCRKEY hKey,
-                                       RTCRDIGEST hDigest, void const *pvSignature, size_t cbSignature);
+    DECLCALLBACKMEMBER(int, pfnVerify,(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, RTCRKEY hKey,
+                                       RTCRDIGEST hDigest, void const *pvSignature, size_t cbSignature));
 
     /**
      * Sign a message digest (fSigning = true).
@@ -276,8 +326,8 @@ typedef struct RTCRPKIXSIGNATUREDESC
      *                          of the returned signature, or the required size in
      *                          case of VERR_BUFFER_OVERFLOW.
      */
-    DECLCALLBACKMEMBER(int, pfnSign)(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, RTCRKEY hKey,
-                                     RTCRDIGEST hDigest, void *pvSignature, size_t *pcbSignature);
+    DECLCALLBACKMEMBER(int, pfnSign,(struct RTCRPKIXSIGNATUREDESC const *pDesc, void *pvState, RTCRKEY hKey,
+                                     RTCRDIGEST hDigest, void *pvSignature, size_t *pcbSignature));
 
 } RTCRPKIXSIGNATUREDESC;
 /** Pointer to a public key signature scheme provider descriptor. */
@@ -289,7 +339,7 @@ typedef RTCRPKIXSIGNATUREDESC const *PCRTCRPKIXSIGNATUREDESC;
  * @param   pszObjId    The ID of the signature to search for.
  * @param   ppvOpaque   Where to store an opaque schema parameter. Optional.
  */
-PCRTCRPKIXSIGNATUREDESC RTCrPkixSignatureFindByObjIdString(const char *pszObjId, void *ppvOpaque);
+PCRTCRPKIXSIGNATUREDESC RTCrPkixSignatureFindByObjIdString(const char *pszObjId, void **ppvOpaque);
 
 /**
  * Locates a signature schema provider descriptor by ASN.1 object ID.
@@ -434,8 +484,8 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      * @param   pParams         Algorithm/key parameters, optional.  Will be NULL if
      *                          none.
      */
-    DECLCALLBACKMEMBER(int, pfnInit)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, void *pvOpaque, bool fEncrypt,
-                                     PCRTASN1BITSTRING pKey, PCRTASN1DYNTYPE pParams);
+    DECLCALLBACKMEMBER(int, pfnInit,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, void *pvOpaque, bool fEncrypt,
+                                     PCRTASN1BITSTRING pKey, PCRTASN1DYNTYPE pParams));
 
     /**
      * Re-initializes the provider state.
@@ -450,7 +500,7 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      * @param   pvState         The opaque provider state.
      * @param   enmOperation    Same as for the earlier pfnInit call.
      */
-    DECLCALLBACKMEMBER(int, pfnReset)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, bool fEncrypt);
+    DECLCALLBACKMEMBER(int, pfnReset,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, bool fEncrypt));
 
     /**
      * Deletes the provider state. Optional.
@@ -463,7 +513,7 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      * @param   pvState         The opaque provider state.
      * @param   enmOperation    Same as for the earlier pfnInit call.
      */
-    DECLCALLBACKMEMBER(void, pfnDelete)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, bool fEncrypt);
+    DECLCALLBACKMEMBER(void, pfnDelete,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState, bool fEncrypt));
 
     /**
      * Encrypt using the public key (fEncrypt = true).
@@ -486,9 +536,9 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      *                          ciphertext returned.
      * @param   fFinal          Whether this is the final call.
      */
-    DECLCALLBACKMEMBER(int, pfnEncrypt)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
+    DECLCALLBACKMEMBER(int, pfnEncrypt,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
                                         void const *pvPlaintext, size_t cbPlaintext,
-                                        void *pvCiphertext, size_t cbMaxCiphertext, size_t *pcbCiphertext, bool fFinal);
+                                        void *pvCiphertext, size_t cbMaxCiphertext, size_t *pcbCiphertext, bool fFinal));
 
     /**
      * Calculate the output buffer size for the next pfnEncrypt call.
@@ -502,8 +552,8 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      *                          be larger than the actual number of bytes return.
      * @param   fFinal          Whether this is the final call.
      */
-    DECLCALLBACKMEMBER(int, pfnEncryptLength)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
-                                              size_t cbPlaintext, size_t *pcbCiphertext, bool fFinal);
+    DECLCALLBACKMEMBER(int, pfnEncryptLength,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
+                                              size_t cbPlaintext, size_t *pcbCiphertext, bool fFinal));
 
     /**
      * Decrypt using the private key (fEncrypt = false).
@@ -526,9 +576,9 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      *                          plaintext returned.
      * @param   fFinal          Whether this is the final call.
      */
-    DECLCALLBACKMEMBER(int, pfnDecrypt)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
+    DECLCALLBACKMEMBER(int, pfnDecrypt,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
                                         void const *pvCiphertext, size_t cbCiphertext,
-                                        void *pvPlaintext, size_t cbMaxPlaintext, size_t *pcbPlaintext, bool fFinal);
+                                        void *pvPlaintext, size_t cbMaxPlaintext, size_t *pcbPlaintext, bool fFinal));
 
     /**
      * Calculate the output buffer size for the next pfnDecrypt call.
@@ -542,8 +592,8 @@ typedef struct RTCRPKIXENCRYPTIONDESC
      *                          be larger than the actual number of bytes return.
      * @param   fFinal          Whether this is the final call.
      */
-    DECLCALLBACKMEMBER(int, pfnDecryptLength)(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
-                                              size_t cbCiphertext, size_t *pcbPlaintext, bool fFinal);
+    DECLCALLBACKMEMBER(int, pfnDecryptLength,(struct RTCRPKIXENCRYPTIONDESC const *pDesc, void *pvState,
+                                              size_t cbCiphertext, size_t *pcbPlaintext, bool fFinal));
 } RTCRPKIXENCRYPTIONDESC;
 /** Pointer to a public key encryption schema provider descriptor. */
 typedef RTCRPKIXENCRYPTIONDESC const *PCRTCRPKIXENCRYPTIONDESC;
