@@ -3,24 +3,34 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 #ifndef IPRT_INCLUDED_vfslowlevel_h
@@ -161,6 +171,30 @@ DECLINLINE(void) RTVfsLockReleaseWrite(RTVFSLOCK hLock)
 
 /** @}  */
 
+/**
+ * Info queried via RTVFSOBJOPS::pfnQueryInfoEx, ++.
+ */
+typedef enum RTVFSQIEX
+{
+    /** Invalid zero value. */
+    RTVFSQIEX_INVALID = 0,
+    /** Volume label.
+     * Returns a UTF-8 string. */
+    RTVFSQIEX_VOL_LABEL,
+    /** Alternative volume label, the primary one for ISOs, otherwise treated same
+     * as RTVFSQIEX_VOL_LABEL. */
+    RTVFSQIEX_VOL_LABEL_ALT,
+    /** Volume serial number.
+     * Returns a uint32_t, uint64_t or RTUUID. */
+    RTVFSQIEX_VOL_SERIAL,
+    /** End of valid queries. */
+    RTVFSQIEX_END,
+
+    /** The usual 32-bit hack. */
+    RTVFSQIEX_32BIT_SIZE_HACK = 0x7fffffff
+} RTVFSQIEX;
+
+
 /**
  * The basis for all virtual file system objects.
  */
@@ -179,19 +213,36 @@ typedef struct RTVFSOBJOPS
      * @returns IPRT status code.
      * @param   pvThis      The implementation specific file data.
      */
-    DECLCALLBACKMEMBER(int, pfnClose)(void *pvThis);
+    DECLCALLBACKMEMBER(int, pfnClose,(void *pvThis));
 
     /**
      * Get information about the file.
      *
      * @returns IPRT status code. See RTVfsObjQueryInfo.
      * @retval  VERR_WRONG_TYPE if file system or file system stream.
+     *
      * @param   pvThis      The implementation specific file data.
      * @param   pObjInfo    Where to return the object info on success.
      * @param   enmAddAttr  Which set of additional attributes to request.
      * @sa      RTVfsObjQueryInfo, RTFileQueryInfo, RTPathQueryInfo
      */
-    DECLCALLBACKMEMBER(int, pfnQueryInfo)(void *pvThis, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAddAttr);
+    DECLCALLBACKMEMBER(int, pfnQueryInfo,(void *pvThis, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAddAttr));
+
+    /**
+     * Query arbritray information about the file, volume, or whatever.
+     *
+     * @returns IPRT status code.
+     * @retval  VERR_BUFFER_OVERFLOW sets pcbRet.
+     *
+     * @param   pvThis      The implementation specific file data.
+     * @param   enmInfo     The information being queried.
+     * @param   pvInfo      Where to return the info.
+     * @param   cbInfo      The size of the @a pvInfo buffer.
+     * @param   pcbRet      The size of the returned data.  In case of
+     *                      VERR_BUFFER_OVERFLOW this will be set to the required
+     *                      buffer size.
+     */
+    DECLCALLBACKMEMBER(int, pfnQueryInfoEx,(void *pvThis, RTVFSQIEX enmInfo, void *pvInfo, size_t cbInfo, size_t *pcbRet));
 
     /** Marks the end of the structure (RTVFSOBJOPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -200,7 +251,7 @@ typedef struct RTVFSOBJOPS
 typedef RTVFSOBJOPS const *PCRTVFSOBJOPS;
 
 /** The RTVFSOBJOPS structure version. */
-#define RTVFSOBJOPS_VERSION         RT_MAKE_U32_FROM_U8(0xff,0x1f,1,0)
+#define RTVFSOBJOPS_VERSION         RT_MAKE_U32_FROM_U8(0xff,0x1f,2,0)
 
 
 /**
@@ -222,7 +273,7 @@ typedef struct RTVFSOPS
      * @param   pvThis      The implementation specific data.
      * @param   phVfsDir    Where to return the handle to the root directory.
      */
-    DECLCALLBACKMEMBER(int, pfnOpenRoot)(void *pvThis, PRTVFSDIR phVfsDir);
+    DECLCALLBACKMEMBER(int, pfnOpenRoot,(void *pvThis, PRTVFSDIR phVfsDir));
 
     /**
      * Query the status of the given storage range (optional).
@@ -236,7 +287,7 @@ typedef struct RTVFSOPS
      * @param   cb          Number of bytes to check.
      * @param   pfUsed      Where to store whether the given range is in use.
      */
-    DECLCALLBACKMEMBER(int, pfnQueryRangeState)(void *pvThis, uint64_t off, size_t cb, bool *pfUsed);
+    DECLCALLBACKMEMBER(int, pfnQueryRangeState,(void *pvThis, uint64_t off, size_t cb, bool *pfUsed));
 
     /** @todo There will be more methods here to optimize opening and
      *        querying. */
@@ -253,7 +304,7 @@ typedef struct RTVFSOPS
      * @param   phVfs???    Return handle to what we've traversed.
      * @param   p???        Return other stuff...
      */
-    DECLCALLBACKMEMBER(int, pfnTraverse)(void *pvThis, const char *pszPath, size_t *poffPath, PRTVFS??? phVfs?, ???* p???);
+    DECLCALLBACKMEMBER(int, pfnTraverse,(void *pvThis, const char *pszPath, size_t *poffPath, PRTVFS??? phVfs?, ???* p???));
 #endif
 
     /** @todo need rename API */
@@ -311,6 +362,17 @@ RTDECL(int) RTVfsNewBaseObj(PCRTVFSOBJOPS pObjOps, size_t cbInstance, RTVFS hVfs
                             PRTVFSOBJ phVfsObj, void **ppvInstance);
 
 
+/**
+ * Gets the private data of a base object.
+ *
+ * @returns Pointer to the private data.  NULL if the handle is invalid in some
+ *          way.
+ * @param   hVfsObj             The I/O base object handle.
+ * @param   pObjOps             The base object operations.  This servers as a
+ *                              sort of password.
+ */
+RTDECL(void *) RTVfsObjToPrivate(RTVFSOBJ hVfsObj, PCRTVFSOBJOPS pObjOps);
+
 /**
  * Additional operations for setting object attributes.
  */
@@ -332,7 +394,7 @@ typedef struct RTVFSOBJSETOPS
      * @note    Optional, failing with VERR_WRITE_PROTECT if NULL.
      * @sa      RTFileSetMode
      */
-    DECLCALLBACKMEMBER(int, pfnSetMode)(void *pvThis, RTFMODE fMode, RTFMODE fMask);
+    DECLCALLBACKMEMBER(int, pfnSetMode,(void *pvThis, RTFMODE fMode, RTFMODE fMask));
 
     /**
      * Set the timestamps associated with the object.
@@ -352,8 +414,8 @@ typedef struct RTVFSOBJSETOPS
      * @note    Optional, failing with VERR_WRITE_PROTECT if NULL.
      * @sa      RTFileSetTimes
      */
-    DECLCALLBACKMEMBER(int, pfnSetTimes)(void *pvThis, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
-                                         PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime);
+    DECLCALLBACKMEMBER(int, pfnSetTimes,(void *pvThis, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
+                                         PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime));
 
     /**
      * Set the unix style owner and group.
@@ -367,7 +429,7 @@ typedef struct RTVFSOBJSETOPS
      * @note    Optional, failing with VERR_WRITE_PROTECT if NULL.
      * @sa      RTFileSetOwner
      */
-    DECLCALLBACKMEMBER(int, pfnSetOwner)(void *pvThis, RTUID uid, RTGID gid);
+    DECLCALLBACKMEMBER(int, pfnSetOwner,(void *pvThis, RTUID uid, RTGID gid));
 
     /** Marks the end of the structure (RTVFSOBJSETOPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -411,7 +473,7 @@ typedef struct RTVFSFSSTREAMOPS
      *
      * @note    Setting this member to NULL is okay for write-only streams.
      */
-    DECLCALLBACKMEMBER(int, pfnNext)(void *pvThis, char **ppszName, RTVFSOBJTYPE *penmType, PRTVFSOBJ phVfsObj);
+    DECLCALLBACKMEMBER(int, pfnNext,(void *pvThis, char **ppszName, RTVFSOBJTYPE *penmType, PRTVFSOBJ phVfsObj));
 
     /**
      * Adds another object into the stream.
@@ -427,7 +489,7 @@ typedef struct RTVFSFSSTREAMOPS
      *
      * @note    Setting this member to NULL is okay for read-only streams.
      */
-    DECLCALLBACKMEMBER(int, pfnAdd)(void *pvThis, const char *pszPath, RTVFSOBJ hVfsObj, uint32_t fFlags);
+    DECLCALLBACKMEMBER(int, pfnAdd,(void *pvThis, const char *pszPath, RTVFSOBJ hVfsObj, uint32_t fFlags));
 
     /**
      * Pushes an byte stream onto the stream (optional).
@@ -455,8 +517,8 @@ typedef struct RTVFSFSSTREAMOPS
      *                      to.  If the FS stream is backed by a file, the returned
      *                      handle can be cast to a file if necessary.
      */
-    DECLCALLBACKMEMBER(int, pfnPushFile)(void *pvThis, const char *pszPath, uint64_t cbFile,
-                                         PCRTFSOBJINFO paObjInfo, uint32_t cObjInfo, uint32_t fFlags, PRTVFSIOSTREAM phVfsIos);
+    DECLCALLBACKMEMBER(int, pfnPushFile,(void *pvThis, const char *pszPath, uint64_t cbFile,
+                                         PCRTFSOBJINFO paObjInfo, uint32_t cObjInfo, uint32_t fFlags, PRTVFSIOSTREAM phVfsIos));
 
     /**
      * Marks the end of the stream.
@@ -469,7 +531,7 @@ typedef struct RTVFSFSSTREAMOPS
      *
      * @note    Setting this member to NULL is okay for read-only streams.
      */
-    DECLCALLBACKMEMBER(int, pfnEnd)(void *pvThis);
+    DECLCALLBACKMEMBER(int, pfnEnd,(void *pvThis));
 
     /** Marks the end of the structure (RTVFSFSSTREAMOPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -492,12 +554,12 @@ typedef RTVFSFSSTREAMOPS const *PCRTVFSFSSTREAMOPS;
  * @param   hLock               Handle to a custom lock to be used with the new
  *                              object.  The reference is consumed.  NIL and
  *                              special lock handles are fine.
- * @param   fReadOnly           Set if read-only, clear if write-only.
+ * @param   fAccess             RTFILE_O_READ and/or RTFILE_O_WRITE.
  * @param   phVfsFss            Where to return the new handle.
  * @param   ppvInstance         Where to return the pointer to the instance data
  *                              (size is @a cbInstance).
  */
-RTDECL(int) RTVfsNewFsStream(PCRTVFSFSSTREAMOPS pFsStreamOps, size_t cbInstance, RTVFS hVfs, RTVFSLOCK hLock, bool fReadOnly,
+RTDECL(int) RTVfsNewFsStream(PCRTVFSFSSTREAMOPS pFsStreamOps, size_t cbInstance, RTVFS hVfs, RTVFSLOCK hLock, uint32_t fAccess,
                              PRTVFSFSSTREAM phVfsFss, void **ppvInstance);
 
 /**
@@ -565,8 +627,8 @@ typedef struct RTVFSDIROPS
      * @param   phVfsObj    Where to return the handle to the opened object.
      * @sa      RTFileOpen, RTDirOpen
      */
-    DECLCALLBACKMEMBER(int, pfnOpen)(void *pvThis, const char *pszEntry, uint64_t fOpenFile,
-                                     uint32_t fObjFlags, PRTVFSOBJ phVfsObj);
+    DECLCALLBACKMEMBER(int, pfnOpen,(void *pvThis, const char *pszEntry, uint64_t fOpenFile,
+                                     uint32_t fObjFlags, PRTVFSOBJ phVfsObj));
 
     /**
      * Optional method for symbolic link handling in the vfsstddir.cpp.
@@ -584,7 +646,7 @@ typedef struct RTVFSDIROPS
      * @param   phVfsDir    Where to return the handle to the specified root
      *                      directory (or may current dir on a drive letter).
      */
-    DECLCALLBACKMEMBER(int, pfnFollowAbsoluteSymlink)(void *pvThis, const char *pszRoot, PRTVFSDIR phVfsDir);
+    DECLCALLBACKMEMBER(int, pfnFollowAbsoluteSymlink,(void *pvThis, const char *pszRoot, PRTVFSDIR phVfsDir));
 
     /**
      * Open or create a file.
@@ -597,7 +659,7 @@ typedef struct RTVFSDIROPS
      * @note    Optional.  RTVFSDIROPS::pfnOpenObj will be used if NULL.
      * @sa      RTFileOpen.
      */
-    DECLCALLBACKMEMBER(int, pfnOpenFile)(void *pvThis, const char *pszFilename, uint64_t fOpen, PRTVFSFILE phVfsFile);
+    DECLCALLBACKMEMBER(int, pfnOpenFile,(void *pvThis, const char *pszFilename, uint64_t fOpen, PRTVFSFILE phVfsFile));
 
     /**
      * Open an existing subdirectory.
@@ -614,7 +676,7 @@ typedef struct RTVFSDIROPS
      * @note    Optional.  RTVFSDIROPS::pfnOpenObj will be used if NULL.
      * @sa      RTDirOpen.
      */
-    DECLCALLBACKMEMBER(int, pfnOpenDir)(void *pvThis, const char *pszSubDir, uint32_t fFlags, PRTVFSDIR phVfsDir);
+    DECLCALLBACKMEMBER(int, pfnOpenDir,(void *pvThis, const char *pszSubDir, uint32_t fFlags, PRTVFSDIR phVfsDir));
 
     /**
      * Creates a new subdirectory.
@@ -628,7 +690,7 @@ typedef struct RTVFSDIROPS
      * @note    Optional.  RTVFSDIROPS::pfnOpenObj will be used if NULL.
      * @sa      RTDirCreate.
      */
-    DECLCALLBACKMEMBER(int, pfnCreateDir)(void *pvThis, const char *pszSubDir, RTFMODE fMode, PRTVFSDIR phVfsDir);
+    DECLCALLBACKMEMBER(int, pfnCreateDir,(void *pvThis, const char *pszSubDir, RTFMODE fMode, PRTVFSDIR phVfsDir));
 
     /**
      * Opens an existing symbolic link.
@@ -641,7 +703,7 @@ typedef struct RTVFSDIROPS
      * @note    Optional.  RTVFSDIROPS::pfnOpenObj will be used if NULL.
      * @sa      RTSymlinkCreate.
      */
-    DECLCALLBACKMEMBER(int, pfnOpenSymlink)(void *pvThis, const char *pszSymlink, PRTVFSSYMLINK phVfsSymlink);
+    DECLCALLBACKMEMBER(int, pfnOpenSymlink,(void *pvThis, const char *pszSymlink, PRTVFSSYMLINK phVfsSymlink));
 
     /**
      * Creates a new symbolic link.
@@ -655,8 +717,8 @@ typedef struct RTVFSDIROPS
      *                      newly create symbolic link.
      * @sa      RTSymlinkCreate.
      */
-    DECLCALLBACKMEMBER(int, pfnCreateSymlink)(void *pvThis, const char *pszSymlink, const char *pszTarget,
-                                              RTSYMLINKTYPE enmType, PRTVFSSYMLINK phVfsSymlink);
+    DECLCALLBACKMEMBER(int, pfnCreateSymlink,(void *pvThis, const char *pszSymlink, const char *pszTarget,
+                                              RTSYMLINKTYPE enmType, PRTVFSSYMLINK phVfsSymlink));
 
     /**
      * Query information about an entry.
@@ -670,8 +732,8 @@ typedef struct RTVFSDIROPS
      *          will be used if NULL.
      * @sa      RTPathQueryInfo, RTVFSOBJOPS::pfnQueryInfo
      */
-    DECLCALLBACKMEMBER(int, pfnQueryEntryInfo)(void *pvThis, const char *pszEntry,
-                                               PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAddAttr);
+    DECLCALLBACKMEMBER(int, pfnQueryEntryInfo,(void *pvThis, const char *pszEntry,
+                                               PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAddAttr));
 
     /**
      * Removes a directory entry.
@@ -684,7 +746,7 @@ typedef struct RTVFSDIROPS
      *                      (RTFS_TYPE_XXX).
      * @sa      RTFileRemove, RTDirRemove, RTSymlinkRemove.
      */
-    DECLCALLBACKMEMBER(int, pfnUnlinkEntry)(void *pvThis, const char *pszEntry, RTFMODE fType);
+    DECLCALLBACKMEMBER(int, pfnUnlinkEntry,(void *pvThis, const char *pszEntry, RTFMODE fType));
 
     /**
      * Renames a directory entry.
@@ -701,7 +763,7 @@ typedef struct RTVFSDIROPS
      * @todo    This API is not flexible enough, must be able to rename between
      *          directories within a file system.
      */
-    DECLCALLBACKMEMBER(int, pfnRenameEntry)(void *pvThis, const char *pszEntry, RTFMODE fType, const char *pszNewName);
+    DECLCALLBACKMEMBER(int, pfnRenameEntry,(void *pvThis, const char *pszEntry, RTFMODE fType, const char *pszNewName));
 
     /**
      * Rewind the directory stream so that the next read returns the first
@@ -710,7 +772,7 @@ typedef struct RTVFSDIROPS
      * @returns IPRT status code.
      * @param   pvThis      The implementation specific directory data.
      */
-    DECLCALLBACKMEMBER(int, pfnRewindDir)(void *pvThis);
+    DECLCALLBACKMEMBER(int, pfnRewindDir,(void *pvThis));
 
     /**
      * Rewind the directory stream so that the next read returns the first
@@ -723,7 +785,7 @@ typedef struct RTVFSDIROPS
      * @param   enmAddAttr  Which set of additional attributes to request.
      * @sa      RTDirReadEx
      */
-    DECLCALLBACKMEMBER(int, pfnReadDir)(void *pvThis, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry, RTFSOBJATTRADD enmAddAttr);
+    DECLCALLBACKMEMBER(int, pfnReadDir,(void *pvThis, PRTDIRENTRYEX pDirEntry, size_t *pcbDirEntry, RTFSOBJATTRADD enmAddAttr));
 
     /** Marks the end of the structure (RTVFSDIROPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -798,7 +860,7 @@ typedef struct RTVFSSYMLINKOPS
      * @param   cbTarget    The size of the target buffer.
      * @sa      RTSymlinkRead
      */
-    DECLCALLBACKMEMBER(int, pfnRead)(void *pvThis, char *pszTarget, size_t cbTarget);
+    DECLCALLBACKMEMBER(int, pfnRead,(void *pvThis, char *pszTarget, size_t cbTarget));
 
     /** Marks the end of the structure (RTVFSSYMLINKOPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -828,6 +890,17 @@ RTDECL(int) RTVfsNewSymlink(PCRTVFSSYMLINKOPS pSymlinkOps, size_t cbInstance, RT
                             PRTVFSSYMLINK phVfsSym, void **ppvInstance);
 
 
+/**
+ * Gets the private data of a symbolic link.
+ *
+ * @returns Pointer to the private data.  NULL if the handle is invalid in some
+ *          way.
+ * @param   hVfsSym             The symlink handle.
+ * @param   pSymlinkOps         The symlink operations.  This servers as a sort
+ *                              of password.
+ */
+RTDECL(void *) RTVfsSymlinkToPrivate(RTVFSSYMLINK hVfsSym, PCRTVFSSYMLINKOPS pSymlinkOps);
+
 /**
  * The basis for all I/O objects (files, pipes, sockets, devices, ++).
  *
@@ -858,7 +931,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @sa      RTVfsIoStrmRead, RTVfsIoStrmSgRead, RTVfsFileRead,
      *          RTVfsFileReadAt, RTFileRead, RTFileReadAt.
      */
-    DECLCALLBACKMEMBER(int, pfnRead)(void *pvThis, RTFOFF off, PCRTSGBUF pSgBuf, bool fBlocking, size_t *pcbRead);
+    DECLCALLBACKMEMBER(int, pfnRead,(void *pvThis, RTFOFF off, PCRTSGBUF pSgBuf, bool fBlocking, size_t *pcbRead));
 
     /**
      * Writes to the file/stream.
@@ -877,7 +950,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @note    Optional, failing with VERR_WRITE_PROTECT if NULL.
      * @sa      RTFileWrite, RTFileWriteAt.
      */
-    DECLCALLBACKMEMBER(int, pfnWrite)(void *pvThis, RTFOFF off, PCRTSGBUF pSgBuf, bool fBlocking, size_t *pcbWritten);
+    DECLCALLBACKMEMBER(int, pfnWrite,(void *pvThis, RTFOFF off, PCRTSGBUF pSgBuf, bool fBlocking, size_t *pcbWritten));
 
     /**
      * Flushes any pending data writes to the stream.
@@ -886,7 +959,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @param   pvThis      The implementation specific file data.
      * @sa      RTFileFlush.
      */
-    DECLCALLBACKMEMBER(int, pfnFlush)(void *pvThis);
+    DECLCALLBACKMEMBER(int, pfnFlush,(void *pvThis));
 
     /**
      * Poll for events.
@@ -903,8 +976,8 @@ typedef struct RTVFSIOSTREAMOPS
      *          events, waiting for errors works like sleep.
      * @sa      RTPollSetAdd, RTPoll, RTPollNoResume.
      */
-    DECLCALLBACKMEMBER(int, pfnPollOne)(void *pvThis, uint32_t fEvents, RTMSINTERVAL cMillies, bool fIntr,
-                                        uint32_t *pfRetEvents);
+    DECLCALLBACKMEMBER(int, pfnPollOne,(void *pvThis, uint32_t fEvents, RTMSINTERVAL cMillies, bool fIntr,
+                                        uint32_t *pfRetEvents));
 
     /**
      * Tells the current file/stream position.
@@ -914,7 +987,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @param   poffActual  Where to return the actual offset.
      * @sa      RTFileTell
      */
-    DECLCALLBACKMEMBER(int, pfnTell)(void *pvThis, PRTFOFF poffActual);
+    DECLCALLBACKMEMBER(int, pfnTell,(void *pvThis, PRTFOFF poffActual));
 
     /**
      * Skips @a cb ahead in the stream.
@@ -924,7 +997,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @param   cb          The number bytes to skip.
      * @remarks This is optional and can be NULL.
      */
-    DECLCALLBACKMEMBER(int, pfnSkip)(void *pvThis, RTFOFF cb);
+    DECLCALLBACKMEMBER(int, pfnSkip,(void *pvThis, RTFOFF cb));
 
     /**
      * Fills the stream with @a cb zeros.
@@ -934,7 +1007,7 @@ typedef struct RTVFSIOSTREAMOPS
      * @param   cb          The number of zero bytes to insert.
      * @remarks This is optional and can be NULL.
      */
-    DECLCALLBACKMEMBER(int, pfnZeroFill)(void *pvThis, RTFOFF cb);
+    DECLCALLBACKMEMBER(int, pfnZeroFill,(void *pvThis, RTFOFF cb));
 
     /** Marks the end of the structure (RTVFSIOSTREAMOPS_VERSION). */
     uintptr_t               uEndMarker;
@@ -1013,7 +1086,7 @@ typedef struct RTVFSFILEOPS
      * @param   poffActual  Where to return the actual offset.
      * @sa      RTFileSeek
      */
-    DECLCALLBACKMEMBER(int, pfnSeek)(void *pvThis, RTFOFF offSeek, unsigned uMethod, PRTFOFF poffActual);
+    DECLCALLBACKMEMBER(int, pfnSeek,(void *pvThis, RTFOFF offSeek, unsigned uMethod, PRTFOFF poffActual));
 
     /**
      * Get the current file size.
@@ -1023,7 +1096,7 @@ typedef struct RTVFSFILEOPS
      * @param   pcbFile     Where to store the current file size.
      * @sa      RTFileQuerySize
      */
-    DECLCALLBACKMEMBER(int, pfnQuerySize)(void *pvThis, uint64_t *pcbFile);
+    DECLCALLBACKMEMBER(int, pfnQuerySize,(void *pvThis, uint64_t *pcbFile));
 
     /**
      * Change the file size.
@@ -1043,7 +1116,7 @@ typedef struct RTVFSFILEOPS
      * @note    Optional.  If NULL, VERR_WRITE_PROTECT will be returned.
      * @sa      RTFileSetSize, RTFileSetAllocationSize
      */
-    DECLCALLBACKMEMBER(int, pfnSetSize)(void *pvThis, uint64_t cbFile, uint32_t fFlags);
+    DECLCALLBACKMEMBER(int, pfnSetSize,(void *pvThis, uint64_t cbFile, uint32_t fFlags));
 
     /**
      * Determine the maximum file size.
@@ -1057,7 +1130,7 @@ typedef struct RTVFSFILEOPS
      * @note    Optional.  If NULL, VERR_NOT_IMPLEMENTED will be returned.
      * @sa      RTFileQueryMaxSizeEx
      */
-    DECLCALLBACKMEMBER(int, pfnQueryMaxSize)(void *pvThis, uint64_t *pcbMax);
+    DECLCALLBACKMEMBER(int, pfnQueryMaxSize,(void *pvThis, uint64_t *pcbMax));
 
     /** @todo There will be more methods here. */
 
@@ -1314,8 +1387,8 @@ typedef struct RTVFSCHAINELEMENTREG
      * @param   pErrInfo        Where to return additional error information, if
      *                          available.  Optional.
      */
-    DECLCALLBACKMEMBER(int, pfnValidate)(PCRTVFSCHAINELEMENTREG pProviderReg, PRTVFSCHAINSPEC pSpec,
-                                         PRTVFSCHAINELEMSPEC pElement, uint32_t *poffError, PRTERRINFO pErrInfo);
+    DECLCALLBACKMEMBER(int, pfnValidate,(PCRTVFSCHAINELEMENTREG pProviderReg, PRTVFSCHAINSPEC pSpec,
+                                         PRTVFSCHAINELEMSPEC pElement, uint32_t *poffError, PRTERRINFO pErrInfo));
 
     /**
      * Create a VFS object according to the element specification.
@@ -1333,9 +1406,9 @@ typedef struct RTVFSCHAINELEMENTREG
      * @param   pErrInfo        Where to return additional error information, if
      *                          available.  Optional.
      */
-    DECLCALLBACKMEMBER(int, pfnInstantiate)(PCRTVFSCHAINELEMENTREG pProviderReg, PCRTVFSCHAINSPEC pSpec,
+    DECLCALLBACKMEMBER(int, pfnInstantiate,(PCRTVFSCHAINELEMENTREG pProviderReg, PCRTVFSCHAINSPEC pSpec,
                                             PCRTVFSCHAINELEMSPEC pElement, RTVFSOBJ hPrevVfsObj,
-                                            PRTVFSOBJ phVfsObj, uint32_t *poffError, PRTERRINFO pErrInfo);
+                                            PRTVFSOBJ phVfsObj, uint32_t *poffError, PRTERRINFO pErrInfo));
 
     /**
      * Determins whether the element can be reused.
@@ -1354,9 +1427,9 @@ typedef struct RTVFSCHAINELEMENTREG
      * @param   pReuseElement   The chain element specification of the existing
      *                          element that is being considered for reuse.
      */
-    DECLCALLBACKMEMBER(bool, pfnCanReuseElement)(PCRTVFSCHAINELEMENTREG pProviderReg,
+    DECLCALLBACKMEMBER(bool, pfnCanReuseElement,(PCRTVFSCHAINELEMENTREG pProviderReg,
                                                  PCRTVFSCHAINSPEC pSpec, PCRTVFSCHAINELEMSPEC pElement,
-                                                 PCRTVFSCHAINSPEC pReuseSpec, PCRTVFSCHAINELEMSPEC pReuseElement);
+                                                 PCRTVFSCHAINSPEC pReuseSpec, PCRTVFSCHAINELEMSPEC pReuseElement));
 
     /** End marker (RTVFSCHAINELEMENTREG_VERSION). */
     uintptr_t               uEndMarker;
