@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -166,7 +176,8 @@ static void vgsvcPageSharingRegisterModule(PVGSVCPGSHKNOWNMOD pModule, bool fVal
 /** @todo r=bird: Mixing ANSI and TCHAR crap again.  This code is a mess.  We
  * always use the wide version of the API and convert to UTF-8/whatever. */
 
-        sprintf(szFileVersionLocation, TEXT("\\StringFileInfo\\%04x%04x\\FileVersion"), lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);
+        RTStrPrintf(szFileVersionLocation, sizeof(szFileVersionLocation),
+                    "\\StringFileInfo\\%04x%04x\\FileVersion", lpTranslate[i].wLanguage, lpTranslate[i].wCodePage);
         fRet = VerQueryValue(pVersionInfo, szFileVersionLocation, (LPVOID *)&pszFileVersion, &cbFileVersion);
         if (fRet)
         {
@@ -443,7 +454,9 @@ static void vgsvcPageSharingInspectGuest(void)
                     if (!pModule)
                         break;
 
-/** @todo FullPathName not an UTF-8 string is! An ANSI string it is. */
+/** @todo FullPathName not an UTF-8 string is! An ANSI string it is
+ * according to the SYSTEM locale.  Best use RtlAnsiStringToUnicodeString to
+ * convert to UTF-16. */
                     strcpy(pModule->Info.szModule,
                            (const char *)&pSystemModules->Modules[i].FullPathName[pSystemModules->Modules[i].OffsetToFileName]);
                     GetSystemDirectoryA(szFullFilePath, sizeof(szFullFilePath));
@@ -453,14 +466,14 @@ static void vgsvcPageSharingInspectGuest(void)
                     if (!lpPath)
                     {
                         /* Seen just file names in XP; try to locate the file in the system32 and system32\drivers directories. */
-                        strcat(szFullFilePath, "\\");
-                        strcat(szFullFilePath, (const char *)pSystemModules->Modules[i].FullPathName);
+                        RTStrCat(szFullFilePath, sizeof(szFullFilePath), "\\");
+                        RTStrCat(szFullFilePath, sizeof(szFullFilePath), (const char *)pSystemModules->Modules[i].FullPathName);
                         VGSvcVerbose(3, "Unexpected kernel module name try %s\n", szFullFilePath);
                         if (RTFileExists(szFullFilePath) == false)
                         {
                             GetSystemDirectoryA(szFullFilePath, sizeof(szFullFilePath));
-                            strcat(szFullFilePath, "\\drivers\\");
-                            strcat(szFullFilePath, (const char *)pSystemModules->Modules[i].FullPathName);
+                            RTStrCat(szFullFilePath, sizeof(szFullFilePath), "\\drivers\\");
+                            RTStrCat(szFullFilePath, sizeof(szFullFilePath), (const char *)pSystemModules->Modules[i].FullPathName);
                             VGSvcVerbose(3, "Unexpected kernel module name try %s\n", szFullFilePath);
                             if (RTFileExists(szFullFilePath) == false)
                             {
@@ -480,7 +493,7 @@ static void vgsvcPageSharingInspectGuest(void)
                             continue;
                         }
 
-                        strcat(szFullFilePath, lpPath);
+                        RTStrCat(szFullFilePath, sizeof(szFullFilePath), lpPath);
                     }
 
                     strcpy(pModule->Info.szExePath, szFullFilePath);
@@ -498,8 +511,8 @@ static void vgsvcPageSharingInspectGuest(void)
 
                     pRec = &pModule->Core;
                 }
-                bool ret = RTAvlPVInsert(&pNewTree, pRec);
-                Assert(ret); NOREF(ret);
+                bool fRet = RTAvlPVInsert(&pNewTree, pRec);
+                Assert(fRet); NOREF(fRet);
             }
         }
 skipkernelmodules:
