@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2008-2017 Oracle Corporation
+ * Copyright (C) 2008-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -21,14 +31,18 @@
 *********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_DBGF /// @todo add new log group.
 #include "DBGPlugIns.h"
-#include <VBox/vmm/dbgf.h>
-#include <iprt/string.h>
+#include <VBox/vmm/vmmr3vtable.h>
+#include <iprt/err.h>
 #include <iprt/mem.h>
 #include <iprt/stream.h>
+#include <iprt/string.h>
 #include <iprt/uuid.h>
 #include <iprt/ctype.h>
 #include <iprt/formats/mach-o.h>
 
+#undef LogRel2
+#define LogRel2 LogRel
+
 
 /*********************************************************************************************************************************
 *   Structures and Typedefs                                                                                                      *
@@ -127,14 +141,15 @@ typedef DBGDIGGERDARWIN *PDBGDIGGERDARWIN;
 /*********************************************************************************************************************************
 *   Internal Functions                                                                                                           *
 *********************************************************************************************************************************/
-static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData);
+static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData);
+
 
 
 /**
  * @interface_method_impl{DBGFOSIDMESG,pfnQueryKernelLog}
  */
-static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis, PUVM pUVM, uint32_t fFlags, uint32_t cMessages,
-                                                             char *pszBuf, size_t cbBuf, size_t *pcbActual)
+static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis, PUVM pUVM, PCVMMR3VTABLE pVMM, uint32_t fFlags,
+                                                             uint32_t cMessages, char *pszBuf, size_t cbBuf, size_t *pcbActual)
 {
     RT_NOREF1(fFlags);
     PDBGDIGGERDARWIN pData = RT_FROM_MEMBER(pThis, DBGDIGGERDARWIN, IDmesg);
@@ -145,7 +160,7 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
     /*
      * The 'msgbufp' variable points to a struct msgbuf (bsd/kern/subr_log.c).
      */
-    RTDBGAS  hAs = DBGFR3AsResolveAndRetain(pUVM, DBGF_AS_KERNEL);
+    RTDBGAS  hAs = pVMM->pfnDBGFR3AsResolveAndRetain(pUVM, DBGF_AS_KERNEL);
     RTDBGMOD hMod;
     int rc = RTDbgAsModuleByName(hAs, "mach_kernel", 0, &hMod);
     if (RT_FAILURE(rc))
@@ -158,16 +173,17 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
     rc = RTDbgModSymbolByName(hMod, "_msgbufp", &SymInfo);
     if (RT_SUCCESS(rc))
     {
-        rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, DBGFR3AddrFromFlat(pUVM, &Addr, SymInfo.Value + pData->AddrKernel.FlatPtr),
-                           &GCPtrMsgBufP, pData->f64Bit ? sizeof(uint64_t) : sizeof(uint32_t));
+        rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/,
+                                    pVMM->pfnDBGFR3AddrFromFlat(pUVM, &Addr, SymInfo.Value + pData->AddrKernel.FlatPtr),
+                                    &GCPtrMsgBufP, pData->f64Bit ? sizeof(uint64_t) : sizeof(uint32_t));
         if (RT_FAILURE(rc))
         {
-            Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to read _msgbufp at %RGv: %Rrc\n", Addr.FlatPtr, rc));
+            LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to read _msgbufp at %RGv: %Rrc\n", Addr.FlatPtr, rc));
             return VERR_NOT_FOUND;
         }
         if (!OSX_VALID_ADDRESS(pData->f64Bit, GCPtrMsgBufP))
         {
-            Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid address for _msgbufp: %RGv\n", GCPtrMsgBufP));
+            LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid address for _msgbufp: %RGv\n", GCPtrMsgBufP));
             return VERR_NOT_FOUND;
         }
     }
@@ -176,13 +192,13 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
         rc = RTDbgModSymbolByName(hMod, "_msgbuf", &SymInfo);
         if (RT_FAILURE(rc))
         {
-            Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to find _msgbufp and _msgbuf: %Rrc\n", rc));
+            LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to find _msgbufp and _msgbuf: %Rrc\n", rc));
             return VERR_NOT_FOUND;
         }
         GCPtrMsgBufP = SymInfo.Value + pData->AddrKernel.FlatPtr;
         if (!OSX_VALID_ADDRESS(pData->f64Bit, GCPtrMsgBufP))
         {
-            Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid address for _msgbuf: %RGv\n", GCPtrMsgBufP));
+            LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid address for _msgbuf: %RGv\n", GCPtrMsgBufP));
             return VERR_NOT_FOUND;
         }
     }
@@ -198,11 +214,11 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
         uint32_t msg_bufr;
         uint64_t msg_bufc; /**< Size depends on windows size. */
     } MsgBuf;
-    rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, DBGFR3AddrFromFlat(pUVM, &Addr, GCPtrMsgBufP),
-                       &MsgBuf, sizeof(MsgBuf) - (pData->f64Bit ? 0 : sizeof(uint32_t)) );
+    rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, pVMM->pfnDBGFR3AddrFromFlat(pUVM, &Addr, GCPtrMsgBufP),
+                                &MsgBuf, sizeof(MsgBuf) - (pData->f64Bit ? 0 : sizeof(uint32_t)) );
     if (RT_FAILURE(rc))
     {
-        Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to read msgbuf struct at %RGv: %Rrc\n", Addr.FlatPtr, rc));
+        LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: failed to read msgbuf struct at %RGv: %Rrc\n", Addr.FlatPtr, rc));
         return VERR_NOT_FOUND;
     }
     if (!pData->f64Bit)
@@ -218,8 +234,8 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
         || MsgBuf.msg_bufr > MsgBuf.msg_size
         || !OSX_VALID_ADDRESS(pData->f64Bit, MsgBuf.msg_bufc) )
     {
-        Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid MsgBuf data: magic=%#x size=%#x bufx=%#x bufr=%#x bufc=%RGv\n",
-             MsgBuf.msg_magic, MsgBuf.msg_size, MsgBuf.msg_bufx, MsgBuf.msg_bufr, MsgBuf.msg_bufc));
+        LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: Invalid MsgBuf data: magic=%#x size=%#x bufx=%#x bufr=%#x bufc=%RGv\n",
+                MsgBuf.msg_magic, MsgBuf.msg_size, MsgBuf.msg_bufx, MsgBuf.msg_bufr, MsgBuf.msg_bufc));
         return VERR_INVALID_STATE;
     }
 
@@ -229,11 +245,12 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
     char *pchMsgBuf = (char *)RTMemAlloc(MsgBuf.msg_size);
     if (!pchMsgBuf)
     {
-        Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: Failed to allocate %#x bytes of memory for the log buffer\n",
-             MsgBuf.msg_size));
+        LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: Failed to allocate %#x bytes of memory for the log buffer\n",
+                MsgBuf.msg_size));
         return VERR_INVALID_STATE;
     }
-    rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, DBGFR3AddrFromFlat(pUVM, &Addr, MsgBuf.msg_bufc), pchMsgBuf, MsgBuf.msg_size);
+    rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/,
+                                pVMM->pfnDBGFR3AddrFromFlat(pUVM, &Addr, MsgBuf.msg_bufc), pchMsgBuf, MsgBuf.msg_size);
     if (RT_SUCCESS(rc))
     {
         /*
@@ -296,18 +313,31 @@ static DECLCALLBACK(int) dbgDiggerDarwinIDmsg_QueryKernelLog(PDBGFOSIDMESG pThis
             *pcbActual = offDst;
     }
     else
-        Log(("dbgDiggerDarwinIDmsg_QueryKernelLog: Error reading %#x bytes at %RGv: %Rrc\n", MsgBuf.msg_size, MsgBuf.msg_bufc, rc));
+        LogRel(("dbgDiggerDarwinIDmsg_QueryKernelLog: Error reading %#x bytes at %RGv: %Rrc\n",
+                MsgBuf.msg_size, MsgBuf.msg_bufc, rc));
     RTMemFree(pchMsgBuf);
     return rc;
 }
 
 
+/**
+ * @copydoc DBGFOSREG::pfnStackUnwindAssist
+ */
+static DECLCALLBACK(int) dbgDiggerDarwinStackUnwindAssist(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData, VMCPUID idCpu,
+                                                          PDBGFSTACKFRAME pFrame, PRTDBGUNWINDSTATE pState, PCCPUMCTX pInitialCtx,
+                                                          RTDBGAS hAs, uint64_t *puScratch)
+{
+    RT_NOREF(pUVM, pVMM, pvData, idCpu, pFrame, pState, pInitialCtx, hAs, puScratch);
+    return VINF_SUCCESS;
+}
+
+
 /**
  * @copydoc DBGFOSREG::pfnQueryInterface
  */
-static DECLCALLBACK(void *) dbgDiggerDarwinQueryInterface(PUVM pUVM, void *pvData, DBGFOSINTERFACE enmIf)
+static DECLCALLBACK(void *) dbgDiggerDarwinQueryInterface(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData, DBGFOSINTERFACE enmIf)
 {
-    RT_NOREF1(pUVM);
+    RT_NOREF(pUVM, pVMM);
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
     switch (enmIf)
     {
@@ -323,7 +353,8 @@ static DECLCALLBACK(void *) dbgDiggerDarwinQueryInterface(PUVM pUVM, void *pvDat
 /**
  * @copydoc DBGFOSREG::pfnQueryVersion
  */
-static DECLCALLBACK(int)  dbgDiggerDarwinQueryVersion(PUVM pUVM, void *pvData, char *pszVersion, size_t cchVersion)
+static DECLCALLBACK(int)  dbgDiggerDarwinQueryVersion(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData,
+                                                      char *pszVersion, size_t cchVersion)
 {
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
     Assert(pThis->fValid);
@@ -331,7 +362,7 @@ static DECLCALLBACK(int)  dbgDiggerDarwinQueryVersion(PUVM pUVM, void *pvData, c
     /*
      * It's all in the linux banner.
      */
-    int rc = DBGFR3MemReadString(pUVM, 0, &pThis->AddrKernelVersion, pszVersion, cchVersion);
+    int rc = pVMM->pfnDBGFR3MemReadString(pUVM, 0, &pThis->AddrKernelVersion, pszVersion, cchVersion);
     if (RT_SUCCESS(rc))
     {
         char *pszEnd = RTStrEnd(pszVersion, cchVersion);
@@ -351,9 +382,9 @@ static DECLCALLBACK(int)  dbgDiggerDarwinQueryVersion(PUVM pUVM, void *pvData, c
 /**
  * @copydoc DBGFOSREG::pfnTerm
  */
-static DECLCALLBACK(void)  dbgDiggerDarwinTerm(PUVM pUVM, void *pvData)
+static DECLCALLBACK(void)  dbgDiggerDarwinTerm(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
-    RT_NOREF1(pUVM);
+    RT_NOREF(pUVM, pVMM);
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
 
     pThis->fValid = false;
@@ -363,7 +394,7 @@ static DECLCALLBACK(void)  dbgDiggerDarwinTerm(PUVM pUVM, void *pvData)
 /**
  * @copydoc DBGFOSREG::pfnRefresh
  */
-static DECLCALLBACK(int)  dbgDiggerDarwinRefresh(PUVM pUVM, void *pvData)
+static DECLCALLBACK(int)  dbgDiggerDarwinRefresh(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
     NOREF(pThis);
@@ -372,8 +403,80 @@ static DECLCALLBACK(int)  dbgDiggerDarwinRefresh(PUVM pUVM, void *pvData)
     /*
      * For now we'll flush and reload everything.
      */
-    dbgDiggerDarwinTerm(pUVM, pvData);
-    return dbgDiggerDarwinInit(pUVM, pvData);
+    dbgDiggerDarwinTerm(pUVM, pVMM, pvData);
+    return dbgDiggerDarwinInit(pUVM, pVMM, pvData);
+}
+
+
+/**
+ * Helper function that tries to accertain whether a segment (__LINKEDIT) is
+ * present or not.
+ *
+ * @returns true if present, false if not.
+ * @param   pUVM                The user mode VM structure.
+ * @param   pVMM                The VMM function table.
+ * @param   uSegAddr            The segment addresss.
+ * @param   cbSeg               The segment size.
+ * @param   uMinAddr            Lowest allowed address.
+ * @param   uMaxAddr            Highest allowed address.
+ */
+static bool dbgDiggerDarwinIsSegmentPresent(PUVM pUVM, PCVMMR3VTABLE pVMM, uint64_t uSegAddr, uint64_t cbSeg,
+                                            uint64_t uMinAddr, uint64_t uMaxAddr)
+{
+    /*
+     * Validate the size and address.
+     */
+    if (cbSeg < 32)
+    {
+        LogRel(("OSXDig: __LINKEDIT too small %#RX64\n", cbSeg));
+        return false;
+    }
+    if (cbSeg > uMaxAddr - uMinAddr)
+    {
+        LogRel(("OSXDig: __LINKEDIT too big %#RX64, max %#RX64\n", cbSeg, uMaxAddr - uMinAddr));
+        return false;
+    }
+
+    if (uSegAddr < uMinAddr)
+    {
+        LogRel(("OSXDig: __LINKEDIT too low %#RX64, min %#RX64\n", uSegAddr, uMinAddr));
+        return false;
+    }
+    if (uSegAddr > uMaxAddr)
+    {
+        LogRel(("OSXDig: __LINKEDIT too high %#RX64, max %#RX64\n", uSegAddr, uMaxAddr));
+        return false;
+    }
+    if (uSegAddr + cbSeg > uMaxAddr)
+    {
+        LogRel(("OSXDig: __LINKEDIT ends too high %#RX64 (%#RX64+%#RX64), max %#RX64\n",
+                 uSegAddr + cbSeg, uSegAddr, cbSeg, uMaxAddr));
+        return false;
+    }
+
+    /*
+     * Check that all the pages are present.
+     */
+    cbSeg    += uSegAddr & X86_PAGE_OFFSET_MASK;
+    uSegAddr &= ~(uint64_t)X86_PAGE_OFFSET_MASK;
+    for (;;)
+    {
+        uint8_t     abBuf[8];
+        DBGFADDRESS Addr;
+        int rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, pVMM->pfnDBGFR3AddrFromFlat(pUVM, &Addr, uSegAddr),
+                                        abBuf, sizeof(abBuf));
+        if (RT_FAILURE(rc))
+        {
+            LogRel(("OSXDig: __LINKEDIT read error at %#RX64: %Rrc\n", uSegAddr, rc));
+            return false;
+        }
+
+        /* Advance */
+        if (cbSeg <= X86_PAGE_SIZE)
+            return true;
+        cbSeg    -= X86_PAGE_SIZE;
+        uSegAddr += X86_PAGE_SIZE;
+    }
 }
 
 
@@ -412,7 +515,8 @@ static bool dbgDiggerDarwinIsValidSegOrSectName(const char *pszName, size_t cbNa
 }
 
 
-static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t uModAddr, const char *pszName, bool *pf64Bit)
+static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, PCVMMR3VTABLE pVMM,
+                                    uint64_t uModAddr, const char *pszName, bool *pf64Bit)
 {
     RT_NOREF1(pThis);
     union
@@ -424,7 +528,8 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
 
     /* Read the first page of the image. */
     DBGFADDRESS ModAddr;
-    int rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, DBGFR3AddrFromFlat(pUVM, &ModAddr, uModAddr), uBuf.ab, X86_PAGE_4K_SIZE);
+    int rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/,
+                                    pVMM->pfnDBGFR3AddrFromFlat(pUVM, &ModAddr, uModAddr), uBuf.ab, X86_PAGE_4K_SIZE);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -451,8 +556,8 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
     /* Do we need to read a 2nd page to get all the load commands? If so, do it. */
     if (uBuf.Hdr32.sizeofcmds + (f64Bit ? sizeof(mach_header_64_t) : sizeof(mach_header_32_t)) > X86_PAGE_4K_SIZE)
     {
-        rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, DBGFR3AddrFromFlat(pUVM, &ModAddr, uModAddr + X86_PAGE_4K_SIZE),
-                           &uBuf.ab[X86_PAGE_4K_SIZE], X86_PAGE_4K_SIZE);
+        rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, pVMM->pfnDBGFR3AddrFromFlat(pUVM, &ModAddr, uModAddr + X86_PAGE_4K_SIZE),
+                                    &uBuf.ab[X86_PAGE_4K_SIZE], X86_PAGE_4K_SIZE);
         if (RT_FAILURE(rc))
             return rc;
     }
@@ -460,9 +565,10 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
     /*
      * Process the load commands.
      */
+    RTUUID          Uuid   = RTUUID_INITIALIZE_NULL;
     RTDBGSEGMENT    aSegs[24];
     uint32_t        cSegs  = 0;
-    RTUUID          Uuid   = RTUUID_INITIALIZE_NULL;
+    bool            fHasLinkEdit = false;
     uint32_t        cLeft  = uBuf.Hdr32.ncmds;
     uint32_t        cbLeft = uBuf.Hdr32.sizeofcmds;
     union
@@ -471,9 +577,6 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
         load_command_t const       *pGenric;
         segment_command_32_t const *pSeg32;
         segment_command_64_t const *pSeg64;
-        section_32_t const         *pSect32;
-        section_64_t const         *pSect64;
-        symtab_command_t const     *pSymTab;
         uuid_command_t const       *pUuid;
     } uLCmd;
     uLCmd.pb = &uBuf.ab[f64Bit ? sizeof(mach_header_64_t) : sizeof(mach_header_32_t)];
@@ -491,7 +594,9 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
                     return VERR_BAD_EXE_FORMAT;
                 if (!dbgDiggerDarwinIsValidSegOrSectName(uLCmd.pSeg32->segname, sizeof(uLCmd.pSeg32->segname)))
                     return VERR_INVALID_NAME;
-                if (!strcmp(uLCmd.pSeg32->segname, "__LINKEDIT"))
+                if (   !strcmp(uLCmd.pSeg32->segname, "__LINKEDIT")
+                    && !(fHasLinkEdit = dbgDiggerDarwinIsSegmentPresent(pUVM, pVMM, uLCmd.pSeg32->vmaddr, uLCmd.pSeg32->vmsize,
+                                                                        uModAddr, uModAddr + _64M)))
                     break; /* This usually is discarded or not loaded at all. */
                 if (cSegs >= RT_ELEMENTS(aSegs))
                     return VERR_BUFFER_OVERFLOW;
@@ -510,7 +615,9 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
                     return VERR_BAD_EXE_FORMAT;
                 if (!dbgDiggerDarwinIsValidSegOrSectName(uLCmd.pSeg64->segname, sizeof(uLCmd.pSeg64->segname)))
                     return VERR_INVALID_NAME;
-                if (!strcmp(uLCmd.pSeg64->segname, "__LINKEDIT"))
+                if (   !strcmp(uLCmd.pSeg64->segname, "__LINKEDIT")
+                    && !(fHasLinkEdit = dbgDiggerDarwinIsSegmentPresent(pUVM, pVMM, uLCmd.pSeg64->vmaddr, uLCmd.pSeg64->vmsize,
+                                                                        uModAddr, uModAddr + _128M)))
                     break; /* This usually is discarded or not loaded at all. */
                 if (cSegs >= RT_ELEMENTS(aSegs))
                     return VERR_BUFFER_OVERFLOW;
@@ -545,7 +652,10 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
     }
 
     if (cbLeft != 0)
+    {
+        LogRel(("OSXDig: uModAddr=%#RX64 - %u bytes of command left over!\n", uModAddr, cbLeft));
         return VERR_BAD_EXE_FORMAT;
+    }
 
     /*
      * Some post processing checks.
@@ -555,20 +665,42 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
         if (aSegs[iSeg].Address == uModAddr)
             break;
     if (iSeg >= cSegs)
+    {
+        LogRel2(("OSXDig: uModAddr=%#RX64 was not found among the segments segments\n", uModAddr));
         return VERR_ADDRESS_CONFLICT;
+    }
 
     /*
      * Create a debug module.
      */
     RTDBGMOD hMod;
-    rc = RTDbgModCreateFromMachOImage(&hMod, pszName, NULL, f64Bit ? RTLDRARCH_AMD64 : RTLDRARCH_X86_32, 0 /*cbImage*/,
-                                      cSegs, aSegs, &Uuid, DBGFR3AsGetConfig(pUVM), RTDBGMOD_F_NOT_DEFERRED);
+    rc = RTDbgModCreateFromMachOImage(&hMod, pszName, NULL, f64Bit ? RTLDRARCH_AMD64 : RTLDRARCH_X86_32, NULL /*phLdrModIn*/,
+                                      0 /*cbImage*/, cSegs, aSegs, &Uuid, pVMM->pfnDBGFR3AsGetConfig(pUVM),
+                                      RTDBGMOD_F_NOT_DEFERRED | (fHasLinkEdit ? RTDBGMOD_F_MACHO_LOAD_LINKEDIT : 0));
 
+
+    /*
+     * If module creation failed and we've got a linkedit segment, try open the
+     * image in-memory, because that will at a minimum give us symbol table symbols.
+     */
+    if (RT_FAILURE(rc) && fHasLinkEdit)
+    {
+        DBGFADDRESS DbgfAddr;
+        RTERRINFOSTATIC ErrInfo;
+        rc = pVMM->pfnDBGFR3ModInMem(pUVM, pVMM->pfnDBGFR3AddrFromFlat(pUVM, &DbgfAddr, uModAddr),
+                                     DBGFMODINMEM_F_NO_CONTAINER_FALLBACK,
+                                     pszName, NULL /*pszFilename*/, f64Bit ? RTLDRARCH_AMD64 : RTLDRARCH_X86_32, 0 /*cbImage */,
+                                     &hMod, RTErrInfoInitStatic(&ErrInfo));
+        if (RT_FAILURE(rc))
+            LogRel(("OSXDig: Failed to do an in-memory-opening of '%s' at %#RX64: %Rrc%s%s\n", pszName, uModAddr, rc,
+                    RTErrInfoIsSet(&ErrInfo.Core) ? " - " : "", RTErrInfoIsSet(&ErrInfo.Core) ? ErrInfo.Core.pszMsg : ""));
+    }
+
+    /*
+     * Final fallback is a container module.
+     */
     if (RT_FAILURE(rc))
     {
-        /*
-         * Final fallback is a container module.
-         */
         rc = RTDbgModCreate(&hMod, pszName, 0, 0);
         if (RT_FAILURE(rc))
             return rc;
@@ -603,7 +735,7 @@ static int dbgDiggerDarwinAddModule(PDBGDIGGERDARWIN pThis, PUVM pUVM, uint64_t
     /*
      * Link the module.
      */
-    RTDBGAS hAs = DBGFR3AsResolveAndRetain(pUVM, DBGF_AS_KERNEL);
+    RTDBGAS hAs = pVMM->pfnDBGFR3AsResolveAndRetain(pUVM, DBGF_AS_KERNEL);
     if (hAs != NIL_RTDBGAS)
     {
         //uint64_t uRvaNext = 0; - what was this?
@@ -673,7 +805,7 @@ static bool dbgDiggerDarwinIsValidVersion(const char *pszVersion)
 /**
  * @copydoc DBGFOSREG::pfnInit
  */
-static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
+static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
     Assert(!pThis->fValid);
@@ -682,7 +814,7 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
      * Add the kernel module.
      */
     bool f64Bit;
-    int rc = dbgDiggerDarwinAddModule(pThis, pUVM, pThis->AddrKernel.FlatPtr, "mach_kernel", &f64Bit);
+    int rc = dbgDiggerDarwinAddModule(pThis, pUVM, pVMM, pThis->AddrKernel.FlatPtr, "mach_kernel", &f64Bit);
     if (RT_SUCCESS(rc))
     {
         /*
@@ -694,23 +826,23 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
          *       it's any easier to find without any kernel map than 'kmod'.
          */
         RTDBGSYMBOL SymInfo;
-        rc = DBGFR3AsSymbolByName(pUVM, DBGF_AS_KERNEL, "mach_kernel!kmod", &SymInfo, NULL);
+        rc = pVMM->pfnDBGFR3AsSymbolByName(pUVM, DBGF_AS_KERNEL, "mach_kernel!kmod", &SymInfo, NULL);
         if (RT_FAILURE(rc))
-            rc = DBGFR3AsSymbolByName(pUVM, DBGF_AS_KERNEL, "mach_kernel!_kmod", &SymInfo, NULL);
+            rc = pVMM->pfnDBGFR3AsSymbolByName(pUVM, DBGF_AS_KERNEL, "mach_kernel!_kmod", &SymInfo, NULL);
         if (RT_SUCCESS(rc))
         {
             DBGFADDRESS AddrModInfo;
-            DBGFR3AddrFromFlat(pUVM, &AddrModInfo, SymInfo.Value);
+            pVMM->pfnDBGFR3AddrFromFlat(pUVM, &AddrModInfo, SymInfo.Value);
 
             /* Read the variable. */
             RTUINT64U uKmodValue = { 0 };
             if (f64Bit)
-                rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, &AddrModInfo, &uKmodValue.u, sizeof(uKmodValue.u));
+                rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, &AddrModInfo, &uKmodValue.u, sizeof(uKmodValue.u));
             else
-                rc = DBGFR3MemRead (pUVM, 0 /*idCpu*/, &AddrModInfo, &uKmodValue.s.Lo, sizeof(uKmodValue.s.Lo));
+                rc = pVMM->pfnDBGFR3MemRead (pUVM, 0 /*idCpu*/, &AddrModInfo, &uKmodValue.s.Lo, sizeof(uKmodValue.s.Lo));
             if (RT_SUCCESS(rc))
             {
-                DBGFR3AddrFromFlat(pUVM, &AddrModInfo, uKmodValue.u);
+                pVMM->pfnDBGFR3AddrFromFlat(pUVM, &AddrModInfo, uKmodValue.u);
 
                 /* Walk the list of modules. */
                 uint32_t cIterations = 0;
@@ -719,17 +851,17 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                     /* Some extra loop conditions... */
                     if (!OSX_VALID_ADDRESS(f64Bit, AddrModInfo.FlatPtr))
                     {
-                        Log(("OSXDig: Invalid kmod_info pointer: %RGv\n", AddrModInfo.FlatPtr));
+                        LogRel(("OSXDig: Invalid kmod_info pointer: %RGv\n", AddrModInfo.FlatPtr));
                         break;
                     }
                     if (AddrModInfo.FlatPtr == uKmodValue.u && cIterations != 0)
                     {
-                        Log(("OSXDig: kmod_info list looped back to the start.\n"));
+                        LogRel(("OSXDig: kmod_info list looped back to the start.\n"));
                         break;
                     }
                     if (cIterations++ >= 2048)
                     {
-                        Log(("OSXDig: Too many mod_info loops (%u)\n", cIterations));
+                        LogRel(("OSXDig: Too many mod_info loops (%u)\n", cIterations));
                         break;
                     }
 
@@ -742,11 +874,11 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                         OSX32_kmod_info_t   Info32;
                     } uMod;
                     RT_ZERO(uMod);
-                    rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, &AddrModInfo, &uMod,
-                                       f64Bit ? sizeof(uMod.Info64) : sizeof(uMod.Info32));
+                    rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, &AddrModInfo, &uMod,
+                                                f64Bit ? sizeof(uMod.Info64) : sizeof(uMod.Info32));
                     if (RT_FAILURE(rc))
                     {
-                        Log(("OSXDig: Error reading kmod_info structure at %RGv: %Rrc\n", AddrModInfo.FlatPtr, rc));
+                        LogRel(("OSXDig: Error reading kmod_info structure at %RGv: %Rrc\n", AddrModInfo.FlatPtr, rc));
                         break;
                     }
 
@@ -756,7 +888,7 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                     int32_t iInfoVer = f64Bit ? uMod.Info64.info_version : uMod.Info32.info_version;
                     if (iInfoVer != OSX_KMOD_INFO_VERSION)
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad info_version %d\n", AddrModInfo.FlatPtr, iInfoVer));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad info_version %d\n", AddrModInfo.FlatPtr, iInfoVer));
                         break;
                     }
 
@@ -765,8 +897,8 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                         || !RTStrEnd(pszName, sizeof(uMod.Info64.name))
                         || !dbgDiggerDarwinIsValidName(pszName) )
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad name '%.*s'\n", AddrModInfo.FlatPtr,
-                             sizeof(uMod.Info64.name), pszName));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad name '%.*s'\n", AddrModInfo.FlatPtr,
+                                sizeof(uMod.Info64.name), pszName));
                         break;
                     }
 
@@ -774,7 +906,7 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                     if (   !RTStrEnd(pszVersion, sizeof(uMod.Info64.version))
                         || !dbgDiggerDarwinIsValidVersion(pszVersion) )
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad version '%.*s'\n", AddrModInfo.FlatPtr,
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad version '%.*s'\n", AddrModInfo.FlatPtr,
                              sizeof(uMod.Info64.version), pszVersion));
                         break;
                     }
@@ -782,64 +914,64 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
                     int32_t cRefs = f64Bit ? uMod.Info64.reference_count : uMod.Info32.reference_count;
                     if (cRefs < -1 || cRefs > 16384)
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad reference_count %d\n", AddrModInfo.FlatPtr, cRefs));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad reference_count %d\n", AddrModInfo.FlatPtr, cRefs));
                         break;
                     }
 
                     uint64_t uImageAddr = f64Bit ? uMod.Info64.address : uMod.Info32.address;
                     if (!OSX_VALID_ADDRESS(f64Bit, uImageAddr))
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad address %#llx\n", AddrModInfo.FlatPtr, uImageAddr));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad address %#llx\n", AddrModInfo.FlatPtr, uImageAddr));
                         break;
                     }
 
                     uint64_t cbImage = f64Bit ? uMod.Info64.size : uMod.Info32.size;
                     if (cbImage > 64U*_1M)
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad size %#llx\n", AddrModInfo.FlatPtr, cbImage));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad size %#llx\n", AddrModInfo.FlatPtr, cbImage));
                         break;
                     }
 
                     uint64_t cbHdr = f64Bit ? uMod.Info64.hdr_size : uMod.Info32.hdr_size;
                     if (cbHdr > 16U*_1M)
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad hdr_size %#llx\n", AddrModInfo.FlatPtr, cbHdr));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad hdr_size %#llx\n", AddrModInfo.FlatPtr, cbHdr));
                         break;
                     }
 
                     uint64_t uStartAddr = f64Bit ? uMod.Info64.start : uMod.Info32.start;
                     if (!uStartAddr && !OSX_VALID_ADDRESS(f64Bit, uStartAddr))
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad start function %#llx\n", AddrModInfo.FlatPtr, uStartAddr));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad start function %#llx\n", AddrModInfo.FlatPtr, uStartAddr));
                         break;
                     }
 
                     uint64_t uStopAddr = f64Bit ? uMod.Info64.stop : uMod.Info32.stop;
                     if (!uStopAddr && !OSX_VALID_ADDRESS(f64Bit, uStopAddr))
                     {
-                        Log(("OSXDig: kmod_info @%RGv: Bad stop function %#llx\n", AddrModInfo.FlatPtr, uStopAddr));
+                        LogRel(("OSXDig: kmod_info @%RGv: Bad stop function %#llx\n", AddrModInfo.FlatPtr, uStopAddr));
                         break;
                     }
 
                     /*
                      * Try add the module.
                      */
-                    Log(("OSXDig: kmod_info @%RGv: '%s' ver '%s', image @%#llx LB %#llx cbHdr=%#llx\n", AddrModInfo.FlatPtr,
-                         pszName, pszVersion, uImageAddr, cbImage, cbHdr));
-                    rc = dbgDiggerDarwinAddModule(pThis, pUVM, uImageAddr, pszName, NULL);
+                    LogRel(("OSXDig: kmod_info @%RGv: '%s' ver '%s', image @%#llx LB %#llx cbHdr=%#llx\n", AddrModInfo.FlatPtr,
+                            pszName, pszVersion, uImageAddr, cbImage, cbHdr));
+                    rc = dbgDiggerDarwinAddModule(pThis, pUVM, pVMM, uImageAddr, pszName, NULL);
 
 
                     /*
                      * Advance to the next kmod_info entry.
                      */
-                    DBGFR3AddrFromFlat(pUVM, &AddrModInfo, f64Bit ? uMod.Info64.next : uMod.Info32.next);
+                    pVMM->pfnDBGFR3AddrFromFlat(pUVM, &AddrModInfo, f64Bit ? uMod.Info64.next : uMod.Info32.next);
                 }
             }
             else
-                Log(("OSXDig: Error reading the 'kmod' variable: %Rrc\n", rc));
+                LogRel(("OSXDig: Error reading the 'kmod' variable: %Rrc\n", rc));
         }
         else
-            Log(("OSXDig: Failed to locate the 'kmod' variable in mach_kernel.\n"));
+            LogRel(("OSXDig: Failed to locate the 'kmod' variable in mach_kernel.\n"));
 
         pThis->fValid = true;
         return VINF_SUCCESS;
@@ -852,7 +984,7 @@ static DECLCALLBACK(int)  dbgDiggerDarwinInit(PUVM pUVM, void *pvData)
 /**
  * @copydoc DBGFOSREG::pfnProbe
  */
-static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
+static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
 
@@ -871,12 +1003,12 @@ static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
            with cpu in long mode that darwin used for a number of versions. */
         { UINT64_C(0x00001000), UINT64_C(0x0ffff000), }
     };
-    for (unsigned iRange = DBGFR3CpuGetMode(pUVM, 0 /*idCpu*/) != CPUMMODE_LONG;
+    for (unsigned iRange = pVMM->pfnDBGFR3CpuGetMode(pUVM, 0 /*idCpu*/) != CPUMMODE_LONG;
           iRange < RT_ELEMENTS(s_aRanges);
           iRange++)
     {
         DBGFADDRESS     KernelAddr;
-        for (DBGFR3AddrFromFlat(pUVM, &KernelAddr, s_aRanges[iRange].uStart);
+        for (pVMM->pfnDBGFR3AddrFromFlat(pUVM, &KernelAddr, s_aRanges[iRange].uStart);
              KernelAddr.FlatPtr < s_aRanges[iRange].uEnd;
              KernelAddr.FlatPtr += X86_PAGE_4K_SIZE)
         {
@@ -886,11 +1018,11 @@ static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
                 '_','_','K','L','D',  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* section_32_t::segname. */
             };
 
-            int rc = DBGFR3MemScan(pUVM, 0 /*idCpu*/, &KernelAddr, s_aRanges[iRange].uEnd - KernelAddr.FlatPtr,
-                                   1, s_abNeedle, sizeof(s_abNeedle), &KernelAddr);
+            int rc = pVMM->pfnDBGFR3MemScan(pUVM, 0 /*idCpu*/, &KernelAddr, s_aRanges[iRange].uEnd - KernelAddr.FlatPtr,
+                                            1, s_abNeedle, sizeof(s_abNeedle), &KernelAddr);
             if (RT_FAILURE(rc))
                 break;
-            DBGFR3AddrSub(&KernelAddr, KernelAddr.FlatPtr & X86_PAGE_4K_OFFSET_MASK);
+            pVMM->pfnDBGFR3AddrSub(&KernelAddr, KernelAddr.FlatPtr & X86_PAGE_4K_OFFSET_MASK);
 
             /*
              * Read the first page of the image and check the headers.
@@ -901,7 +1033,7 @@ static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
                 mach_header_64_t    Hdr64;
                 mach_header_32_t    Hdr32;
             } uBuf;
-            rc = DBGFR3MemRead(pUVM, 0 /*idCpu*/, &KernelAddr, uBuf.ab, X86_PAGE_4K_SIZE);
+            rc = pVMM->pfnDBGFR3MemRead(pUVM, 0 /*idCpu*/, &KernelAddr, uBuf.ab, X86_PAGE_4K_SIZE);
             if (RT_FAILURE(rc))
                 continue;
             AssertCompileMembersSameSizeAndOffset(mach_header_64_t, magic,   mach_header_32_t, magic);
@@ -932,10 +1064,10 @@ static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
             /*
              * Finally, find the kernel version string.
              */
-            rc = DBGFR3MemScan(pUVM, 0 /*idCpu*/, &KernelAddr, 32*_1M, 1, RT_STR_TUPLE("Darwin Kernel Version"),
-                               &pThis->AddrKernelVersion);
+            rc = pVMM->pfnDBGFR3MemScan(pUVM, 0 /*idCpu*/, &KernelAddr, 32*_1M, 1, RT_STR_TUPLE("Darwin Kernel Version"),
+                                        &pThis->AddrKernelVersion);
             if (RT_FAILURE(rc))
-                DBGFR3AddrFromFlat(pUVM, &pThis->AddrKernelVersion, 0);
+                pVMM->pfnDBGFR3AddrFromFlat(pUVM, &pThis->AddrKernelVersion, 0);
             return true;
         }
     }
@@ -946,19 +1078,18 @@ static DECLCALLBACK(bool)  dbgDiggerDarwinProbe(PUVM pUVM, void *pvData)
 /**
  * @copydoc DBGFOSREG::pfnDestruct
  */
-static DECLCALLBACK(void)  dbgDiggerDarwinDestruct(PUVM pUVM, void *pvData)
+static DECLCALLBACK(void)  dbgDiggerDarwinDestruct(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
-    RT_NOREF2(pUVM, pvData);
-
+    RT_NOREF(pUVM, pVMM, pvData);
 }
 
 
 /**
  * @copydoc DBGFOSREG::pfnConstruct
  */
-static DECLCALLBACK(int)  dbgDiggerDarwinConstruct(PUVM pUVM, void *pvData)
+static DECLCALLBACK(int)  dbgDiggerDarwinConstruct(PUVM pUVM, PCVMMR3VTABLE pVMM, void *pvData)
 {
-    RT_NOREF1(pUVM);
+    RT_NOREF(pUVM, pVMM);
     PDBGDIGGERDARWIN pThis = (PDBGDIGGERDARWIN)pvData;
 
     pThis->IDmesg.u32Magic = DBGFOSIDMESG_MAGIC;
@@ -971,18 +1102,19 @@ static DECLCALLBACK(int)  dbgDiggerDarwinConstruct(PUVM pUVM, void *pvData)
 
 const DBGFOSREG g_DBGDiggerDarwin =
 {
-    /* .u32Magic = */           DBGFOSREG_MAGIC,
-    /* .fFlags = */             0,
-    /* .cbData = */             sizeof(DBGDIGGERDARWIN),
-    /* .szName = */             "Darwin",
-    /* .pfnConstruct = */       dbgDiggerDarwinConstruct,
-    /* .pfnDestruct = */        dbgDiggerDarwinDestruct,
-    /* .pfnProbe = */           dbgDiggerDarwinProbe,
-    /* .pfnInit = */            dbgDiggerDarwinInit,
-    /* .pfnRefresh = */         dbgDiggerDarwinRefresh,
-    /* .pfnTerm = */            dbgDiggerDarwinTerm,
-    /* .pfnQueryVersion = */    dbgDiggerDarwinQueryVersion,
-    /* .pfnQueryInterface = */  dbgDiggerDarwinQueryInterface,
-    /* .u32EndMagic = */        DBGFOSREG_MAGIC
+    /* .u32Magic = */               DBGFOSREG_MAGIC,
+    /* .fFlags = */                 0,
+    /* .cbData = */                 sizeof(DBGDIGGERDARWIN),
+    /* .szName = */                 "Darwin",
+    /* .pfnConstruct = */           dbgDiggerDarwinConstruct,
+    /* .pfnDestruct = */            dbgDiggerDarwinDestruct,
+    /* .pfnProbe = */               dbgDiggerDarwinProbe,
+    /* .pfnInit = */                dbgDiggerDarwinInit,
+    /* .pfnRefresh = */             dbgDiggerDarwinRefresh,
+    /* .pfnTerm = */                dbgDiggerDarwinTerm,
+    /* .pfnQueryVersion = */        dbgDiggerDarwinQueryVersion,
+    /* .pfnQueryInterface = */      dbgDiggerDarwinQueryInterface,
+    /* .pfnStackUnwindAssist = */   dbgDiggerDarwinStackUnwindAssist,
+    /* .u32EndMagic = */            DBGFOSREG_MAGIC
 };
 
