@@ -1,6 +1,6 @@
 /** @file
 
-  Copyright (c) 2017 - 2019, ARM Limited. All rights reserved.
+  Copyright (c) 2017 - 2021, Arm Limited. All rights reserved.<BR>
 
   SPDX-License-Identifier: BSD-2-Clause-Patent
 
@@ -21,37 +21,43 @@
     in the ARM Namespace
 */
 typedef enum ArmObjectID {
-  EArmObjReserved,                    ///<  0 - Reserved
-  EArmObjBootArchInfo,                ///<  1 - Boot Architecture Info
-  EArmObjCpuInfo,                     ///<  2 - CPU Info
-  EArmObjPowerManagementProfileInfo,  ///<  3 - Power Management Profile Info
-  EArmObjGicCInfo,                    ///<  4 - GIC CPU Interface Info
-  EArmObjGicDInfo,                    ///<  5 - GIC Distributor Info
-  EArmObjGicMsiFrameInfo,             ///<  6 - GIC MSI Frame Info
-  EArmObjGicRedistributorInfo,        ///<  7 - GIC Redistributor Info
-  EArmObjGicItsInfo,                  ///<  8 - GIC ITS Info
-  EArmObjSerialConsolePortInfo,       ///<  9 - Serial Console Port Info
-  EArmObjSerialDebugPortInfo,         ///< 10 - Serial Debug Port Info
-  EArmObjGenericTimerInfo,            ///< 11 - Generic Timer Info
-  EArmObjPlatformGTBlockInfo,         ///< 12 - Platform GT Block Info
-  EArmObjGTBlockTimerFrameInfo,       ///< 13 - Generic Timer Block Frame Info
-  EArmObjPlatformGenericWatchdogInfo, ///< 14 - Platform Generic Watchdog
-  EArmObjPciConfigSpaceInfo,          ///< 15 - PCI Configuration Space Info
-  EArmObjHypervisorVendorIdentity,    ///< 16 - Hypervisor Vendor Id
-  EArmObjFixedFeatureFlags,           ///< 17 - Fixed feature flags for FADT
-  EArmObjItsGroup,                    ///< 18 - ITS Group
-  EArmObjNamedComponent,              ///< 19 - Named Component
-  EArmObjRootComplex,                 ///< 20 - Root Complex
-  EArmObjSmmuV1SmmuV2,                ///< 21 - SMMUv1 or SMMUv2
-  EArmObjSmmuV3,                      ///< 22 - SMMUv3
-  EArmObjPmcg,                        ///< 23 - PMCG
-  EArmObjGicItsIdentifierArray,       ///< 24 - GIC ITS Identifier Array
-  EArmObjIdMappingArray,              ///< 25 - ID Mapping Array
-  EArmObjSmmuInterruptArray,          ///< 26 - SMMU Interrupt Array
-  EArmObjProcHierarchyInfo,           ///< 27 - Processor Hierarchy Info
-  EArmObjCacheInfo,                   ///< 28 - Cache Info
-  EArmObjProcNodeIdInfo,              ///< 29 - Processor Hierarchy Node ID Info
-  EArmObjCmRef,                       ///< 30 - CM Object Reference
+  EArmObjReserved,                     ///<  0 - Reserved
+  EArmObjBootArchInfo,                 ///<  1 - Boot Architecture Info
+  EArmObjCpuInfo,                      ///<  2 - CPU Info
+  EArmObjPowerManagementProfileInfo,   ///<  3 - Power Management Profile Info
+  EArmObjGicCInfo,                     ///<  4 - GIC CPU Interface Info
+  EArmObjGicDInfo,                     ///<  5 - GIC Distributor Info
+  EArmObjGicMsiFrameInfo,              ///<  6 - GIC MSI Frame Info
+  EArmObjGicRedistributorInfo,         ///<  7 - GIC Redistributor Info
+  EArmObjGicItsInfo,                   ///<  8 - GIC ITS Info
+  EArmObjSerialConsolePortInfo,        ///<  9 - Serial Console Port Info
+  EArmObjSerialDebugPortInfo,          ///< 10 - Serial Debug Port Info
+  EArmObjGenericTimerInfo,             ///< 11 - Generic Timer Info
+  EArmObjPlatformGTBlockInfo,          ///< 12 - Platform GT Block Info
+  EArmObjGTBlockTimerFrameInfo,        ///< 13 - Generic Timer Block Frame Info
+  EArmObjPlatformGenericWatchdogInfo,  ///< 14 - Platform Generic Watchdog
+  EArmObjPciConfigSpaceInfo,           ///< 15 - PCI Configuration Space Info
+  EArmObjHypervisorVendorIdentity,     ///< 16 - Hypervisor Vendor Id
+  EArmObjFixedFeatureFlags,            ///< 17 - Fixed feature flags for FADT
+  EArmObjItsGroup,                     ///< 18 - ITS Group
+  EArmObjNamedComponent,               ///< 19 - Named Component
+  EArmObjRootComplex,                  ///< 20 - Root Complex
+  EArmObjSmmuV1SmmuV2,                 ///< 21 - SMMUv1 or SMMUv2
+  EArmObjSmmuV3,                       ///< 22 - SMMUv3
+  EArmObjPmcg,                         ///< 23 - PMCG
+  EArmObjGicItsIdentifierArray,        ///< 24 - GIC ITS Identifier Array
+  EArmObjIdMappingArray,               ///< 25 - ID Mapping Array
+  EArmObjSmmuInterruptArray,           ///< 26 - SMMU Interrupt Array
+  EArmObjProcHierarchyInfo,            ///< 27 - Processor Hierarchy Info
+  EArmObjCacheInfo,                    ///< 28 - Cache Info
+  EArmObjProcNodeIdInfo,               ///< 29 - Processor Node ID Info
+  EArmObjCmRef,                        ///< 30 - CM Object Reference
+  EArmObjMemoryAffinityInfo,           ///< 31 - Memory Affinity Info
+  EArmObjDeviceHandleAcpi,             ///< 32 - Device Handle Acpi
+  EArmObjDeviceHandlePci,              ///< 33 - Device Handle Pci
+  EArmObjGenericInitiatorAffinityInfo, ///< 34 - Generic Initiator Affinity
+  EArmObjSerialPortInfo,               ///< 35 - Generic Serial Port Info
+  EArmObjCmn600Info,                   ///< 36 - CMN-600 Info
   EArmObjMax
 } EARM_OBJECT_ID;
 
@@ -64,13 +70,9 @@ typedef struct CmArmBootArchInfo {
   /** This is the ARM_BOOT_ARCH flags field of the FADT Table
       described in the ACPI Table Specification.
   */
-  UINT32  BootArchFlags;
+  UINT16  BootArchFlags;
 } CM_ARM_BOOT_ARCH_INFO;
 
-typedef struct CmArmCpuInfo {
-  // Reserved for use when SMBIOS tables are implemented
-} CM_ARM_CPU_INFO;
-
 /** A structure that describes the
     Power Management Profile Information for the Platform.
 
@@ -166,6 +168,23 @@ typedef struct CmArmGicCInfo {
       generating MADT revision 4 or lower.
   */
   UINT16  SpeOverflowInterrupt;
+
+  /** The proximity domain to which the logical processor belongs.
+      This field is used to populate the GICC affinity structure
+      in the SRAT table.
+  */
+  UINT32  ProximityDomain;
+
+  /** The clock domain to which the logical processor belongs.
+      This field is used to populate the GICC affinity structure
+      in the SRAT table.
+  */
+  UINT32  ClockDomain;
+
+  /** The GICC Affinity flags field as described by the GICC Affinity structure
+      in the SRAT table.
+  */
+  UINT32  AffinityFlags;
 } CM_ARM_GICC_INFO;
 
 /** A structure that describes the
@@ -241,13 +260,20 @@ typedef struct CmArmGicItsInfo {
 
   /// The physical address for the Interrupt Translation Service
   UINT64  PhysicalBaseAddress;
+
+  /** The proximity domain to which the logical processor belongs.
+      This field is used to populate the GIC ITS affinity structure
+      in the SRAT table.
+  */
+  UINT32  ProximityDomain;
 } CM_ARM_GIC_ITS_INFO;
 
 /** A structure that describes the
     Serial Port information for the Platform.
 
     ID: EArmObjSerialConsolePortInfo or
-        EArmObjSerialDebugPortInfo
+        EArmObjSerialDebugPortInfo or
+        EArmObjSerialPortInfo
 */
 typedef struct CmArmSerialPortInfo {
   /// The physical base address for the serial port
@@ -264,6 +290,12 @@ typedef struct CmArmSerialPortInfo {
 
   /// Serial Port subtype
   UINT16  PortSubtype;
+
+  /// The Base address length
+  UINT64  BaseAddressLength;
+
+  /// The access size
+  UINT8   AccessSize;
 } CM_ARM_SERIAL_PORT_INFO;
 
 /** A structure that describes the
@@ -436,16 +468,6 @@ typedef struct CmArmItsGroupNode {
   CM_OBJECT_TOKEN   ItsIdToken;
 } CM_ARM_ITS_GROUP_NODE;
 
-/** A structure that describes the
-    GIC ITS Identifiers for an ITS Group node.
-
-    ID: EArmObjGicItsIdentifierArray
-*/
-typedef struct CmArmGicItsIdentifier {
-  /// The ITS Identifier
-  UINT32  ItsId;
-} CM_ARM_ITS_IDENTIFIER;
-
 /** A structure that describes the
     Named component node for the Platform.
 
@@ -607,6 +629,16 @@ typedef struct CmArmPmcgNode {
   CM_OBJECT_TOKEN   ReferenceToken;
 } CM_ARM_PMCG_NODE;
 
+/** A structure that describes the
+    GIC ITS Identifiers for an ITS Group node.
+
+    ID: EArmObjGicItsIdentifierArray
+*/
+typedef struct CmArmGicItsIdentifier {
+  /// The ITS Identifier
+  UINT32  ItsId;
+} CM_ARM_ITS_IDENTIFIER;
+
 /** A structure that describes the
     ID Mappings for the Platform.
 
@@ -625,18 +657,35 @@ typedef struct CmArmIdMapping {
   UINT32    Flags;
 } CM_ARM_ID_MAPPING;
 
-/** A structure that describes the
-    SMMU interrupts for the Platform.
-
-    ID: EArmObjSmmuInterruptArray
+/** A structure that describes the Arm
+    Generic Interrupts.
 */
-typedef struct CmArmSmmuInterrupt {
+typedef struct CmArmGenericInterrupt {
   /// Interrupt number
   UINT32    Interrupt;
 
   /// Flags
   UINT32    Flags;
-} CM_ARM_SMMU_INTERRUPT;
+} CM_ARM_GENERIC_INTERRUPT;
+
+/** A structure that describes the SMMU interrupts for the Platform.
+
+    Interrupt   Interrupt number.
+    Flags       Interrupt flags as defined for SMMU node.
+
+    ID: EArmObjSmmuInterruptArray
+*/
+typedef CM_ARM_GENERIC_INTERRUPT CM_ARM_SMMU_INTERRUPT;
+
+/** A structure that describes the AML Extended Interrupts.
+
+    Interrupt   Interrupt number.
+    Flags       Interrupt flags as defined by the Interrupt
+                Vector Flags (Byte 3) of the Extended Interrupt
+                resource descriptor.
+                See EFI_ACPI_EXTENDED_INTERRUPT_FLAG_xxx in Acpi10.h
+*/
+typedef CM_ARM_GENERIC_INTERRUPT CM_ARM_EXTENDED_INTERRUPT;
 
 /** A structure that describes the Processor Hierarchy Node (Type 0) in PPTT
 
@@ -659,7 +708,7 @@ typedef struct CmArmProcHierarchyInfo {
   UINT32            NoOfPrivateResources;
   /// Token of the array which contains references to the resources private to
   /// this CM_ARM_PROC_HIERARCHY_INFO instance. This field is ignored if
-  /// the NoOfPrivateResources is 0, in which case it is recomended to set
+  /// the NoOfPrivateResources is 0, in which case it is recommended to set
   /// this field to CM_NULL_TOKEN.
   CM_OBJECT_TOKEN   PrivateResourcesArrayToken;
 } CM_ARM_PROC_HIERARCHY_INFO;
@@ -729,6 +778,106 @@ typedef struct CmArmObjRef {
   CM_OBJECT_TOKEN   ReferenceToken;
 } CM_ARM_OBJ_REF;
 
+/** A structure that describes the Memory Affinity Structure (Type 1) in SRAT
+
+    ID: EArmObjMemoryAffinityInfo
+*/
+typedef struct CmArmMemoryAffinityInfo {
+  /// The proximity domain to which the "range of memory" belongs.
+  UINT32            ProximityDomain;
+
+  /// Base Address
+  UINT64            BaseAddress;
+
+  /// Length
+  UINT64            Length;
+
+  /// Flags
+  UINT32            Flags;
+} CM_ARM_MEMORY_AFFINITY_INFO;
+
+/** A structure that describes the ACPI Device Handle (Type 0) in the
+    Generic Initiator Affinity structure in SRAT
+
+    ID: EArmObjDeviceHandleAcpi
+*/
+typedef struct CmArmDeviceHandleAcpi {
+  /// Hardware ID
+  UINT64  Hid;
+
+  /// Unique Id
+  UINT32  Uid;
+} CM_ARM_DEVICE_HANDLE_ACPI;
+
+/** A structure that describes the PCI Device Handle (Type 1) in the
+    Generic Initiator Affinity structure in SRAT
+
+    ID: EArmObjDeviceHandlePci
+*/
+typedef struct CmArmDeviceHandlePci {
+  /// PCI Segment Number
+  UINT16  SegmentNumber;
+
+  /// PCI Bus Number - Max 256 busses (Bits 15:8 of BDF)
+  UINT8  BusNumber;
+
+  /// PCI Device Number - Max 32 devices (Bits 7:3 of BDF)
+  UINT8   DeviceNumber;
+
+  /// PCI Function Number - Max 8 functions (Bits 2:0 of BDF)
+  UINT8   FunctionNumber;
+} CM_ARM_DEVICE_HANDLE_PCI;
+
+/** A structure that describes the Generic Initiator Affinity structure in SRAT
+
+    ID: EArmObjGenericInitiatorAffinityInfo
+*/
+typedef struct CmArmGenericInitiatorAffinityInfo {
+  /// The proximity domain to which the generic initiator belongs.
+  UINT32            ProximityDomain;
+
+  /// Flags
+  UINT32            Flags;
+
+  /// Device Handle Type
+  UINT8             DeviceHandleType;
+
+  /// Reference Token for the Device Handle
+  CM_OBJECT_TOKEN   DeviceHandleToken;
+} CM_ARM_GENERIC_INITIATOR_AFFINITY_INFO;
+
+/** A structure that describes the CMN-600 hardware.
+
+    ID: EArmObjCmn600Info
+*/
+typedef struct CmArmCmn600Info {
+  /// The PERIPHBASE address.
+  /// Corresponds to the Configuration Node Region (CFGR) base address.
+  UINT64                     PeriphBaseAddress;
+
+  /// The PERIPHBASE address length.
+  /// Corresponds to the CFGR base address length.
+  UINT64                     PeriphBaseAddressLength;
+
+  /// The ROOTNODEBASE address.
+  /// Corresponds to the Root node (ROOT) base address.
+  UINT64                     RootNodeBaseAddress;
+
+  /// The Debug and Trace Logic Controller (DTC) count.
+  /// CMN-600 can have maximum 4 DTCs.
+  UINT8                      DtcCount;
+
+  /// DTC Interrupt list.
+  /// The first interrupt resource descriptor pertains to
+  /// DTC[0], the second to DTC[1] and so on.
+  /// DtcCount determines the number of DTC Interrupts that
+  /// are populated. If DTC count is 2 then DtcInterrupt[2]
+  /// and DtcInterrupt[3] are ignored.
+  /// Note: The size of CM_ARM_CMN_600_INFO structure remains
+  /// constant and does not vary with the DTC count.
+  CM_ARM_EXTENDED_INTERRUPT  DtcInterrupt[4];
+} CM_ARM_CMN_600_INFO;
+
 #pragma pack()
 
 #endif // ARM_NAMESPACE_OBJECTS_H_
