@@ -83,7 +83,7 @@ SmmInternalAllocatePagesEx (
   Please Note this algorithm to allocate the memory map descriptor has a property
   that the memory allocated for memory entries always grows, and will never really be freed.
 
-  @return The Memory map descriptor dequed from the mFreeMemoryMapEntryList
+  @return The Memory map descriptor dequeued from the mFreeMemoryMapEntryList
 
 **/
 MEMORY_MAP *
@@ -117,7 +117,7 @@ AllocateMemoryMapEntry (
       FreeDescriptorEntries = (MEMORY_MAP *)(UINTN)Mem;
       //DEBUG((DEBUG_INFO, "New FreeDescriptorEntries - 0x%x\n", FreeDescriptorEntries));
       //
-      // Enque the free memmory map entries into the list
+      // Enqueue the free memory map entries into the list
       //
       for (Index = 0; Index< RUNTIME_PAGE_ALLOCATION_GRANULARITY / sizeof(MEMORY_MAP); Index++) {
         FreeDescriptorEntries[Index].Signature = MEMORY_MAP_SIGNATURE;
