@@ -54,12 +54,12 @@ EFI_STATUS
   @param  TestedMemorySize    Return the tested extended memory size.
   @param  TotalMemorySize     Return the whole system physical memory size.
                               The total memory size does not include memory in a slot with a disabled DIMM.
-  @param  ErrorOut            TRUE if the memory error occured.
+  @param  ErrorOut            TRUE if the memory error occurred.
   @param  IfTestAbort         Indicates that the user pressed "ESC" to skip the memory test.
 
   @retval EFI_SUCCESS         One block of memory passed the test.
   @retval EFI_NOT_FOUND       All memory blocks have already been tested.
-  @retval EFI_DEVICE_ERROR    Memory device error occured, and no agent can handle it.
+  @retval EFI_DEVICE_ERROR    Memory device error occurred, and no agent can handle it.
 
 **/
 typedef
