@@ -1,15 +1,9 @@
 /** @file
   The implementation of common functions shared by IP6 driver.
 
-  Copyright (c) 2009 - 2014, Intel Corporation. All rights reserved.<BR>
+  Copyright (c) 2009 - 2018, Intel Corporation. All rights reserved.<BR>
 
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php.
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -220,7 +214,7 @@ Ip6CreateLinkLocalAddr (
   }
 
   //
-  // Get the interface id if it is manully configured.
+  // Get the interface id if it is manually configured.
   //
   Ip6Config = &IpSb->Ip6ConfigInstance.Ip6Config;
   DataSize  = sizeof (EFI_IP6_CONFIG_INTERFACE_ID);
@@ -428,7 +422,7 @@ Ip6RemoveAddr (
   IP6_ADDRESS_INFO     *AddrInfo;
   EFI_IPv6_ADDRESS     SnMCastAddr;
 
-  if (IsListEmpty (AddressList) || *AddressCount < 1 || PrefixLength > IP6_PREFIX_NUM) {
+  if (IsListEmpty (AddressList) || *AddressCount < 1 || PrefixLength > IP6_PREFIX_MAX) {
     return EFI_INVALID_PARAMETER;
   }
 
@@ -606,7 +600,7 @@ Ip6CopyAddressByPrefix (
   UINT8 Mask;
 
   ASSERT (Dest != NULL && Src != NULL);
-  ASSERT (PrefixLength < IP6_PREFIX_NUM);
+  ASSERT (PrefixLength <= IP6_PREFIX_MAX);
 
   Byte = (UINT8) (PrefixLength / 8);
   Bit  = (UINT8) (PrefixLength % 8);
