@@ -538,7 +538,7 @@ Mtftp6DriverBindingStop (
                               then a new handle is created. If it is a pointer to an existing
                               UEFI handle, then the protocol is added to the existing UEFI handle.
 
-  @retval EFI_SUCCES            The protocol was added to ChildHandle.
+  @retval EFI_SUCCESS           The protocol was added to ChildHandle.
   @retval EFI_INVALID_PARAMETER ChildHandle is NULL.
   @retval Others                The child handle was not created.
 
@@ -637,7 +637,7 @@ ON_ERROR:
   @param[in]  This        Pointer to the EFI_SERVICE_BINDING_PROTOCOL instance.
   @param[in]  ChildHandle Handle of the child to destroy.
 
-  @retval EFI_SUCCES            The protocol was removed from ChildHandle.
+  @retval EFI_SUCCESS           The protocol was removed from ChildHandle.
   @retval EFI_UNSUPPORTED       ChildHandle does not support the protocol that is being removed.
   @retval EFI_INVALID_PARAMETER Child handle is NULL.
   @retval Others                The child handle was not destroyed
