@@ -2,14 +2,8 @@
     Defines DiskImage - the view of the file that is visible at any point,
     as well as the event handlers for editing the file
 
-  Copyright (c) 2005 - 2011, Intel Corporation. All rights reserved. <BR>
-  This program and the accompanying materials
-  are licensed and made available under the terms and conditions of the BSD License
-  which accompanies this distribution.  The full text of the license may be found at
-  http://opensource.org/licenses/bsd-license.php
-
-  THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-  WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+  Copyright (c) 2005 - 2018, Intel Corporation. All rights reserved. <BR>
+  SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
@@ -22,7 +16,7 @@
   Initialization function for HDiskImage
 
   @param[in] EFI_SUCCESS      The operation was successful.
-  @param[in] EFI_LOAD_ERROR   A load error occured.
+  @param[in] EFI_LOAD_ERROR   A load error occurred.
 **/
 EFI_STATUS
 HClipBoardInit (
@@ -33,7 +27,7 @@ HClipBoardInit (
   Initialization function for HDiskImage.
 
   @param[in] EFI_SUCCESS      The operation was successful.
-  @param[in] EFI_LOAD_ERROR   A load error occured.
+  @param[in] EFI_LOAD_ERROR   A load error occurred.
 **/
 EFI_STATUS
 HClipBoardCleanup (
