@@ -23,15 +23,25 @@
  */
 
 /*
- * Copyright (C) 2009-2020 Oracle Corporation
+ * Copyright (C) 2009-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -181,7 +191,7 @@ AssertCompile(HPET_TICKS_IN_100YR_PIIX >= UINT64_C(315360000000000000));
  */
 #define DEVHPET_LOCK_RETURN(a_pDevIns, a_pThis, a_rcBusy)  \
     do { \
-        int rcLock = PDMDevHlpCritSectEnter((a_pDevIns), &(a_pThis)->CritSect, (a_rcBusy)); \
+        int const rcLock = PDMDevHlpCritSectEnter((a_pDevIns), &(a_pThis)->CritSect, (a_rcBusy)); \
         if (RT_LIKELY(rcLock == VINF_SUCCESS)) \
         { /* likely */ } \
         else \
@@ -1303,15 +1313,11 @@ DECLINLINE(uint32_t) hpetR3TimerGetIrq(PHPET pThis, PCHPETTIMER pHpetTimer, uint
 
 
 /**
- * Device timer callback function.
- *
- * @param   pDevIns         Device instance of the device which registered the timer.
- * @param   pTimer          The timer handle.
- * @param   pvUser          Pointer to the HPET timer state.
+ * @callback_method_impl{FNTMTIMERDEV, Device timer callback function.}
  *
  * @note    Only the virtual sync lock is held when called.
  */
-static DECLCALLBACK(void) hpetR3Timer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
+static DECLCALLBACK(void) hpetR3Timer(PPDMDEVINS pDevIns, TMTIMERHANDLE hTimer, void *pvUser)
 {
     PHPET           pThis      = PDMDEVINS_2_DATA(pDevIns, PHPET);
     PHPETTIMER      pHpetTimer = (HPETTIMER *)pvUser;
@@ -1326,7 +1332,7 @@ static DECLCALLBACK(void) hpetR3Timer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void
     uint64_t        uCmp       = ASMAtomicUoReadU64(&pHpetTimer->u64Cmp);
     uint64_t const  uPeriod    = ASMAtomicUoReadU64(&pHpetTimer->u64Period);
     uint64_t const  fConfig    = ASMAtomicUoReadU64(&pHpetTimer->u64Config);
-    RT_NOREF(pTimer);
+    Assert(hTimer == pHpetTimer->hTimer);
 
     if (fConfig & HPET_TN_PERIODIC)
     {
@@ -1341,7 +1347,7 @@ static DECLCALLBACK(void) hpetR3Timer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void
             {
                 uint64_t const tsDeadline = tsNow + hpetTicksToNs(pThis, cTicksDiff);
                 Log4(("HPET[%u]: periodic: next in %llu\n", pHpetTimer->idxTimer, tsDeadline));
-                PDMDevHlpTimerSet(pDevIns, pHpetTimer->hTimer, tsDeadline);
+                PDMDevHlpTimerSet(pDevIns, hTimer, tsDeadline);
                 STAM_REL_COUNTER_INC(&pHpetTimer->StatSetTimer);
             }
             else
@@ -1353,7 +1359,7 @@ static DECLCALLBACK(void) hpetR3Timer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void
     else if (pHpetTimer->u8Wrap && hpet32bitTimerEx(fConfig))
     {
         pHpetTimer->u8Wrap = 0;         /* (only modified while owning the virtual sync lock) */
-        uint64_t const tsNow      = PDMDevHlpTimerGet(pDevIns, pHpetTimer->hTimer);
+        uint64_t const tsNow      = PDMDevHlpTimerGet(pDevIns, hTimer);
         uint64_t const uHpetNow   = nsToHpetTicks(pThis, tsNow + pThis->u64HpetOffset);
         uint64_t const cTicksDiff = hpetComputeDiff(fConfig, uCmp, uHpetNow);
         uint64_t const tsDeadline = tsNow + hpetTicksToNs(pThis, cTicksDiff);
@@ -1593,13 +1599,16 @@ static DECLCALLBACK(int) hpetR3LoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uin
     /*
      * Set the timer frequency hints.
      */
-    PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    rc = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rc, rc);
+
     for (uint32_t iTimer = 0; iTimer < cTimers; iTimer++)
     {
         PHPETTIMER pHpetTimer = &pThis->aTimers[iTimer];
         if (PDMDevHlpTimerIsActive(pDevIns, pHpetTimer->hTimer))
             hpetTimerSetFrequencyHint(pDevIns, pThis, pHpetTimer, pHpetTimer->u64Config, pHpetTimer->u64Period);
     }
+
     PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     return VINF_SUCCESS;
 }
@@ -1737,7 +1746,8 @@ static DECLCALLBACK(int) hpetR3Construct(PPDMDEVINS pDevIns, int iInstance, PCFG
     {
         PHPETTIMER pHpetTimer = &pThis->aTimers[i];
         rc = PDMDevHlpTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, hpetR3Timer, pHpetTimer,
-                                  TMTIMER_FLAGS_NO_CRIT_SECT, s_apszTimerNames[i], &pThis->aTimers[i].hTimer);
+                                  TMTIMER_FLAGS_NO_CRIT_SECT | TMTIMER_FLAGS_RING0,
+                                  s_apszTimerNames[i], &pThis->aTimers[i].hTimer);
         AssertRCReturn(rc, rc);
         uint64_t const cTicksPerSec = PDMDevHlpTimerGetFreq(pDevIns, pThis->aTimers[i].hTimer);
         if (cTicksPerSec != RT_NS_1SEC)
