@@ -6,15 +6,25 @@
  */
 
 /*
- * Copyright (C) 2018-2020 Oracle Corporation
+ * Copyright (C) 2018-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -273,7 +283,6 @@ static const char *s_aszStopBits[] =
 /**
  * Updates the IRQ state based on the current device state.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -410,7 +419,6 @@ DECLINLINE(uint8_t) uartFifoGet(PUARTFIFO pFifo)
 /**
  * Clears the given FIFO.
  *
- * @returns nothing.
  * @param   pFifo               The FIFO to clear.
  */
 DECLINLINE(void) uartFifoClear(PUARTFIFO pFifo)
@@ -500,7 +508,6 @@ DECLINLINE(size_t) uartFifoCopyFrom(PUARTFIFO pFifo, void *pvSrc, size_t cbCopy)
  * Updates the delta bits for the given MSR register value which has the status line
  * bits set.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -527,7 +534,6 @@ static void uartR3MsrUpdate(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pTh
 /**
  * Updates the serial port parameters of the attached driver with the current configuration.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -599,7 +605,6 @@ static void uartR3ParamsUpdate(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC
 /**
  * Updates the internal device state with the given PDM status line states.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -625,7 +630,6 @@ static void uartR3StsLinesUpdate(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCOREC
 /**
  * Fills up the receive FIFO with as much data as possible.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -679,7 +683,6 @@ static void uartR3RecvFifoFill(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC
 /**
  * Fetches a single byte and writes it to RBR.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -700,7 +703,6 @@ static void uartR3ByteFetch(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pTh
 /**
  * Fetches a ready data based on the FIFO setting.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -720,7 +722,6 @@ static void uartR3DataFetch(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pTh
  * Reset the transmit/receive related bits to the standard values
  * (after a detach/attach/reset event).
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -760,7 +761,6 @@ static void uartR3XferReset(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pTh
 /**
  * Tries to copy the specified amount of data from the active TX queue (register or FIFO).
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -876,7 +876,7 @@ static VBOXSTRICTRC uartXmit(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pT
         else
             PDMDevHlpTimerSetRelative(pDevIns, pThis->hTimerTxUnconnected, pThis->cSymbolXferTicks, NULL);
 
-        PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VINF_SUCCESS);
+        rc = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VINF_SUCCESS);
     }
 #endif
 
@@ -1503,12 +1503,12 @@ DECLHIDDEN(VBOXSTRICTRC) uartRegRead(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTC
 /**
  * @callback_method_impl{FNTMTIMERDEV, Fifo timer function.}
  */
-static DECLCALLBACK(void) uartR3RcvFifoTimeoutTimer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
+static DECLCALLBACK(void) uartR3RcvFifoTimeoutTimer(PPDMDEVINS pDevIns, TMTIMERHANDLE hTimer, void *pvUser)
 {
-    LogFlowFunc(("pDevIns=%#p pTimer=%#p pvUser=%#p\n", pDevIns, pTimer, pvUser));
+    LogFlowFunc(("pDevIns=%#p hTimer=%#p pvUser=%#p\n", pDevIns, hTimer, pvUser));
     PUARTCORER3 pThisCC = (PUARTCORECC)pvUser;
     PUARTCORE   pThis   = pThisCC->pShared;
-    RT_NOREF(pTimer);
+    RT_NOREF(hTimer);
 
     if (pThis->FifoRecv.cbUsed < pThis->FifoRecv.cbItl)
     {
@@ -1518,17 +1518,18 @@ static DECLCALLBACK(void) uartR3RcvFifoTimeoutTimer(PPDMDEVINS pDevIns, PTMTIMER
 }
 
 /**
- * @callback_method_impl{FNTMTIMERDEV, TX timer function when there is no driver connected for draining the THR/FIFO.}
+ * @callback_method_impl{FNTMTIMERDEV,
+ *      TX timer function when there is no driver connected for
+ *      draining the THR/FIFO.}
  */
-static DECLCALLBACK(void) uartR3TxUnconnectedTimer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
+static DECLCALLBACK(void) uartR3TxUnconnectedTimer(PPDMDEVINS pDevIns, TMTIMERHANDLE hTimer, void *pvUser)
 {
-    LogFlowFunc(("pDevIns=%#p pTimer=%#p pvUser=%#p\n", pDevIns, pTimer, pvUser));
+    LogFlowFunc(("pDevIns=%#p hTimer=%#p pvUser=%#p\n", pDevIns, hTimer, pvUser));
     PUARTCORER3 pThisCC = (PUARTCORECC)pvUser;
     PUARTCORE   pThis   = pThisCC->pShared;
-    RT_NOREF(pTimer);
+    Assert(hTimer == pThis->hTimerTxUnconnected);
 
-    VBOXSTRICTRC rc1 = PDMDevHlpTimerLockClock2(pDevIns, pThis->hTimerTxUnconnected, &pThis->CritSect,
-                                                VINF_SUCCESS /* must get it */);
+    VBOXSTRICTRC rc1 = PDMDevHlpTimerLockClock2(pDevIns, hTimer, &pThis->CritSect, VINF_SUCCESS /* must get it */);
     AssertRCReturnVoid(VBOXSTRICTRC_VAL(rc1));
 
     uint8_t bVal = 0;
@@ -1570,14 +1571,14 @@ static DECLCALLBACK(void) uartR3TxUnconnectedTimer(PPDMDEVINS pDevIns, PTMTIMER
     }
 
     if (cbRead == 1)
-        PDMDevHlpTimerSetRelative(pDevIns, pThis->hTimerTxUnconnected, pThis->cSymbolXferTicks, NULL);
+        PDMDevHlpTimerSetRelative(pDevIns, hTimer, pThis->cSymbolXferTicks, NULL);
     else
     {
         /* NO data left, set the transmitter holding register as empty. */
         UART_REG_SET(pThis->uRegLsr, UART_REG_LSR_TEMT);
     }
 
-    PDMDevHlpTimerUnlockClock2(pDevIns, pThis->hTimerTxUnconnected, &pThis->CritSect);
+    PDMDevHlpTimerUnlockClock2(pDevIns, hTimer, &pThis->CritSect);
 }
 
 
@@ -1596,7 +1597,9 @@ static DECLCALLBACK(int) uartR3DataAvailRdrNotify(PPDMISERIALPORT pInterface, si
 
     AssertMsg((uint32_t)cbAvail == cbAvail, ("Too much data available\n"));
 
-    PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    int rcLock = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rcLock, rcLock);
+
     uint32_t cbAvailOld = ASMAtomicAddU32(&pThis->cbAvailRdr, (uint32_t)cbAvail);
     LogFlow(("    cbAvailRdr=%u -> cbAvailRdr=%u\n", cbAvailOld, cbAvail + cbAvailOld));
     if (pThis->uRegFcr & UART_REG_FCR_FIFO_EN)
@@ -1605,12 +1608,16 @@ static DECLCALLBACK(int) uartR3DataAvailRdrNotify(PPDMISERIALPORT pInterface, si
     {
         size_t cbRead = 0;
         int rc = pThisCC->pDrvSerial->pfnReadRdr(pThisCC->pDrvSerial, &pThis->uRegRbr, 1, &cbRead);
-        AssertMsg(RT_SUCCESS(rc) && cbRead == 1, ("This shouldn't fail and always return one byte!\n")); RT_NOREF(rc);
-        UART_REG_SET(pThis->uRegLsr, UART_REG_LSR_DR);
-        uartIrqUpdate(pDevIns, pThis, pThisCC);
-    }
-    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
+        AssertRC(rc);
 
+        if (cbRead)
+        {
+            UART_REG_SET(pThis->uRegLsr, UART_REG_LSR_DR);
+            uartIrqUpdate(pDevIns, pThis, pThisCC);
+        }
+    }
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     return VINF_SUCCESS;
 }
 
@@ -1626,10 +1633,13 @@ static DECLCALLBACK(int) uartR3DataSentNotify(PPDMISERIALPORT pInterface)
     PPDMDEVINS  pDevIns = pThisCC->pDevIns;
 
     /* Set the transmitter empty bit because everything was sent. */
-    PDMCritSectEnter(&pThis->CritSect, VERR_IGNORED);
+    int const rcLock = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rcLock, rcLock);
+
     UART_REG_SET(pThis->uRegLsr, UART_REG_LSR_TEMT);
     uartIrqUpdate(pDevIns, pThis, pThisCC);
-    PDMCritSectLeave(&pThis->CritSect);
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     return VINF_SUCCESS;
 }
 
@@ -1646,10 +1656,12 @@ static DECLCALLBACK(int) uartR3ReadWr(PPDMISERIALPORT pInterface, void *pvBuf, s
 
     AssertReturn(cbRead > 0, VERR_INVALID_PARAMETER);
 
-    PDMCritSectEnter(&pThis->CritSect, VERR_IGNORED);
-    uartR3TxQueueCopyFrom(pDevIns, pThis, pThisCC, pvBuf, cbRead, pcbRead);
-    PDMCritSectLeave(&pThis->CritSect);
+    int const rcLock = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rcLock, rcLock);
 
+    uartR3TxQueueCopyFrom(pDevIns, pThis, pThisCC, pvBuf, cbRead, pcbRead);
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     LogFlowFunc(("-> VINF_SUCCESS{*pcbRead=%zu}\n", *pcbRead));
     return VINF_SUCCESS;
 }
@@ -1664,10 +1676,12 @@ static DECLCALLBACK(int) uartR3NotifyStsLinesChanged(PPDMISERIALPORT pInterface,
     PUARTCORECC pThisCC = RT_FROM_MEMBER(pInterface, UARTCORECC, ISerialPort);
     PUARTCORE   pThis   = pThisCC->pShared;
     PPDMDEVINS  pDevIns = pThisCC->pDevIns;
+    int const   rcLock  = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rcLock, rcLock);
 
-    PDMCritSectEnter(&pThis->CritSect, VERR_IGNORED);
     uartR3StsLinesUpdate(pDevIns, pThis, pThisCC, fNewStatusLines);
-    PDMCritSectLeave(&pThis->CritSect);
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     return VINF_SUCCESS;
 }
 
@@ -1681,11 +1695,13 @@ static DECLCALLBACK(int) uartR3NotifyBrk(PPDMISERIALPORT pInterface)
     PUARTCORECC pThisCC = RT_FROM_MEMBER(pInterface, UARTCORECC, ISerialPort);
     PUARTCORE   pThis   = pThisCC->pShared;
     PPDMDEVINS  pDevIns = pThisCC->pDevIns;
+    int const   rcLock  = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    AssertRCReturn(rcLock, rcLock);
 
-    PDMCritSectEnter(&pThis->CritSect, VERR_IGNORED);
     UART_REG_SET(pThis->uRegLsr, UART_REG_LSR_BI);
     uartIrqUpdate(pDevIns, pThis, pThisCC);
-    PDMCritSectLeave(&pThis->CritSect);
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
     return VINF_SUCCESS;
 }
 
@@ -1893,7 +1909,6 @@ DECLHIDDEN(int) uartR3LoadDone(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC
 /**
  * Resets the given UART core instance.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -1943,18 +1958,23 @@ DECLHIDDEN(int) uartR3Attach(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pT
             AssertLogRelMsgFailed(("Configuration error: instance %d has no serial interface!\n", pDevIns->iInstance));
             return VERR_PDM_MISSING_INTERFACE;
         }
-        PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
-        uartR3XferReset(pDevIns, pThis, pThisCC);
-        PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
+        rc = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+        if (RT_SUCCESS(rc))
+        {
+            uartR3XferReset(pDevIns, pThis, pThisCC);
+            PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
+        }
     }
     else if (rc == VERR_PDM_NO_ATTACHED_DRIVER)
     {
         pThisCC->pDrvBase = NULL;
         pThisCC->pDrvSerial = NULL;
-        rc = VINF_SUCCESS;
-        PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
-        uartR3XferReset(pDevIns, pThis, pThisCC);
-        PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
+        rc = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+        if (RT_SUCCESS(rc))
+        {
+            uartR3XferReset(pDevIns, pThis, pThisCC);
+            PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
+        }
         LogRel(("Serial#%d: no unit\n", pDevIns->iInstance));
     }
     else /* Don't call VMSetError here as we assume that the driver already set an appropriate error */
@@ -1967,7 +1987,6 @@ DECLHIDDEN(int) uartR3Attach(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pT
 /**
  * Detaches any attached driver from the given UART core instance.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared serial port instance data.
  * @param   pThisCC             The serial port instance data for the current context.
@@ -1977,14 +1996,18 @@ DECLHIDDEN(void) uartR3Detach(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC p
     /* Zero out important members. */
     pThisCC->pDrvBase   = NULL;
     pThisCC->pDrvSerial = NULL;
+    int const rcLock = PDMDevHlpCritSectEnter(pDevIns, &pThis->CritSect, VERR_IGNORED);
+    PDM_CRITSECT_RELEASE_ASSERT_RC_DEV(pDevIns, &pThis->CritSect, rcLock);
+
     uartR3XferReset(pDevIns, pThis, pThisCC);
+
+    PDMDevHlpCritSectLeave(pDevIns, &pThis->CritSect);
 }
 
 
 /**
  * Destroys the given UART core instance freeing all allocated resources.
  *
- * @returns nothing.
  * @param   pDevIns             The device instance.
  * @param   pThis               The shared UART core instance data..
  */
@@ -2068,7 +2091,7 @@ DECLHIDDEN(int) uartR3Init(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pThi
      * Create the receive FIFO character timeout indicator timer.
      */
     rc = PDMDevHlpTimerCreate(pDevIns, TMCLOCK_VIRTUAL, uartR3RcvFifoTimeoutTimer, pThisCC,
-                              TMTIMER_FLAGS_NO_CRIT_SECT, "UART Rcv FIFO Timer",
+                              TMTIMER_FLAGS_NO_CRIT_SECT | TMTIMER_FLAGS_RING0, "UART Rcv FIFO",
                               &pThis->hTimerRcvFifoTimeout);
     AssertRCReturn(rc, rc);
 
@@ -2079,7 +2102,7 @@ DECLHIDDEN(int) uartR3Init(PPDMDEVINS pDevIns, PUARTCORE pThis, PUARTCORECC pThi
      * Create the transmit timer when no device is connected.
      */
     rc = PDMDevHlpTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, uartR3TxUnconnectedTimer, pThisCC,
-                              TMTIMER_FLAGS_NO_CRIT_SECT, "UART TX uncon. Timer",
+                              TMTIMER_FLAGS_NO_CRIT_SECT | TMTIMER_FLAGS_NO_RING0, "UART TX unconnect",
                               &pThis->hTimerTxUnconnected);
     AssertRCReturn(rc, rc);
 
