@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2017 Oracle Corporation
+ * Copyright (C) 2017-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -38,12 +48,16 @@
 #include <iprt/trace.h>
 
 #include "tstDeviceInternal.h"
+#include "tstDeviceCfg.h"
+#include "tstDeviceBuiltin.h"
 
 
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
 
+#define PDM_MAX_DEVICE_INSTANCE_SIZE      _4M
+
 
 /*********************************************************************************************************************************
 *   Structures and Typedefs                                                                                                      *
@@ -140,25 +154,6 @@ typedef struct TSTDEVPDMMOD
 } TSTDEVPDMMOD;
 /** Pointer to a PDM module descriptor. */
 typedef TSTDEVPDMMOD *PTSTDEVPDMMOD;
-/** Pointer to a const PDM module descriptor. */
-typedef const TSTDEVPDMMOD *PCTSTDEVPDMMOD;
-
-/**
- * PDM device descriptor.
- */
-typedef struct TSTDEVPDMDEV
-{
-    /** Node for the known device list. */
-    RTLISTNODE                      NdPdmDevs;
-    /** Pointer to the PDM module containing the device. */
-    PCTSTDEVPDMMOD                  pPdmMod;
-    /** Device registration structure. */
-    const struct PDMDEVREG          *pReg;
-} TSTDEVPDMDEV;
-/** Pointer to a PDM device descriptor .*/
-typedef TSTDEVPDMDEV *PTSTDEVPDMDEV;
-/** Pointer to a constant PDM device descriptor .*/
-typedef const TSTDEVPDMDEV *PCTSTDEVPDMDEV;
 
 
 /**
@@ -190,6 +185,9 @@ typedef struct TSTDEVPDMR0IMPORTS
 } TSTDEVPDMR0IMPORTS;
 typedef const TSTDEVPDMR0IMPORTS *PCTSTDEVPDMR0IMPORTS;
 
+typedef DECLCALLBACKTYPE(int, FNR0MODULEINIT,(void *hMod));
+typedef FNR0MODULEINIT *PFNR0MODULEINIT;
+
 
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
@@ -200,30 +198,25 @@ RTLISTANCHOR g_LstPlugins;
 RTLISTANCHOR g_LstTestcases;
 /** List of registered PDM modules. */
 RTLISTANCHOR g_LstPdmMods;
+/** List of registered PDM R0 modules. */
+RTLISTANCHOR g_LstPdmR0Mods;
 /** List of registered PDM devices. */
 RTLISTANCHOR g_LstPdmDevs;
 
-extern const TSTDEVVMMCALLBACKS g_tstDevVmmCallbacks;
+static int tstDevPdmR0RegisterModule(void *hMod, PPDMDEVMODREGR0 pModReg);
 
 /**
  * PDM R0 imports we implement.
  */
 static const TSTDEVPDMR0IMPORTS g_aPdmR0Imports[] =
 {
-    {"IOMMMIOMapMMIO2Page",            (PFNRT)IOMMMIOMapMMIO2Page},
-    {"IOMMMIOResetRegion",             (PFNRT)IOMMMIOResetRegion},
+    {"SUPR0TracerFireProbe",           (PFNRT)NULL},
+    {"SUPSemEventSignal",              (PFNRT)NULL},
+    {"PDMR0DeviceRegisterModule",      (PFNRT)tstDevPdmR0RegisterModule},
+    {"PDMR0DeviceDeregisterModule",    (PFNRT)NULL},
+    {"PGMShwMakePageWritable",         (PFNRT)NULL},
     {"IntNetR0IfSend",                 (PFNRT)/*IntNetR0IfSend*/NULL},
     {"IntNetR0IfSetPromiscuousMode",   (PFNRT)/*IntNetR0IfSetPromiscuousMode*/NULL},
-    {"PDMCritSectEnterDebug",          (PFNRT)PDMCritSectEnterDebug},
-    {"PDMCritSectIsOwner",             (PFNRT)PDMCritSectIsOwner},
-    {"PDMCritSectLeave",               (PFNRT)PDMCritSectLeave},
-    {"PDMCritSectTryEnterDebug",       (PFNRT)PDMCritSectTryEnterDebug},
-    {"PDMHCCritSectScheduleExitEvent", (PFNRT)PDMHCCritSectScheduleExitEvent},
-    {"PDMNsAllocateBandwidth",         (PFNRT)PDMNsAllocateBandwidth},
-    {"PDMQueueAlloc",                  (PFNRT)PDMQueueAlloc},
-    {"PDMQueueInsert",                 (PFNRT)PDMQueueInsert},
-    {"PGMHandlerPhysicalPageTempOff",  (PFNRT)PGMHandlerPhysicalPageTempOff},
-    {"PGMShwMakePageWritable",         (PFNRT)PGMShwMakePageWritable},
     {"RTAssertMsg1Weak",               (PFNRT)RTAssertMsg1Weak},
     {"RTAssertMsg2Weak",               (PFNRT)RTAssertMsg2Weak},
     {"RTAssertShouldPanic",            (PFNRT)RTAssertShouldPanic},
@@ -236,19 +229,9 @@ static const TSTDEVPDMR0IMPORTS g_aPdmR0Imports[] =
     {"RTTimeMilliTS",                  (PFNRT)RTTimeMilliTS},
     {"RTTimeNanoTS",                   (PFNRT)RTTimeNanoTS},
     {"RTTraceBufAddMsgF",              (PFNRT)RTTraceBufAddMsgF},
-    {"SUPSemEventSignal",              (PFNRT)SUPSemEventSignal},
-    {"TMTimerGet",                     (PFNRT)TMTimerGet},
-    {"TMTimerGetFreq",                 (PFNRT)TMTimerGetFreq},
-    {"TMTimerIsActive",                (PFNRT)TMTimerIsActive},
-    {"TMTimerIsLockOwner",             (PFNRT)TMTimerIsLockOwner},
-    {"TMTimerLock",                    (PFNRT)TMTimerLock},
-    {"TMTimerSet",                     (PFNRT)TMTimerSet},
-    {"TMTimerSetFrequencyHint",        (PFNRT)TMTimerSetFrequencyHint},
-    {"TMTimerSetMicro",                (PFNRT)TMTimerSetMicro},
-    {"TMTimerSetMillies",              (PFNRT)TMTimerSetMillies},
-    {"TMTimerSetNano",                 (PFNRT)TMTimerSetNano},
-    {"TMTimerStop",                    (PFNRT)TMTimerStop},
-    {"TMTimerUnlock",                  (PFNRT)TMTimerUnlock},
+    {"RTMemAllocZTag",                 (PFNRT)RTMemAllocZTag},
+    {"RTMemFree",                      (PFNRT)RTMemFree},
+    {"RTStrPrintf",                    (PFNRT)RTStrPrintf},
     {"nocrt_memcmp",                   (PFNRT)memcmp},
     {"nocrt_memcpy",                   (PFNRT)memcpy},
     {"nocrt_memmove",                  (PFNRT)memmove},
@@ -326,7 +309,8 @@ static DECLCALLBACK(int) tstDevRegisterTestcase(void *pvUser, PCTSTDEVTESTCASERE
         if (RT_LIKELY(pTestcase))
         {
             pTestcase->pPlugin = pPlugin;
-            pPlugin->cRefs++;
+            if (pPlugin)
+                pPlugin->cRefs++;
             pTestcase->pTestcaseReg = pTestcaseReg;
             RTListAppend(&g_LstTestcases, &pTestcase->NdTestcases);
             return VINF_SUCCESS;
@@ -427,15 +411,38 @@ static int tstDevLoadPlugin(const char *pszFilename)
  * Checks whether the given testcase name is already existing.
  *
  * @returns Pointer to already loaded plugin, NULL if not found.
- * @param   pszFilename    The filename to check.
+ * @param   pszFilename     The filename to check.
+ * @param   ppR0Reg         Where to store the pointer to the R0 registration record
+ *                          if existing, optional.
  */
-static PCTSTDEVPDMDEV tstDevPdmDeviceFind(const char *pszName)
+DECLHIDDEN(PCTSTDEVPDMDEV) tstDevPdmDeviceFind(const char *pszName, PCPDMDEVREGR0 *ppR0Reg)
 {
     PCTSTDEVPDMDEV pIt;
     RTListForEach(&g_LstPdmDevs, pIt, TSTDEVPDMDEV, NdPdmDevs)
     {
         if (!RTStrCmp(pIt->pReg->szName, pszName))
+        {
+            if (ppR0Reg)
+            {
+                *ppR0Reg = NULL;
+
+                PPDMDEVMODREGR0 pItR0;
+                RTListForEach(&g_LstPdmR0Mods, pItR0, PDMDEVMODREGR0, ListEntry)
+                {
+                    for (uint32_t i = 0; i < pItR0->cDevRegs; i++)
+                    {
+                        PCPDMDEVREGR0 pReg = pItR0->papDevRegs[i];
+                        if (!RTStrCmp(pReg->szName, pszName))
+                        {
+                            *ppR0Reg = pReg;
+                            return pIt;
+                        }
+                    }
+                }
+            }
+
             return pIt;
+        }
     }
 
     return NULL;
@@ -480,16 +487,6 @@ static DECLCALLBACK(int) tstDevPdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCP
                     &&  tstDevPdmR3IsValidName(pReg->szName),
                     ("Invalid name '%.*s'\n", sizeof(pReg->szName), pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
-    AssertMsgReturn(   !(pReg->fFlags & PDM_DEVREG_FLAGS_RC)
-                    || (   pReg->szRCMod[0]
-                        && strlen(pReg->szRCMod) < sizeof(pReg->szRCMod)),
-                    ("Invalid GC module name '%s' - (Device %s)\n", pReg->szRCMod, pReg->szName),
-                    VERR_PDM_INVALID_DEVICE_REGISTRATION);
-    AssertMsgReturn(   !(pReg->fFlags & PDM_DEVREG_FLAGS_R0)
-                    || (   pReg->szR0Mod[0]
-                        && strlen(pReg->szR0Mod) < sizeof(pReg->szR0Mod)),
-                    ("Invalid R0 module name '%s' - (Device %s)\n", pReg->szR0Mod, pReg->szName),
-                    VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn((pReg->fFlags & PDM_DEVREG_FLAGS_HOST_BITS_MASK) == PDM_DEVREG_FLAGS_HOST_BITS_DEFAULT,
                     ("Invalid host bits flags! fFlags=%#x (Device %s)\n", pReg->fFlags, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_HOST_BITS);
@@ -502,8 +499,8 @@ static DECLCALLBACK(int) tstDevPdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCP
     AssertMsgReturn(pReg->cMaxInstances > 0,
                     ("Max instances %u! (Device %s)\n", pReg->cMaxInstances, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
-    AssertMsgReturn(pReg->cbInstance <= (uint32_t)(pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0)  ? 96 * _1K : _1M),
-                    ("Instance size %d bytes! (Device %s)\n", pReg->cbInstance, pReg->szName),
+    AssertMsgReturn(pReg->cbInstanceCC <= (uint32_t)(pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0)  ? 96 * _1K : _1M),
+                    ("Instance size %d bytes! (Device %s)\n", pReg->cbInstanceCC, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn(pReg->pfnConstruct,
                     ("No constructor! (Device %s)\n", pReg->szName),
@@ -520,7 +517,7 @@ static DECLCALLBACK(int) tstDevPdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCP
      */
     int rc = VINF_SUCCESS;
     PCTSTDEVPDMDEVREGCBINT pRegCB = (PCTSTDEVPDMDEVREGCBINT)pCallbacks;
-    if (!tstDevPdmDeviceFind(pReg->szName))
+    if (!tstDevPdmDeviceFind(pReg->szName, NULL /*ppR0Reg*/))
     {
         PTSTDEVPDMDEV pPdmDev = (PTSTDEVPDMDEV)RTMemAllocZ(sizeof(TSTDEVPDMDEV));
         if (RT_LIKELY(pPdmDev))
@@ -577,6 +574,8 @@ static DECLCALLBACK(int) tstDevPdmLoadR0RcModGetImport(RTLDRMOD hLdrMod, const c
     RT_NOREF(hLdrMod, uSymbol, pszModule);
     PTSTDEVPDMMOD pMod = (PTSTDEVPDMMOD)pvUser;
 
+    RTPrintf("Looking for %s\n", pszSymbol);
+
     /* Resolve the import. */
     PCTSTDEVPDMR0IMPORTS pImpDesc = NULL;
     bool fFound = false;
@@ -638,6 +637,22 @@ static DECLCALLBACK(int) tstDevPdmLoadR0RcModGetImport(RTLDRMOD hLdrMod, const c
 }
 
 
+/**
+ * The PDMR0RegisterModule() export called by loaded R0 modules.
+ *
+ * @returns VBox status code.
+ * @param   hMod                    The module handle.
+ * @param   pModReg                 The module registration structure.
+ */
+static int tstDevPdmR0RegisterModule(void *hMod, PPDMDEVMODREGR0 pModReg)
+{
+    RT_NOREF(hMod);
+    RTListAppend(&g_LstPdmR0Mods, &pModReg->ListEntry);
+    return VINF_SUCCESS;
+}
+
+
+
 /**
  * Loads a new R0 modules given by the filename.
  *
@@ -682,7 +697,16 @@ static int tstDevPdmLoadR0RcMod(PTSTDEVPDMMOD pMod)
                 /* Get the bits. */
                 rc = RTLdrGetBits(pMod->hLdrMod, pMod->R0Rc.pvBits, (uintptr_t)pMod->R0Rc.pvBits,
                                   tstDevPdmLoadR0RcModGetImport, pMod);
-                if (RT_FAILURE(rc))
+                if (RT_SUCCESS(rc))
+                {
+                    /* Resolve module init entry and call it. */
+                    PFNR0MODULEINIT pfnR0ModuleInit;
+                    rc = RTLdrGetSymbolEx(pMod->hLdrMod, pMod->R0Rc.pvBits, (uintptr_t)pMod->R0Rc.pvBits,
+                                          UINT32_MAX, "ModuleInit", (PRTLDRADDR)&pfnR0ModuleInit);
+                    if (RT_SUCCESS(rc))
+                        rc = pfnR0ModuleInit(pMod);
+                }
+                else
                     RTMemFreeEx(pMod->R0Rc.pbTrampoline, pMod->R0Rc.cbBits);
             }
 
@@ -812,68 +836,69 @@ DECLHIDDEN(int) tstDevPdmLdrGetSymbol(PTSTDEVDUTINT pThis, const char *pszMod, T
 }
 
 
-static TSTDEVCFGITEM s_aTestcaseCfg[] =
-{
-    {"CtrlMemBufSize", TSTDEVCFGITEMTYPE_INTEGER, "0"  },
-    {NULL,             TSTDEVCFGITEMTYPE_INVALID, NULL }
-};
-
-static TSTDEVTESTCASEREG s_TestcaseDef =
-{
-    "test",
-    "Testcase during implementation",
-    "nvme",
-    0,
-    &s_aTestcaseCfg[0],
-    NULL,
-};
-
 /**
  * Create a new PDM device with default config.
  *
  * @returns VBox status code.
  * @param   pszName                 Name of the device to create.
+ * @param   pDut                    The device under test structure the created PDM device instance is exercised under.
  */
-static int tstDevPdmDevCreate(const char *pszName)
+static int tstDevPdmDevR3Create(const char *pszName, PTSTDEVDUTINT pDut)
 {
     int rc = VINF_SUCCESS;
-    PCTSTDEVPDMDEV pPdmDev = tstDevPdmDeviceFind(pszName);
+    PCTSTDEVPDMDEV pPdmDev = tstDevPdmDeviceFind(pszName, NULL);
     if (RT_LIKELY(pPdmDev))
     {
-        TSTDEVDUTINT Dut;
-        Dut.pTestcaseReg    = &s_TestcaseDef;
-        Dut.enmCtx          = TSTDEVDUTCTX_R3;
-        Dut.pVm             = NULL;
-        Dut.SupSession.pDut = &Dut;
-        RTListInit(&Dut.LstIoPorts);
-        RTListInit(&Dut.LstTimers);
-        RTListInit(&Dut.LstMmHeap);
-        RTListInit(&Dut.LstPdmThreads);
-        RTListInit(&Dut.SupSession.LstSupSem);
-        CFGMNODE Cfg;
-        Cfg.pVmmCallbacks = &g_tstDevVmmCallbacks;
-        Cfg.pDut = &Dut;
+        PPDMCRITSECT pCritSect;
+        /* Figure out how much we need. */
+        uint32_t cb = RT_UOFFSETOF_DYN(PDMDEVINS, achInstanceData[pPdmDev->pReg->cbInstanceCC]);
+        cb  = RT_ALIGN_32(cb, 64);
+        uint32_t const offShared   = cb;
+        cb += RT_ALIGN_32(pPdmDev->pReg->cbInstanceShared, 64);
+        uint32_t const cbCritSect  = RT_ALIGN_32(sizeof(*pCritSect), 64);
+        cb += cbCritSect;
+        uint32_t const cbMsixState = RT_ALIGN_32(pPdmDev->pReg->cMaxMsixVectors * 16 + (pPdmDev->pReg->cMaxMsixVectors + 7) / 8, _4K);
+        uint32_t const cbPciDev    = RT_ALIGN_32(RT_UOFFSETOF_DYN(PDMPCIDEV, abMsixState[cbMsixState]), 64);
+        uint32_t const cPciDevs    = RT_MIN(pPdmDev->pReg->cMaxPciDevices, 1024);
+        uint32_t const cbPciDevs   = cbPciDev * cPciDevs;
+        cb += cbPciDevs;
 
-        rc = RTCritSectRwInit(&Dut.CritSectLists);
-        AssertRC(rc);
-
-        PPDMDEVINS pDevIns = (PPDMDEVINS)RTMemAllocZ(RT_UOFFSETOF_DYN(PDMDEVINS, achInstanceData[pPdmDev->pReg->cbInstance]));
+        PPDMDEVINS pDevIns = (PPDMDEVINS)RTMemAllocZ(cb);
         pDevIns->u32Version               = PDM_DEVINS_VERSION;
         pDevIns->iInstance                = 0;
         pDevIns->pReg                     = pPdmDev->pReg;
         pDevIns->pvInstanceDataR3         = &pDevIns->achInstanceData[0];
         pDevIns->pHlpR3                   = &g_tstDevPdmDevHlpR3;
-        pDevIns->pCfg                     = &Cfg;
-        pDevIns->Internal.s.pVmmCallbacks = &g_tstDevVmmCallbacks;
-        pDevIns->Internal.s.pDut          = &Dut;
-        rc = pPdmDev->pReg->pfnConstruct(pDevIns, 0, &Cfg);
-        if (RT_SUCCESS(rc))
+        pDevIns->pCfg                     = &pDut->Cfg;
+        pDevIns->Internal.s.pDut          = pDut;
+        pDevIns->cbRing3                  = cb;
+        pDevIns->fR0Enabled               = false;
+        pDevIns->fRCEnabled               = false;
+        pDevIns->pvInstanceDataR3         = (uint8_t *)pDevIns + offShared;
+        pDevIns->pvInstanceDataForR3      = &pDevIns->achInstanceData[0];
+        pCritSect = (PPDMCRITSECT)((uint8_t *)pDevIns + offShared + RT_ALIGN_32(pPdmDev->pReg->cbInstanceShared, 64));
+        pDevIns->pCritSectRoR3            = pCritSect;
+        pDevIns->cbPciDev                 = cbPciDev;
+        pDevIns->cPciDevs                 = cPciDevs;
+        for (uint32_t iPciDev = 0; iPciDev < cPciDevs; iPciDev++)
         {
-            PRTDEVDUTIOPORT pIoPort = RTListGetFirst(&Dut.LstIoPorts, RTDEVDUTIOPORT, NdIoPorts);
-            uint32_t uVal = 0;
-            PDMCritSectEnter(pDevIns->pCritSectRoR3, VERR_IGNORED);
-            pIoPort->pfnInR0(pDevIns, pIoPort->pvUserR0, pIoPort->PortStart, &uVal, sizeof(uint8_t));
-            PDMCritSectLeave(pDevIns->pCritSectRoR3);
+            PPDMPCIDEV pPciDev = (PPDMPCIDEV)((uint8_t *)pDevIns->pCritSectRoR3 + cbCritSect + cbPciDev * iPciDev);
+            if (iPciDev < RT_ELEMENTS(pDevIns->apPciDevs))
+                pDevIns->apPciDevs[iPciDev] = pPciDev;
+            pPciDev->cbConfig           = _4K;
+            pPciDev->cbMsixState        = cbMsixState;
+            pPciDev->idxSubDev          = (uint16_t)iPciDev;
+            pPciDev->u32Magic           = PDMPCIDEV_MAGIC;
+        }
+
+        RTCritSectInit(&pCritSect->s.CritSect);
+        rc = pPdmDev->pReg->pfnConstruct(pDevIns, 0, pDevIns->pCfg);
+        if (RT_SUCCESS(rc))
+            pDut->pDevIns = pDevIns;
+        else
+        {
+            rc = pPdmDev->pReg->pfnDestruct(pDevIns);
+            RTMemFree(pDevIns);
         }
     }
     else
@@ -882,6 +907,87 @@ static int tstDevPdmDevCreate(const char *pszName)
     return rc;
 }
 
+
+DECLHIDDEN(int) tstDevPdmDeviceR3Construct(PTSTDEVDUTINT pDut)
+{
+    PPDMDEVINS pDevInsR3 = pDut->pDevIns;
+
+    pDevInsR3->pReg                     = pDut->pPdmDev->pReg;
+    pDevInsR3->pHlpR3                   = &g_tstDevPdmDevHlpR3;
+    pDevInsR3->pCfg                     = &pDut->Cfg;
+    pDevInsR3->Internal.s.pDut          = pDut;
+
+    return pDevInsR3->pReg->pfnConstruct(pDevInsR3, 0, &pDut->Cfg);
+}
+
+
+DECLCALLBACK(void *) tstDevTestsRun_QueryInterface(PPDMIBASE pInterface, const char *pszIID)
+{
+    RT_NOREF(pInterface, pszIID);
+#if 0
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pDrvIns->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMILEDCONNECTORS, &pThis->ILedConnectors);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIMEDIANOTIFY, &pThis->IMediaNotify);
+#endif
+    return NULL;
+}
+
+/**
+ * Run a given test config.
+ *
+ * @returns VBox status code.
+ * @param   pDevTstCfg          The test config to run.
+ */
+static int tstDevTestsRun(PCTSTDEVCFG pDevTstCfg)
+{
+    int rc = VINF_SUCCESS;
+
+    for (uint32_t i = 0; i < pDevTstCfg->cTests; i++)
+    {
+        PCTSTDEVTEST pTest = &pDevTstCfg->aTests[i];
+
+        TSTDEVDUTINT Dut;
+        Dut.pTest           = pTest;
+        Dut.enmCtx          = TSTDEVDUTCTX_R3;
+        Dut.pVm             = (PVM)0x1000;
+        Dut.SupSession.pDut = &Dut;
+        Dut.Cfg.pDut        = &Dut;
+        Dut.pPdmDev         = tstDevPdmDeviceFind(pDevTstCfg->pszDevName, NULL /*ppPdmDevR0*/);
+
+        Dut.IBaseSts.pfnQueryInterface = tstDevTestsRun_QueryInterface;
+
+        RTListInit(&Dut.LstIoPorts);
+        RTListInit(&Dut.LstMmio);
+        RTListInit(&Dut.LstTimers);
+        RTListInit(&Dut.LstMmHeap);
+        RTListInit(&Dut.LstPdmThreads);
+        RTListInit(&Dut.LstSsmHandlers);
+        RTListInit(&Dut.SupSession.LstSupSem);
+
+        rc = RTCritSectRwInit(&Dut.CritSectLists);
+        AssertRC(rc);
+
+        rc = RTCritSectInitEx(&Dut.CritSectNop.s.CritSect, RTCRITSECT_FLAGS_NOP, NIL_RTLOCKVALCLASS, RTLOCKVAL_SUB_CLASS_NONE, "DutNop");
+        AssertRC(rc);
+
+        if (!pTest->fR0Enabled)
+            rc = tstDevPdmDevR3Create(pDevTstCfg->pszDevName, &Dut);
+        else
+            rc = tstDevPdmDevR0R3Create(pDevTstCfg->pszDevName, pTest->fRCEnabled, &Dut);
+        if (RT_SUCCESS(rc))
+        {
+            PCTSTDEVTESTCASE pTestcase = tstDevTestcaseFind(pTest->papszTestcaseIds[i]);
+            if (pTestcase)
+                rc = pTestcase->pTestcaseReg->pfnTestEntry(&Dut, pTest->papTestcaseCfg[i], pTest->pacTestcaseCfgItems[i]);
+            else
+                rc = VERR_NOT_FOUND;
+        }
+    }
+
+    return rc;
+}
+
+
 int main(int argc, char *argv[])
 {
     /*
@@ -894,21 +1000,40 @@ int main(int argc, char *argv[])
         RTListInit(&g_LstPlugins);
         RTListInit(&g_LstTestcases);
         RTListInit(&g_LstPdmMods);
+        RTListInit(&g_LstPdmR0Mods);
         RTListInit(&g_LstPdmDevs);
 
-        rc = tstDevLoadPlugin("TSTDevTestcases");
-        if (RT_SUCCESS(rc) || true)
+        /* Register builtin tests. */
+        tstDevRegisterTestcase(NULL, &g_TestcaseSsmFuzz);
+        tstDevRegisterTestcase(NULL, &g_TestcaseSsmLoadDbg);
+        tstDevRegisterTestcase(NULL, &g_TestcaseIoFuzz);
+
+        PCTSTDEVCFG pDevTstCfg = NULL;
+        rc = tstDevCfgLoad(argv[1], NULL, &pDevTstCfg);
+        if (RT_SUCCESS(rc))
         {
-            rc = tstDevPdmLoadMod(argv[1], TSTDEVPDMMODTYPE_R3);
+            if (pDevTstCfg->pszTstDevMod)
+                rc = tstDevLoadPlugin(pDevTstCfg->pszTstDevMod);
             if (RT_SUCCESS(rc))
-                rc = tstDevPdmDevCreate("nvme");
+            {
+                rc = tstDevPdmLoadMod(pDevTstCfg->pszPdmR3Mod, TSTDEVPDMMODTYPE_R3);
+                if (   RT_SUCCESS(rc)
+                    && pDevTstCfg->pszPdmR0Mod)
+                    rc = tstDevPdmLoadMod(pDevTstCfg->pszPdmR0Mod, TSTDEVPDMMODTYPE_R0);
+                if (   RT_SUCCESS(rc)
+                    && pDevTstCfg->pszPdmRCMod)
+                    rc = tstDevPdmLoadMod(pDevTstCfg->pszPdmRCMod, TSTDEVPDMMODTYPE_RC);
+
+                if (RT_SUCCESS(rc))
+                    rc = tstDevTestsRun(pDevTstCfg);
+                else
+                    rcExit = RTEXITCODE_FAILURE;
+            }
             else
                 rcExit = RTEXITCODE_FAILURE;
         }
-        else
-            rcExit = RTEXITCODE_FAILURE;
 
-        //rcExit = tstDevParseOptions(argc, argv);
+        tstDevCfgDestroy(pDevTstCfg);
     }
     else
         rcExit = RTEXITCODE_FAILURE;
