@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2012-2020 Oracle Corporation
+ * Copyright (C) 2012-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
@@ -28,18 +38,17 @@
 #include "UIIconPool.h"
 #include "UIVirtualBoxManager.h"
 
-/* Other VBox includes: */
-#include "iprt/cpp/utils.h"
-
 
 UIChooserItemGlobal::UIChooserItemGlobal(UIChooserItem *pParent, UIChooserNodeGlobal *pNode)
-    : UIChooserItem(pParent, pNode, 0, 100)
-    , m_iDefaultLightnessMin(0)
-    , m_iDefaultLightnessMax(0)
-    , m_iHoverLightnessMin(0)
-    , m_iHoverLightnessMax(0)
-    , m_iHighlightLightnessMin(0)
-    , m_iHighlightLightnessMax(0)
+    : UIChooserItem(pParent, pNode)
+#ifdef VBOX_WS_MAC
+    , m_iDefaultDarknessStart(0)
+    , m_iDefaultDarknessFinal(0)
+#endif
+    , m_iHoverLightnessStart(0)
+    , m_iHoverLightnessFinal(0)
+    , m_iHighlightLightnessStart(0)
+    , m_iHighlightLightnessFinal(0)
     , m_iMinimumNameWidth(0)
     , m_iMaximumNameWidth(0)
     , m_iHeightHint(0)
@@ -52,6 +61,11 @@ UIChooserItemGlobal::~UIChooserItemGlobal()
     cleanup();
 }
 
+UIChooserNodeGlobal *UIChooserItemGlobal::nodeToGlobalType() const
+{
+    return node() ? node()->toGlobalNode() : 0;
+}
+
 bool UIChooserItemGlobal::isToolButtonArea(const QPoint &position, int iMarginMultiplier /* = 1 */) const
 {
     const int iFullWidth = geometry().width();
@@ -189,7 +203,7 @@ void UIChooserItemGlobal::updateToolTip()
     // Nothing for now..
 }
 
-QList<UIChooserItem*> UIChooserItemGlobal::items(UIChooserItemType) const
+QList<UIChooserItem*> UIChooserItemGlobal::items(UIChooserNodeType) const
 {
     AssertMsgFailedReturn(("Global graphics item do NOT support children!"), QList<UIChooserItem*>());
 }
@@ -204,10 +218,10 @@ void UIChooserItemGlobal::removeItem(UIChooserItem *)
     AssertMsgFailed(("Global graphics item do NOT support children!"));
 }
 
-UIChooserItem *UIChooserItemGlobal::searchForItem(const QString &, int iItemSearchFlags)
+UIChooserItem *UIChooserItemGlobal::searchForItem(const QString &, int iSearchFlags)
 {
-    /* Ignoring if we are not searching for the global-item? */
-    if (!(iItemSearchFlags & UIChooserItemSearchFlag_Global))
+    /* Ignore if we are not searching for the global-item: */
+    if (!(iSearchFlags & UIChooserItemSearchFlag_Global))
         return 0;
 
     /* Returning this: */
@@ -324,22 +338,25 @@ void UIChooserItemGlobal::sltHandleWindowRemapped()
 
 void UIChooserItemGlobal::prepare()
 {
-    /* Colors: */
-#ifdef VBOX_WS_MAC
-    m_iHighlightLightnessMin = 105;
-    m_iHighlightLightnessMax = 115;
-    m_iHoverLightnessMin = 115;
-    m_iHoverLightnessMax = 125;
-    m_iDefaultLightnessMin = 125;
-    m_iDefaultLightnessMax = 130;
-#else /* VBOX_WS_MAC */
-    m_iHighlightLightnessMin = 130;
-    m_iHighlightLightnessMax = 160;
-    m_iHoverLightnessMin = 160;
-    m_iHoverLightnessMax = 190;
-    m_iDefaultLightnessMin = 160;
-    m_iDefaultLightnessMax = 190;
-#endif /* !VBOX_WS_MAC */
+    /* Color tones: */
+#if defined(VBOX_WS_MAC)
+    m_iDefaultDarknessStart = 105;
+    m_iDefaultDarknessFinal = 115;
+    m_iHoverLightnessStart = 125;
+    m_iHoverLightnessFinal = 115;
+    m_iHighlightLightnessStart = 115;
+    m_iHighlightLightnessFinal = 105;
+#elif defined(VBOX_WS_WIN)
+    m_iHoverLightnessStart = 220;
+    m_iHoverLightnessFinal = 210;
+    m_iHighlightLightnessStart = 190;
+    m_iHighlightLightnessFinal = 180;
+#else /* !VBOX_WS_MAC && !VBOX_WS_WIN */
+    m_iHoverLightnessStart = 125;
+    m_iHoverLightnessFinal = 115;
+    m_iHighlightLightnessStart = 110;
+    m_iHighlightLightnessFinal = 100;
+#endif /* !VBOX_WS_MAC && !VBOX_WS_WIN */
 
     /* Fonts: */
     m_nameFont = font();
@@ -491,7 +508,12 @@ void UIChooserItemGlobal::updateMinimumNameWidth()
     /* Calculate new minimum name width: */
     QPaintDevice *pPaintDevice = model()->paintDevice();
     const QFontMetrics fm(m_nameFont, pPaintDevice);
+#if QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
+    const int iMinimumNameWidth = fm.horizontalAdvance(compressText(m_nameFont, pPaintDevice, name(),
+                                                                    textWidth(m_nameFont, pPaintDevice, 15)));
+#else
     const int iMinimumNameWidth = fm.width(compressText(m_nameFont, pPaintDevice, name(), textWidth(m_nameFont, pPaintDevice, 15)));
+#endif
 
     /* Is there something changed? */
     if (m_iMinimumNameWidth == iMinimumNameWidth)
@@ -547,23 +569,23 @@ void UIChooserItemGlobal::updateVisibleName()
     }
 }
 
-void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &rectangle)
 {
     /* Save painter: */
     pPainter->save();
 
     /* Prepare color: */
-    const QPalette pal = palette();
+    const QPalette pal = QApplication::palette();
 
     /* Selected-item background: */
-    if (model()->selectedItems().contains(unconst(this)))
+    if (model()->selectedItems().contains(this))
     {
         /* Prepare color: */
         const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Highlight);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHighlightLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHighlightLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHighlightLightnessStart));
+        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHighlightLightnessFinal));
         pPainter->fillRect(rectangle, bgGrad);
 
         if (isHovered())
@@ -599,8 +621,8 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
         const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Highlight);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHoverLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHoverLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.lighter(m_iHoverLightnessStart));
+        bgGrad.setColorAt(1, backgroundColor.lighter(m_iHoverLightnessFinal));
         pPainter->fillRect(rectangle, bgGrad);
 
         /* Prepare color: */
@@ -631,17 +653,15 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
     {
 #ifdef VBOX_WS_MAC
         /* Prepare color: */
-        const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Mid);
+        const QColor backgroundColor = pal.color(QPalette::Active, QPalette::Window);
         /* Draw gradient: */
         QLinearGradient bgGrad(rectangle.topLeft(), rectangle.bottomLeft());
-        bgGrad.setColorAt(0, backgroundColor.lighter(m_iDefaultLightnessMax));
-        bgGrad.setColorAt(1, backgroundColor.lighter(m_iDefaultLightnessMin));
+        bgGrad.setColorAt(0, backgroundColor.darker(m_iDefaultDarknessStart));
+        bgGrad.setColorAt(1, backgroundColor.darker(m_iDefaultDarknessFinal));
         pPainter->fillRect(rectangle, bgGrad);
 #else
-        /* Prepare color: */
-        QColor backgroundColor = pal.color(QPalette::Active, QPalette::Mid).lighter(160);
-        /* Draw gradient: */
-        pPainter->fillRect(rectangle, backgroundColor);
+        /* Draw simple background: */
+        pPainter->fillRect(rectangle, pal.color(QPalette::Active, QPalette::Window));
 #endif
     }
 
@@ -649,28 +669,25 @@ void UIChooserItemGlobal::paintBackground(QPainter *pPainter, const QRect &recta
     pPainter->restore();
 }
 
-void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle)
 {
     /* Only selected and/or hovered item should have a frame: */
-    if (!model()->selectedItems().contains(unconst(this)) && !isHovered())
+    if (!model()->selectedItems().contains(this) && !isHovered())
         return;
 
     /* Save painter: */
     pPainter->save();
 
     /* Prepare color: */
-    const QPalette pal = palette();
+    const QPalette pal = QApplication::palette();
     QColor strokeColor;
 
     /* Selected-item frame: */
-    if (model()->selectedItems().contains(unconst(this)))
-        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHighlightLightnessMin - 40);
+    if (model()->selectedItems().contains(this))
+        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHighlightLightnessStart - 40);
     /* Hovered-item frame: */
     else if (isHovered())
-        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHoverLightnessMin - 50);
-    /* Default frame: */
-    else
-        strokeColor = pal.color(QPalette::Active, QPalette::Mid).lighter(m_iDefaultLightnessMin);
+        strokeColor = pal.color(QPalette::Active, QPalette::Highlight).lighter(m_iHoverLightnessStart - 40);
 
     /* Create/assign pen: */
     QPen pen(strokeColor);
@@ -686,7 +703,7 @@ void UIChooserItemGlobal::paintFrame(QPainter *pPainter, const QRect &rectangle)
     pPainter->restore();
 }
 
-void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &rectangle) const
+void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &rectangle)
 {
     /* Prepare variables: */
     const int iFullWidth = rectangle.width();
@@ -696,23 +713,35 @@ void UIChooserItemGlobal::paintGlobalInfo(QPainter *pPainter, const QRect &recta
     const int iSpacing = data(GlobalItemData_Spacing).toInt();
     const int iButtonMargin = data(GlobalItemData_ButtonMargin).toInt();
 
-    /* Selected-item foreground: */
-    if (model()->selectedItems().contains(unconst(this)))
+    /* Selected or hovered item foreground: */
+    if (model()->selectedItems().contains(this) || isHovered())
     {
-        const QPalette pal = palette();
-        pPainter->setPen(pal.color(QPalette::HighlightedText));
-    }
-    /* Hovered item foreground: */
-    else if (isHovered())
-    {
-        /* Prepare color: */
-        const QPalette pal = palette();
+        /* Prepare palette: */
+        const QPalette pal = QApplication::palette();
+
+        /* Get background color: */
         const QColor highlight = pal.color(QPalette::Active, QPalette::Highlight);
-        const QColor hhl = highlight.lighter(m_iHoverLightnessMax);
-        if (hhl.value() - hhl.saturation() > 0)
-            pPainter->setPen(pal.color(QPalette::Active, QPalette::Text));
+        const QColor background = model()->selectedItems().contains(this)
+                                ? highlight.lighter(m_iHighlightLightnessStart)
+                                : highlight.lighter(m_iHoverLightnessStart);
+
+        /* Get foreground color: */
+        const QColor simpleText = pal.color(QPalette::Active, QPalette::Text);
+        const QColor highlightText = pal.color(QPalette::Active, QPalette::HighlightedText);
+        QColor lightText = simpleText.black() < highlightText.black() ? simpleText : highlightText;
+        QColor darkText = simpleText.black() > highlightText.black() ? simpleText : highlightText;
+        if (lightText.black() > 128)
+            lightText = QColor(Qt::white);
+        if (darkText.black() < 128)
+            darkText = QColor(Qt::black);
+
+        /* Gather foreground color for background one: */
+        double dLuminance = (0.299 * background.red() + 0.587 * background.green() + 0.114 * background.blue()) / 255;
+        //printf("luminance = %f\n", dLuminance);
+        if (dLuminance > 0.5)
+            pPainter->setPen(darkText);
         else
-            pPainter->setPen(pal.color(QPalette::Active, QPalette::HighlightedText));
+            pPainter->setPen(lightText);
     }
 
     /* Calculate indents: */
