@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2013-2020 Oracle Corporation
+ * Copyright (C) 2013-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
@@ -20,8 +30,9 @@
 
 /* GUI includes: */
 #include "UICommon.h"
+#include "UIErrorString.h"
 #include "UIMediumEnumerator.h"
-#include "UIMessageCenter.h"
+#include "UINotificationCenter.h"
 #include "UITask.h"
 #include "UIThreadPool.h"
 #include "UIVirtualBoxEventHandler.h"
@@ -77,7 +88,7 @@ public:
 private:
 
     /** Contains medium-enumeration task body. */
-    virtual void run() /* override */
+    virtual void run() RT_OVERRIDE
     {
         /* Enumerate under a proper lock: */
         m_mutex.lock();
@@ -122,6 +133,13 @@ UIMediumEnumerator::UIMediumEnumerator()
     /* Prepare global thread-pool listener: */
     connect(uiCommon().threadPool(), &UIThreadPool::sigTaskComplete,
             this, &UIMediumEnumerator::sltHandleMediumEnumerationTaskComplete);
+
+    /* We should make sure media map contains at least NULL medium object: */
+    addNullMediumToMap(m_media);
+    /* Notify listener about initial enumeration started/finished instantly: */
+    LogRel(("GUI: UIMediumEnumerator: Initial medium-enumeration finished!\n"));
+    emit sigMediumEnumerationStarted();
+    emit sigMediumEnumerationFinished();
 }
 
 QList<QUuid> UIMediumEnumerator::mediumIDs() const
@@ -148,7 +166,8 @@ void UIMediumEnumerator::createMedium(const UIMedium &guiMedium)
     /* Do not create UIMedium(s) with incorrect ID: */
     AssertReturnVoid(!uMediumID.isNull());
     /* Make sure UIMedium doesn't exist already: */
-    AssertReturnVoid(!m_media.contains(uMediumID));
+    if (m_media.contains(uMediumID))
+        return;
 
     /* Insert UIMedium: */
     m_media[uMediumID] = guiMedium;
@@ -180,7 +199,7 @@ void UIMediumEnumerator::enumerateMedia(const CMediumVector &comMedia /* = CMedi
     }
 
     /* UICommon is cleaning up, abort immediately: */
-    if (UICommon::isCleaningUp())
+    if (uiCommon().isCleaningUp())
         return;
 
     if (comMedia.isEmpty())
@@ -326,10 +345,9 @@ void UIMediumEnumerator::sltHandleMediumRegistered(const QUuid &uMediumId, KDevi
             CVirtualBox comVBox = uiCommon().virtualBox();
             /* Open existing medium, this API can be used to open known medium as well, using ID as location for that: */
             CMedium comMedium = comVBox.OpenMedium(uMediumId.toString(), enmMediumType, KAccessMode_ReadWrite, false);
-
-            /* Show error message if necessary: */
             if (!comVBox.isOk())
-                msgCenter().cannotOpenKnownMedium(comVBox, uMediumId);
+                LogRel(("GUI: UIMediumEnumerator:  Unable to open registered medium! %s\n",
+                        UIErrorString::simplifiedErrorInfo(comVBox).toUtf8().constData()));
             else
             {
                 /* Create new UIMedium: */
@@ -487,7 +505,7 @@ void UIMediumEnumerator::addMediaToMap(const CMediumVector &inputMedia, UIMedium
     foreach (const CMedium &comMedium, inputMedia)
     {
         /* If UICommon is cleaning up, abort immediately: */
-        if (UICommon::isCleaningUp())
+        if (uiCommon().isCleaningUp())
             break;
 
         /* Insert UIMedium to the passed media map.
@@ -518,10 +536,8 @@ void UIMediumEnumerator::parseAttachment(CMediumAttachment comAttachment, QList<
         /* Acquire attachment medium: */
         CMedium comMedium = comAttachment.GetMedium();
         if (!comAttachment.isOk())
-        {
-            LogRel2(("GUI: UIMediumEnumerator:  Unable to acquire attachment medium!\n"));
-            msgCenter().cannotAcquireAttachmentParameter(comAttachment);
-        }
+            LogRel(("GUI: UIMediumEnumerator:  Unable to acquire attachment medium! %s\n",
+                    UIErrorString::simplifiedErrorInfo(comAttachment).toUtf8().constData()));
         else
         {
             /* Parse medium: */
@@ -535,19 +551,15 @@ void UIMediumEnumerator::parseAttachment(CMediumAttachment comAttachment, QList<
             /* Acquire parent VM: */
             CMachine comMachine = comAttachment.GetMachine();
             if (!comAttachment.isOk())
-            {
-                LogRel2(("GUI: UIMediumEnumerator:  Unable to acquire attachment parent machine!\n"));
-                msgCenter().cannotAcquireAttachmentParameter(comAttachment);
-            }
+                LogRel(("GUI: UIMediumEnumerator:  Unable to acquire attachment parent machine! %s\n",
+                        UIErrorString::simplifiedErrorInfo(comAttachment).toUtf8().constData()));
             else
             {
                 /* Acquire machine ID: */
                 const QUuid uMachineId = comMachine.GetId();
                 if (!comMachine.isOk())
-                {
-                    LogRel2(("GUI: UIMediumEnumerator:  Unable to acquire machine ID!\n"));
-                    msgCenter().cannotAcquireMachineParameter(comMachine);
-                }
+                    LogRel(("GUI: UIMediumEnumerator:  Unable to acquire machine ID! %s\n",
+                            UIErrorString::simplifiedErrorInfo(comMachine).toUtf8().constData()));
                 else
                 {
                     /* Enumerate all the media of machine with this ID: */
@@ -572,10 +584,8 @@ void UIMediumEnumerator::parseMedium(CMedium comMedium, QList<QUuid> &result)
         /* Acquire medium ID: */
         const QUuid uMediumId = comMedium.GetId();
         if (!comMedium.isOk())
-        {
-            LogRel2(("GUI: UIMediumEnumerator:  Unable to acquire medium ID!\n"));
-            msgCenter().cannotAcquireMediumAttribute(comMedium);
-        }
+            LogRel(("GUI: UIMediumEnumerator:  Unable to acquire medium ID! %s\n",
+                    UIErrorString::simplifiedErrorInfo(comMedium).toUtf8().constData()));
         else
         {
             //printf(" Medium to recache: %s\n", uMediumId.toString().toUtf8().constData());
