@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2009-2020 Oracle Corporation
+ * Copyright (C) 2009-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /* Qt includes: */
@@ -22,6 +32,7 @@
 #include <QHeaderView>
 #include <QLabel>
 #include <QLineEdit>
+#include <QRegExp>
 #include <QSpinBox>
 #include <QTextEdit>
 #include <QVBoxLayout>
@@ -36,6 +47,7 @@
 #include "UIIconPool.h"
 #include "UILineTextEdit.h"
 #include "UIMessageCenter.h"
+#include "UITranslator.h"
 
 /* COM includes: */
 #include "CSystemProperties.h"
@@ -65,21 +77,21 @@ public:
     /** Appends the passed @a pChildItem to the item's list of children. */
     void appendChild(UIApplianceModelItem *pChildItem);
     /** Returns the child specified by the @a iIndex. */
-    virtual UIApplianceModelItem *childItem(int iIndex) const /* override */;
+    virtual UIApplianceModelItem *childItem(int iIndex) const RT_OVERRIDE;
 
     /** Returns the row of the item in the parent. */
     int row() const;
 
     /** Returns the number of children. */
-    virtual int childCount() const /* override */;
+    virtual int childCount() const RT_OVERRIDE;
     /** Returns the number of columns. */
     int columnCount() const { return 3; }
 
     /** Returns the item text. */
-    virtual QString text() const /* override */;
+    virtual QString text() const RT_OVERRIDE;
 
     /** Returns the item flags for the given @a iColumn. */
-    virtual Qt::ItemFlags itemFlags(int /* iColumn */) const { return 0; }
+    virtual Qt::ItemFlags itemFlags(int /* iColumn */) const { return Qt::ItemFlags(); }
 
     /** Defines the @a iRole data for the item at @a iColumn to @a value. */
     virtual bool setData(int /* iColumn */, const QVariant & /* value */, int /* iRole */) { return false; }
@@ -126,10 +138,10 @@ public:
     UIVirtualSystemItem(int iNumber, CVirtualSystemDescription comDescription, UIApplianceModelItem *pParentItem);
 
     /** Returns the data stored under the given @a iRole for the item referred to by the @a iColumn. */
-    virtual QVariant data(int iColumn, int iRole) const /* override */;
+    virtual QVariant data(int iColumn, int iRole) const RT_OVERRIDE;
 
     /** Cache currently stored values, such as @a finalStates, @a finalValues and @a finalExtraValues. */
-    virtual void putBack(QVector<BOOL> &finalStates, QVector<QString> &finalValues, QVector<QString> &finalExtraValues) /* override */;
+    virtual void putBack(QVector<BOOL> &finalStates, QVector<QString> &finalValues, QVector<QString> &finalExtraValues) RT_OVERRIDE;
 
 private:
 
@@ -169,28 +181,28 @@ public:
                           UIApplianceModelItem *pParentItem);
 
     /** Returns the item flags for the given @a iColumn. */
-    virtual Qt::ItemFlags itemFlags(int iColumn) const /* override */;
+    virtual Qt::ItemFlags itemFlags(int iColumn) const RT_OVERRIDE;
 
     /** Defines the @a iRole data for the item at @a iColumn to @a value. */
-    virtual bool setData(int iColumn, const QVariant &value, int iRole) /* override */;
+    virtual bool setData(int iColumn, const QVariant &value, int iRole) RT_OVERRIDE;
     /** Returns the data stored under the given @a iRole for the item referred to by the @a iColumn. */
-    virtual QVariant data(int iColumn, int iRole) const /* override */;
+    virtual QVariant data(int iColumn, int iRole) const RT_OVERRIDE;
 
     /** Returns the widget used to edit the item specified by @a idx for editing.
       * @param  pParent      Brings the parent to be assigned for newly created editor.
       * @param  styleOption  Bring the style option set for the newly created editor. */
-    virtual QWidget *createEditor(QWidget *pParent, const QStyleOptionViewItem &styleOption, const QModelIndex &idx) const /* override */;
+    virtual QWidget *createEditor(QWidget *pParent, const QStyleOptionViewItem &styleOption, const QModelIndex &idx) const RT_OVERRIDE;
 
     /** Defines the contents of the given @a pEditor to the data for the item at the given @a idx. */
-    virtual bool setEditorData(QWidget *pEditor, const QModelIndex &idx) const /* override */;
+    virtual bool setEditorData(QWidget *pEditor, const QModelIndex &idx) const RT_OVERRIDE;
     /** Defines the data for the item at the given @a idx in the @a pModel to the contents of the given @a pEditor. */
-    virtual bool setModelData(QWidget *pEditor, QAbstractItemModel *pModel, const QModelIndex &idx) /* override */;
+    virtual bool setModelData(QWidget *pEditor, QAbstractItemModel *pModel, const QModelIndex &idx) RT_OVERRIDE;
 
     /** Restores the default values. */
-    virtual void restoreDefaults() /* override */;
+    virtual void restoreDefaults() RT_OVERRIDE;
 
     /** Cache currently stored values, such as @a finalStates, @a finalValues and @a finalExtraValues. */
-    virtual void putBack(QVector<BOOL> &finalStates, QVector<QString> &finalValues, QVector<QString> &finalExtraValues) /* override */;
+    virtual void putBack(QVector<BOOL> &finalStates, QVector<QString> &finalValues, QVector<QString> &finalExtraValues) RT_OVERRIDE;
 
     KVirtualSystemDescriptionType  systemDescriptionType() const;
 
@@ -314,6 +326,7 @@ void UIVirtualSystemItem::putBack(QVector<BOOL> &finalStates, QVector<QString> &
 {
     /* Resize the vectors */
     unsigned long iCount = m_comDescription.GetCount();
+    AssertReturnVoid(iCount > 0);
     finalStates.resize(iCount);
     finalValues.resize(iCount);
     finalExtraValues.resize(iCount);
@@ -340,10 +353,10 @@ UIVirtualHardwareItem::UIVirtualHardwareItem(UIApplianceModel *pParent,
     , m_pParent(pParent)
     , m_enmVSDType(enmVSDType)
     , m_strRef(strRef)
-    , m_strOrigValue(strOrigValue)
-    , m_strConfigValue(strConfigValue)
+    , m_strOrigValue(enmVSDType == KVirtualSystemDescriptionType_Memory ? UITranslator::byteStringToMegaByteString(strOrigValue) : strOrigValue)
+    , m_strConfigValue(enmVSDType == KVirtualSystemDescriptionType_Memory ? UITranslator::byteStringToMegaByteString(strConfigValue) : strConfigValue)
     , m_strConfigDefaultValue(strConfigValue)
-    , m_strExtraConfigValue(strExtraConfigValue)
+    , m_strExtraConfigValue(enmVSDType == KVirtualSystemDescriptionType_Memory ? UITranslator::byteStringToMegaByteString(strExtraConfigValue) : strExtraConfigValue)
     , m_checkState(Qt::Checked)
     , m_fModified(false)
 {
@@ -351,7 +364,7 @@ UIVirtualHardwareItem::UIVirtualHardwareItem(UIApplianceModel *pParent,
 
 Qt::ItemFlags UIVirtualHardwareItem::itemFlags(int iColumn) const
 {
-    Qt::ItemFlags enmFlags = 0;
+    Qt::ItemFlags enmFlags = Qt::ItemFlags();
     if (iColumn == ApplianceViewSection_ConfigValue)
     {
         /* Some items are checkable */
@@ -481,7 +494,9 @@ QVariant UIVirtualHardwareItem::data(int iColumn, int iRole) const
                     case KVirtualSystemDescriptionType_HardDiskControllerIDE:  value = UIApplianceEditorWidget::tr("Storage Controller (IDE)"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSATA: value = UIApplianceEditorWidget::tr("Storage Controller (SATA)"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSCSI: value = UIApplianceEditorWidget::tr("Storage Controller (SCSI)"); break;
+                    case KVirtualSystemDescriptionType_HardDiskControllerVirtioSCSI:  value = UIApplianceEditorWidget::tr("Storage Controller (VirtioSCSI)"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSAS:  value = UIApplianceEditorWidget::tr("Storage Controller (SAS)"); break;
+                    case KVirtualSystemDescriptionType_HardDiskControllerNVMe:  value = UIApplianceEditorWidget::tr("Storage Controller (NVMe)"); break;
                     case KVirtualSystemDescriptionType_CDROM:                  value = UIApplianceEditorWidget::tr("DVD"); break;
                     case KVirtualSystemDescriptionType_Floppy:                 value = UIApplianceEditorWidget::tr("Floppy"); break;
                     case KVirtualSystemDescriptionType_NetworkAdapter:         value = UIApplianceEditorWidget::tr("Network Adapter"); break;
@@ -655,14 +670,16 @@ QVariant UIVirtualHardwareItem::data(int iColumn, int iRole) const
                     case KVirtualSystemDescriptionType_HardDiskControllerIDE:  value = UIIconPool::iconSet(":/ide_16px.png"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSATA: value = UIIconPool::iconSet(":/sata_16px.png"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSCSI: value = UIIconPool::iconSet(":/scsi_16px.png"); break;
+                    case KVirtualSystemDescriptionType_HardDiskControllerVirtioSCSI:  value = UIIconPool::iconSet(":/virtio_scsi_16px.png"); break;
                     case KVirtualSystemDescriptionType_HardDiskControllerSAS:  value = UIIconPool::iconSet(":/sas_16px.png"); break;
+                    case KVirtualSystemDescriptionType_HardDiskControllerNVMe:  value = UIIconPool::iconSet(":/pcie_16px.png"); break;
                     case KVirtualSystemDescriptionType_HardDiskImage:          value = UIIconPool::iconSet(":/hd_16px.png"); break;
                     case KVirtualSystemDescriptionType_CDROM:                  value = UIIconPool::iconSet(":/cd_16px.png"); break;
                     case KVirtualSystemDescriptionType_Floppy:                 value = UIIconPool::iconSet(":/fd_16px.png"); break;
                     case KVirtualSystemDescriptionType_NetworkAdapter:         value = UIIconPool::iconSet(":/nw_16px.png"); break;
                     case KVirtualSystemDescriptionType_USBController:          value = UIIconPool::iconSet(":/usb_16px.png"); break;
                     case KVirtualSystemDescriptionType_SoundCard:              value = UIIconPool::iconSet(":/sound_16px.png"); break;
-                    case KVirtualSystemDescriptionType_BaseFolder:             value = uiCommon().icon(QFileIconProvider::Folder); break;
+                    case KVirtualSystemDescriptionType_BaseFolder:             value = generalIconPool().defaultSystemIcon(QFileIconProvider::Folder); break;
                     case KVirtualSystemDescriptionType_PrimaryGroup:           value = UIIconPool::iconSet(":/vm_group_name_16px.png"); break;
                     case KVirtualSystemDescriptionType_CloudProfileName:
                     case KVirtualSystemDescriptionType_CloudInstanceShape:
@@ -678,7 +695,7 @@ QVariant UIVirtualHardwareItem::data(int iColumn, int iRole) const
                 }
             }
             else if (iColumn == ApplianceViewSection_ConfigValue && m_enmVSDType == KVirtualSystemDescriptionType_OS)
-                value = uiCommon().vmGuestOSTypeIcon(m_strConfigValue);
+                value = generalIconPool().guestOSTypeIcon(m_strConfigValue);
             break;
         }
         case Qt::FontRole:
@@ -1287,8 +1304,10 @@ void UIVirtualHardwareItem::restoreDefaults()
 void UIVirtualHardwareItem::putBack(QVector<BOOL> &finalStates, QVector<QString> &finalValues, QVector<QString> &finalExtraValues)
 {
     finalStates[m_iNumber] = m_checkState == Qt::Checked;
-    finalValues[m_iNumber] = m_strConfigValue;
-    finalExtraValues[m_iNumber] = m_strExtraConfigValue;
+    /* It's always stored in bytes in VSD according to the old internal agreement within the team */
+    finalValues[m_iNumber] = m_enmVSDType == KVirtualSystemDescriptionType_Memory ? UITranslator::megabyteStringToByteString(m_strConfigValue) : m_strConfigValue;
+    finalExtraValues[m_iNumber] = m_enmVSDType == KVirtualSystemDescriptionType_Memory ? UITranslator::megabyteStringToByteString(m_strExtraConfigValue) : m_strExtraConfigValue;
+
     UIApplianceModelItem::putBack(finalStates, finalValues, finalExtraValues);
 }
 
@@ -1340,7 +1359,9 @@ UIApplianceModel::UIApplianceModel(QVector<CVirtualSystemDescription>& aVSDs, QI
                 if (types[i] == KVirtualSystemDescriptionType_HardDiskControllerIDE ||
                     types[i] == KVirtualSystemDescriptionType_HardDiskControllerSATA ||
                     types[i] == KVirtualSystemDescriptionType_HardDiskControllerSCSI ||
-                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerSAS)
+                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerVirtioSCSI ||
+                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerSAS ||
+                    types[i] == KVirtualSystemDescriptionType_HardDiskControllerNVMe)
                     controllerMap[i] = pHardwareItem;
             }
         }
@@ -1416,7 +1437,7 @@ int UIApplianceModel::columnCount(const QModelIndex &parentIdx /* = QModelIndex(
 Qt::ItemFlags UIApplianceModel::flags(const QModelIndex &idx) const
 {
     if (!idx.isValid())
-        return 0;
+        return Qt::ItemFlags();
 
     UIApplianceModelItem *pItem = static_cast<UIApplianceModelItem*>(idx.internalPointer());
 
@@ -1557,8 +1578,8 @@ QVariant UIApplianceModel::getHint(KVirtualSystemDescriptionType enmType) const
 *   Class UIApplianceDelegate implementation.                                                                                    *
 *********************************************************************************************************************************/
 
-UIApplianceDelegate::UIApplianceDelegate(QAbstractProxyModel *pProxy, QObject *pParent /* = 0 */)
-    : QItemDelegate(pParent)
+UIApplianceDelegate::UIApplianceDelegate(QAbstractProxyModel *pProxy)
+    : QItemDelegate(pProxy)
     , m_pProxy(pProxy)
 {
 }
@@ -1680,7 +1701,9 @@ KVirtualSystemDescriptionType UIApplianceSortProxyModel::s_aSortList[] =
     KVirtualSystemDescriptionType_HardDiskControllerIDE,
     KVirtualSystemDescriptionType_HardDiskControllerSATA,
     KVirtualSystemDescriptionType_HardDiskControllerSCSI,
+    KVirtualSystemDescriptionType_HardDiskControllerVirtioSCSI,
     KVirtualSystemDescriptionType_HardDiskControllerSAS,
+    KVirtualSystemDescriptionType_HardDiskControllerNVMe,
     /* OCI */
     KVirtualSystemDescriptionType_CloudProfileName,
     KVirtualSystemDescriptionType_CloudBucket,
@@ -1704,7 +1727,7 @@ bool UIApplianceSortProxyModel::filterAcceptsRow(int iSourceRow, const QModelInd
     /* By default enable all, we will explicitly filter out below */
     if (srcParenIdx.isValid())
     {
-        QModelIndex i = srcParenIdx.child(iSourceRow, 0);
+        QModelIndex i = index(iSourceRow, 0, srcParenIdx);
         if (i.isValid())
         {
             UIApplianceModelItem *pItem = static_cast<UIApplianceModelItem*>(i.internalPointer());
@@ -1763,7 +1786,6 @@ int UIApplianceEditorWidget::m_maxGuestCPUCount = -1;
 
 UIApplianceEditorWidget::UIApplianceEditorWidget(QWidget *pParent /* = 0 */)
     : QIWithRetranslateUI<QWidget>(pParent)
-    , m_pAppliance(0)
     , m_pModel(0)
 {
     /* Make sure all static content is properly initialized */
@@ -1848,6 +1870,21 @@ UIApplianceEditorWidget::UIApplianceEditorWidget(QWidget *pParent /* = 0 */)
     retranslateUi();
 }
 
+void UIApplianceEditorWidget::clear()
+{
+    /* Wipe model: */
+    delete m_pModel;
+    m_pModel = 0;
+
+    /* And appliance: */
+    m_comAppliance = CAppliance();
+}
+
+void UIApplianceEditorWidget::setAppliance(const CAppliance &comAppliance)
+{
+    m_comAppliance = comAppliance;
+}
+
 void UIApplianceEditorWidget::setVsdHints(const AbstractVSDParameterList &hints)
 {
     /* Save here as well: */
@@ -1858,6 +1895,13 @@ void UIApplianceEditorWidget::setVsdHints(const AbstractVSDParameterList &hints)
         m_pModel->setVsdHints(m_listVsdHints);
 }
 
+void UIApplianceEditorWidget::setVirtualSystemBaseFolder(const QString &strPath)
+{
+    /* Make sure model exists, it's being created in sub-classes: */
+    if (m_pModel)
+        m_pModel->setVirtualSystemBaseFolder(strPath);
+}
+
 void UIApplianceEditorWidget::restoreDefaults()
 {
     /* Make sure model exists, it's being created in sub-classes: */
@@ -1889,12 +1933,5 @@ void UIApplianceEditorWidget::initSystemSettings()
     }
 }
 
-void UIApplianceEditorWidget::setVirtualSystemBaseFolder(const QString& path)
-{
-    if (!m_pModel)
-        return;
-    m_pModel->setVirtualSystemBaseFolder(path);
-}
-
 
 #include "UIApplianceEditorWidget.moc"
