@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef MAIN_INCLUDED_SessionImpl_h
@@ -28,7 +38,7 @@
 # include "win/resource.h"
 #endif
 
-#ifdef RT_OS_WINDOWS
+#if defined(RT_OS_WINDOWS) && !RT_MSC_PREREQ(RT_MSC_VER_VC140)
 [threading(free)]
 #endif
 class ATL_NO_VTABLE Session :
@@ -46,7 +56,7 @@ public:
 
     DECLARE_NOT_AGGREGATABLE(Session)
 
-    DECLARE_EMPTY_CTOR_DTOR(Session)
+    DECLARE_COMMON_CLASS_METHODS(Session)
 
     HRESULT FinalConstruct();
     void FinalRelease();
@@ -90,6 +100,9 @@ private:
     HRESULT onNetworkAdapterChange(const ComPtr<INetworkAdapter> &aNetworkAdapter,
                                    BOOL aChangeAdapter);
     HRESULT onAudioAdapterChange(const ComPtr<IAudioAdapter> &aAudioAdapter);
+    HRESULT onHostAudioDeviceChange(const ComPtr<IHostAudioDevice> &aDevice,
+                                    BOOL aNew, AudioDeviceState_T aState,
+                                    const ComPtr<IVirtualBoxErrorInfo> &aErrInfo);
     HRESULT onSerialPortChange(const ComPtr<ISerialPort> &aSerialPort);
     HRESULT onParallelPortChange(const ComPtr<IParallelPort> &aParallelPort);
     HRESULT onStorageControllerChange(const Guid &aMachineId, const com::Utf8Str& aControllerName);
@@ -109,6 +122,7 @@ private:
     HRESULT onRecordingChange(BOOL aEnable);
     HRESULT onUSBControllerChange();
     HRESULT onSharedFolderChange(BOOL aGlobal);
+    HRESULT onGuestDebugControlChange(const ComPtr<IGuestDebugControl> &aGuestDebugControl);
     HRESULT onUSBDeviceAttach(const ComPtr<IUSBDevice> &aDevice,
                               const ComPtr<IVirtualBoxErrorInfo> &aError,
                               ULONG aMaskedInterfaces,
