@@ -1,28 +1,41 @@
 /* $Id: AutoCaller.cpp $ */
-
 /** @file
- *
  * VirtualBox object state implementation
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN
 #include <iprt/semaphore.h>
 
 #include "VirtualBoxBase.h"
 #include "AutoCaller.h"
-#include "Logging.h"
+#include "LoggingNew.h"
 
+#include "VBoxNls.h"
+
+
+DECLARE_TRANSLATION_CONTEXT(AutoCallerCtx);
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -130,13 +143,13 @@ HRESULT ObjectState::addCaller(bool aLimited /* = false */)
 {
     AutoWriteLock stateLock(mStateLock COMMA_LOCKVAL_SRC_POS);
 
-    HRESULT rc = E_ACCESSDENIED;
+    HRESULT hrc = E_ACCESSDENIED;
 
     if (mState == Ready || (aLimited && mState == Limited))
     {
         /* if Ready or allows Limited, increase the number of callers */
         ++mCallers;
-        rc = S_OK;
+        hrc = S_OK;
     }
     else
     if (mState == InInit || mState == InUninit)
@@ -145,7 +158,7 @@ HRESULT ObjectState::addCaller(bool aLimited /* = false */)
         {
             /* Called from the same thread that is doing AutoInitSpan or
              * AutoUninitSpan, just succeed */
-            rc = S_OK;
+            hrc = S_OK;
         }
         else if (mState == InInit)
         {
@@ -184,7 +197,7 @@ HRESULT ObjectState::addCaller(bool aLimited /* = false */)
             }
 
             if (mState == Ready || (aLimited && mState == Limited))
-                rc = S_OK;
+                hrc = S_OK;
             else
             {
                 Assert(mCallers != 0);
@@ -198,22 +211,22 @@ HRESULT ObjectState::addCaller(bool aLimited /* = false */)
         }
     }
 
-    if (FAILED(rc))
+    if (FAILED(hrc))
     {
         if (mState == Limited)
-            rc = mObj->setError(rc, "The object functionality is limited");
+            hrc = mObj->setError(hrc, AutoCallerCtx::tr("The object functionality is limited"));
         else if (FAILED(mFailedRC) && mFailedRC != E_ACCESSDENIED)
         {
             /* replay recorded error information */
             if (mpFailedEI)
                 ErrorInfoKeeper eik(*mpFailedEI);
-            rc = mFailedRC;
+            hrc = mFailedRC;
         }
         else
-            rc = mObj->setError(rc, "The object is not ready");
+            hrc = mObj->setError(hrc, AutoCallerCtx::tr("The object is not ready"));
     }
 
-    return rc;
+    return hrc;
 }
 
 /**
@@ -294,12 +307,13 @@ void ObjectState::autoInitSpanDestructor(State aNewState, HRESULT aFailedRC, com
         RTSemEventMultiSignal(mInitUninitSem);
     }
 
-    if (aNewState == InitFailed)
+    if (aNewState == InitFailed || aNewState == Limited)
     {
         mFailedRC = aFailedRC;
-        /* apFailedEI may be NULL, when there is no explicit setFailed() call,
-         * which also implies that aFailedRC is S_OK. This case is used by
-         * objects (the majority) which don't want delayed error signalling. */
+        /* apFailedEI may be NULL, when there is no explicit setFailed() or
+         * setLimited() call, which also implies that aFailedRC is S_OK.
+         * This case is used by objects (the majority) which don't want
+         * delayed error signalling. */
         mpFailedEI = apFailedEI;
     }
     else
@@ -312,7 +326,7 @@ void ObjectState::autoInitSpanDestructor(State aNewState, HRESULT aFailedRC, com
     setState(aNewState);
 }
 
-ObjectState::State ObjectState::autoUninitSpanConstructor()
+ObjectState::State ObjectState::autoUninitSpanConstructor(bool fTry)
 {
     AutoWriteLock stateLock(mStateLock COMMA_LOCKVAL_SRC_POS);
 
@@ -329,6 +343,9 @@ ObjectState::State ObjectState::autoUninitSpanConstructor()
          * completion. This is necessary to make sure that when this method
          * returns, the object state is well-defined (NotReady). */
 
+        if (fTry)
+            return Ready;
+
         /* lazy semaphore creation */
         if (mInitUninitSem == NIL_RTSEMEVENTMULTI)
         {
@@ -360,6 +377,9 @@ ObjectState::State ObjectState::autoUninitSpanConstructor()
     /* wait for already existing callers to drop to zero */
     if (mCallers > 0)
     {
+        if (fTry)
+            return Ready;
+
         /* lazy creation */
         Assert(mZeroCallersSem == NIL_RTSEMEVENT);
         RTSemEventCreate(&mZeroCallersSem);
@@ -517,19 +537,24 @@ AutoReinitSpan::~AutoReinitSpan()
  *
  * @param aObj  |this| pointer of the VirtualBoxBase object whose uninit()
  *              method is being called.
+ * @param fTry  @c true if the wait for other callers should be skipped,
+ *              requiring checking if the uninit span is actually operational.
  */
-AutoUninitSpan::AutoUninitSpan(VirtualBoxBase *aObj)
+AutoUninitSpan::AutoUninitSpan(VirtualBoxBase *aObj, bool fTry /* = false */)
     : mObj(aObj),
       mInitFailed(false),
-      mUninitDone(false)
+      mUninitDone(false),
+      mUninitFailed(false)
 {
     Assert(mObj);
     ObjectState::State state;
-    state = mObj->getObjectState().autoUninitSpanConstructor();
+    state = mObj->getObjectState().autoUninitSpanConstructor(fTry);
     if (state == ObjectState::InitFailed)
         mInitFailed = true;
     else if (state == ObjectState::NotReady)
         mUninitDone = true;
+    else if (state == ObjectState::Ready)
+        mUninitFailed = true;
 }
 
 /**
@@ -538,7 +563,7 @@ AutoUninitSpan::AutoUninitSpan(VirtualBoxBase *aObj)
 AutoUninitSpan::~AutoUninitSpan()
 {
     /* do nothing if already uninitialized */
-    if (mUninitDone)
+    if (mUninitDone || mUninitFailed)
         return;
 
     mObj->getObjectState().autoUninitSpanDestructor();
@@ -552,7 +577,7 @@ AutoUninitSpan::~AutoUninitSpan()
 void AutoUninitSpan::setSucceeded()
 {
     /* do nothing if already uninitialized */
-    if (mUninitDone)
+    if (mUninitDone || mUninitFailed)
         return;
 
     mObj->getObjectState().autoUninitSpanDestructor();
