@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /** @page pg_main_events    Events
@@ -57,6 +67,7 @@
 #include "EventImpl.h"
 #include "AutoCaller.h"
 #include "LoggingNew.h"
+#include "VBoxEvents.h"
 
 #include <iprt/asm.h>
 #include <iprt/critsect.h>
@@ -105,8 +116,6 @@ void VBoxEvent::FinalRelease()
 
 HRESULT VBoxEvent::init(IEventSource *aSource, VBoxEventType_T aType, BOOL aWaitable)
 {
-    HRESULT rc = S_OK;
-
     AssertReturn(aSource != NULL, E_INVALIDARG);
 
     AutoInitSpan autoInitSpan(this);
@@ -135,7 +144,7 @@ HRESULT VBoxEvent::init(IEventSource *aSource, VBoxEventType_T aType, BOOL aWait
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
 
-    return rc;
+    return S_OK;
 }
 
 void VBoxEvent::uninit()
@@ -213,7 +222,7 @@ HRESULT VBoxEvent::waitProcessed(LONG aTimeout, BOOL *aResult)
     // must drop lock while waiting, because setProcessed() needs synchronization.
     alock.release();
     /** @todo maybe while loop for spurious wakeups? */
-    int vrc = ::RTSemEventWait(m->mWaitEvent, aTimeout);
+    int vrc = ::RTSemEventWait(m->mWaitEvent, aTimeout < 0 ? RT_INDEFINITE_WAIT : (RTMSINTERVAL)aTimeout);
     AssertMsg(RT_SUCCESS(vrc) || vrc == VERR_TIMEOUT || vrc == VERR_INTERRUPTED,
               ("RTSemEventWait returned %Rrc\n", vrc));
     alock.acquire();
@@ -257,9 +266,9 @@ struct VBoxVetoEvent::Data
 HRESULT VBoxVetoEvent::FinalConstruct()
 {
     m = new Data;
-    HRESULT rc = m->mEvent.createObject();
+    HRESULT hrc = m->mEvent.createObject();
     BaseFinalConstruct();
-    return rc;
+    return hrc;
 }
 
 void VBoxVetoEvent::FinalRelease()
@@ -277,11 +286,10 @@ DEFINE_EMPTY_CTOR_DTOR(VBoxVetoEvent)
 
 HRESULT VBoxVetoEvent::init(IEventSource *aSource, VBoxEventType_T aType)
 {
-    HRESULT rc = S_OK;
     // all veto events are waitable
-    rc = m->mEvent->init(aSource, aType, TRUE);
-    if (FAILED(rc))
-        return rc;
+    HRESULT hrc = m->mEvent->init(aSource, aType, TRUE);
+    if (FAILED(hrc))
+        return hrc;
 
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
@@ -394,7 +402,7 @@ HRESULT VBoxVetoEvent::getApprovals(std::vector<com::Utf8Str> &aResult)
 }
 
 static const int FirstEvent = (int)VBoxEventType_LastWildcard + 1;
-static const int LastEvent = (int)VBoxEventType_Last;
+static const int LastEvent = (int)VBoxEventType_End;
 static const int NumEvents = LastEvent - FirstEvent;
 
 /**
@@ -840,19 +848,19 @@ HRESULT ListenerRecord::process(IEvent *aEvent,
         /*
          * We release lock here to allow modifying ops on EventSource inside callback.
          */
-        HRESULT rc = S_OK;
+        HRESULT hrc = S_OK;
         if (mListener)
         {
             aAlock.release();
-            rc = mListener->HandleEvent(aEvent);
+            hrc = mListener->HandleEvent(aEvent);
 #ifdef RT_OS_WINDOWS
-            Assert(rc != RPC_E_WRONG_THREAD);
+            Assert(hrc != RPC_E_WRONG_THREAD);
 #endif
             aAlock.acquire();
         }
         if (aWaitable)
             eventProcessed(aEvent, pit);
-        return rc;
+        return hrc;
     }
     return enqueue(aEvent);
 }
@@ -929,7 +937,7 @@ HRESULT ListenerRecord::dequeue(IEvent **aEvent,
             // release lock while waiting, listener will not go away due to above holder
             aAlock.release();
 
-            ::RTSemEventWait(hEvt, aTimeout);
+            ::RTSemEventWait(hEvt, aTimeout < 0 ? RT_INDEFINITE_WAIT : (RTMSINTERVAL)aTimeout);
             ASMAtomicDecS32(&mQEventBusyCnt);
 
             // reacquire lock
@@ -1022,14 +1030,12 @@ void EventSource::FinalRelease()
 
 HRESULT EventSource::init()
 {
-    HRESULT rc = S_OK;
-
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
-    return rc;
+    return S_OK;
 }
 
 void EventSource::uninit()
@@ -1081,16 +1087,14 @@ HRESULT EventSource::registerListener(const ComPtr<IEventListener> &aListener,
     RecordHolder<ListenerRecord> lrh(new ListenerRecord(aListener, interested, aActive, this));
     m->mListeners.insert(Listeners::value_type((IEventListener *)aListener, lrh));
 
-    VBoxEventDesc evDesc;
-    evDesc.init(this, VBoxEventType_OnEventSourceChanged, (IEventListener *)aListener, TRUE);
-    evDesc.fire(0);
+    ::FireEventSourceChangedEvent(this, (IEventListener *)aListener, TRUE /*add*/);
 
     return S_OK;
 }
 
 HRESULT EventSource::unregisterListener(const ComPtr<IEventListener> &aListener)
 {
-    HRESULT rc = S_OK;;
+    HRESULT hrc = S_OK;;
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -1101,22 +1105,14 @@ HRESULT EventSource::unregisterListener(const ComPtr<IEventListener> &aListener)
         it->second.obj()->shutdown();
         m->mListeners.erase(it);
         // destructor removes refs from the event map
-        rc = S_OK;
+        ::FireEventSourceChangedEvent(this, (IEventListener *)aListener, FALSE /*add*/);
+        hrc = S_OK;
     }
     else
-    {
-        rc = setError(VBOX_E_OBJECT_NOT_FOUND,
-                      tr("Listener was never registered"));
-    }
+        hrc = setError(VBOX_E_OBJECT_NOT_FOUND,
+                       tr("Listener was never registered"));
 
-    if (SUCCEEDED(rc))
-    {
-        VBoxEventDesc evDesc;
-        evDesc.init(this, VBoxEventType_OnEventSourceChanged, (IEventListener *)aListener, FALSE);
-        evDesc.fire(0);
-    }
-
-    return rc;
+    return hrc;
 }
 
 HRESULT EventSource::fireEvent(const ComPtr<IEvent> &aEvent,
@@ -1242,18 +1238,18 @@ HRESULT EventSource::getEvent(const ComPtr<IEventListener> &aListener,
                         tr("This event source is already shut down"));
 
     Listeners::iterator it = m->mListeners.find(aListener);
-    HRESULT rc = S_OK;
+    HRESULT hrc = S_OK;
 
     if (it != m->mListeners.end())
-        rc = it->second.obj()->dequeue(aEvent.asOutParam(), aTimeout, alock);
+        hrc = it->second.obj()->dequeue(aEvent.asOutParam(), aTimeout, alock);
     else
-        rc = setError(VBOX_E_OBJECT_NOT_FOUND,
-                      tr("Listener was never registered"));
+        hrc = setError(VBOX_E_OBJECT_NOT_FOUND,
+                       tr("Listener was never registered"));
 
-    if (rc == VBOX_E_INVALID_OBJECT_STATE)
-        return setError(rc, tr("Listener must be passive"));
+    if (hrc == VBOX_E_INVALID_OBJECT_STATE)
+        return setError(hrc, tr("Listener must be passive"));
 
-    return rc;
+    return hrc;
 }
 
 HRESULT EventSource::eventProcessed(const ComPtr<IEventListener> &aListener,
@@ -1347,7 +1343,7 @@ public:
     // IEventListener methods
     STDMETHOD(HandleEvent)(IEvent *)
     {
-        ComAssertMsgRet(false, ("HandleEvent() of wrapper shall never be called"),
+        ComAssertMsgRet(false, (tr("HandleEvent() of wrapper shall never be called")),
                         E_FAIL);
     }
 };
@@ -1490,8 +1486,8 @@ HRESULT EventSource::createListener(ComPtr<IEventListener> &aListener)
 {
     ComObjPtr<PassiveEventListener> listener;
 
-    HRESULT rc = listener.createObject();
-    ComAssertMsgRet(SUCCEEDED(rc), ("Could not create wrapper object (%Rhrc)", rc),
+    HRESULT hrc = listener.createObject();
+    ComAssertMsgRet(SUCCEEDED(hrc), (tr("Could not create wrapper object (%Rhrc)"), hrc),
                     E_FAIL);
     listener.queryInterfaceTo(aListener.asOutParam());
     return S_OK;
@@ -1502,13 +1498,13 @@ HRESULT EventSource::createAggregator(const std::vector<ComPtr<IEventSource> > &
 {
     ComObjPtr<EventSourceAggregator> agg;
 
-    HRESULT rc = agg.createObject();
-    ComAssertMsgRet(SUCCEEDED(rc), ("Could not create aggregator (%Rhrc)", rc),
+    HRESULT hrc = agg.createObject();
+    ComAssertMsgRet(SUCCEEDED(hrc), (tr("Could not create aggregator (%Rhrc)"), hrc),
                     E_FAIL);
 
-    rc = agg->init(aSubordinates);
-    if (FAILED(rc))
-        return rc;
+    hrc = agg->init(aSubordinates);
+    if (FAILED(hrc))
+        return hrc;
 
     agg.queryInterfaceTo(aResult.asOutParam());
     return S_OK;
@@ -1516,16 +1512,14 @@ HRESULT EventSource::createAggregator(const std::vector<ComPtr<IEventSource> > &
 
 HRESULT EventSourceAggregator::init(const std::vector<ComPtr<IEventSource> >  aSourcesIn)
 {
-    HRESULT rc;
-
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    rc = mSource.createObject();
-    ComAssertMsgRet(SUCCEEDED(rc), ("Could not create source (%Rhrc)", rc),
+    HRESULT hrc = mSource.createObject();
+    ComAssertMsgRet(SUCCEEDED(hrc), (tr("Could not create source (%Rhrc)"), hrc),
                     E_FAIL);
-    rc = mSource->init();
-    ComAssertMsgRet(SUCCEEDED(rc), ("Could not init source (%Rhrc)", rc),
+    hrc = mSource->init();
+    ComAssertMsgRet(SUCCEEDED(hrc), (tr("Could not init source (%Rhrc)"), hrc),
                     E_FAIL);
 
     for (size_t i = 0; i < aSourcesIn.size(); i++)
@@ -1537,7 +1531,7 @@ HRESULT EventSourceAggregator::init(const std::vector<ComPtr<IEventSource> >  aS
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
 
-    return rc;
+    return hrc;
 }
 
 STDMETHODIMP EventSourceAggregator::CreateListener(IEventListener **aListener)
@@ -1559,15 +1553,13 @@ STDMETHODIMP EventSourceAggregator::RegisterListener(IEventListener *aListener,
     CheckComArgSafeArrayNotNull(aInterested);
 
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc()))
-        return autoCaller.rc();
-
-    HRESULT rc;
+    if (FAILED(autoCaller.hrc()))
+        return autoCaller.hrc();
 
     ComPtr<IEventListener> proxy;
-    rc = createProxyListener(aListener, proxy.asOutParam());
-    if (FAILED(rc))
-        return rc;
+    HRESULT hrc = createProxyListener(aListener, proxy.asOutParam());
+    if (FAILED(hrc))
+        return hrc;
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
     for (EventSourceList::const_iterator it = mEventSources.begin(); it != mEventSources.end();
@@ -1575,14 +1567,12 @@ STDMETHODIMP EventSourceAggregator::RegisterListener(IEventListener *aListener,
     {
         ComPtr<IEventSource> es = *it;
         /* Register active proxy listener on real event source */
-        rc = es->RegisterListener(proxy, ComSafeArrayInArg(aInterested), TRUE);
+        hrc = es->RegisterListener(proxy, ComSafeArrayInArg(aInterested), TRUE);
     }
     /* And add real listener on our event source */
-    rc = mSource->RegisterListener(aListener, ComSafeArrayInArg(aInterested), aActive);
+    hrc = mSource->RegisterListener(aListener, ComSafeArrayInArg(aInterested), aActive);
 
-    rc = S_OK;
-
-    return rc;
+    return S_OK;
 }
 
 STDMETHODIMP EventSourceAggregator::UnregisterListener(IEventListener *aListener)
@@ -1590,25 +1580,23 @@ STDMETHODIMP EventSourceAggregator::UnregisterListener(IEventListener *aListener
     CheckComArgNotNull(aListener);
 
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc()))
-        return autoCaller.rc();
-
-    HRESULT rc = S_OK;
+    if (FAILED(autoCaller.hrc()))
+        return autoCaller.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     ComPtr<IEventListener> proxy;
-    rc = getProxyListener(aListener, proxy.asOutParam());
-    if (FAILED(rc))
-        return rc;
+    HRESULT hrc = getProxyListener(aListener, proxy.asOutParam());
+    if (FAILED(hrc))
+        return hrc;
 
     for (EventSourceList::const_iterator it = mEventSources.begin(); it != mEventSources.end();
          ++it)
     {
         ComPtr<IEventSource> es = *it;
-        rc = es->UnregisterListener(proxy);
+        hrc = es->UnregisterListener(proxy);
     }
-    rc = mSource->UnregisterListener(aListener);
+    hrc = mSource->UnregisterListener(aListener);
 
     return removeProxyListener(aListener);
 
@@ -1622,10 +1610,9 @@ STDMETHODIMP EventSourceAggregator::FireEvent(IEvent *aEvent,
     CheckComArgOutPointerValid(aProcessed);
 
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc()))
-        return autoCaller.rc();
+    if (FAILED(autoCaller.hrc()))
+        return autoCaller.hrc();
 
-    HRESULT rc = S_OK;
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
     /* Aggregator event source shall not have direct event firing, but we may
        wish to support aggregation chains */
@@ -1633,10 +1620,10 @@ STDMETHODIMP EventSourceAggregator::FireEvent(IEvent *aEvent,
          ++it)
     {
         ComPtr<IEventSource> es = *it;
-        rc = es->FireEvent(aEvent, aTimeout, aProcessed);
+        HRESULT hrc = es->FireEvent(aEvent, aTimeout, aProcessed);
         /* Current behavior is that aggregator's FireEvent() always succeeds,
            so that multiple event sources don't affect each other. */
-        NOREF(rc);
+        NOREF(hrc);
     }
 
     return S_OK;
@@ -1660,13 +1647,13 @@ HRESULT EventSourceAggregator::createProxyListener(IEventListener *aListener,
 {
     ComObjPtr<ProxyEventListener> proxy;
 
-    HRESULT rc = proxy.createObject();
-    ComAssertMsgRet(SUCCEEDED(rc), ("Could not create proxy (%Rhrc)", rc),
+    HRESULT hrc = proxy.createObject();
+    ComAssertMsgRet(SUCCEEDED(hrc), (tr("Could not create proxy (%Rhrc)"), hrc),
                     E_FAIL);
 
-    rc = proxy->init(mSource);
-    if (FAILED(rc))
-        return rc;
+    hrc = proxy->init(mSource);
+    if (FAILED(hrc))
+        return hrc;
 
     ProxyListenerMap::const_iterator it = mListenerProxies.find(aListener);
     if (it != mListenerProxies.end())
