@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2017-2020 Oracle Corporation
+ * Copyright (C) 2017-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -94,6 +104,7 @@
 #include <iprt/buildconfig.h>
 #include <iprt/dir.h>
 #include <iprt/env.h>
+#include <iprt/err.h>
 #include <iprt/getopt.h>
 #include <iprt/initterm.h>
 #include <iprt/path.h>
@@ -341,8 +352,8 @@ protected:
     {
         uint32_t const uPrevState = ASMAtomicXchgU32((uint32_t volatile *)&m_Status.dwCurrentState, dwState);
         if (!::SetServiceStatus(m_hServiceStatus, &m_Status))
-            LogRel(("Error: SetServiceStatus(%p, %u) failed: %u (uPrevState=%u)\n",
-                    m_hServiceStatus, dwState, GetLastError(), uPrevState));
+            LogRel(("Error: SetServiceStatus(,%u) failed: %u (uPrevState=%u)\n",
+                    dwState, GetLastError(), uPrevState));
     }
 
 
@@ -392,11 +403,11 @@ public:
                     wszFilePath[cwcFilePath + 1] = L'\"';
                     wszFilePath[cwcFilePath + 2] = L'\0';
 
-                    SC_HANDLE hService = ::CreateServiceW(hSCM, m_wszServiceName, m_wszServiceDisplayName,
-                                                          SERVICE_CHANGE_CONFIG,
-                                                          SERVICE_WIN32_OWN_PROCESS,
-                                                          SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,
-                                                          wszFilePath, NULL, NULL, L"RPCSS\0", NULL, NULL);
+                    hService = ::CreateServiceW(hSCM, m_wszServiceName, m_wszServiceDisplayName,
+                                                SERVICE_CHANGE_CONFIG,
+                                                SERVICE_WIN32_OWN_PROCESS,
+                                                SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,
+                                                wszFilePath, NULL, NULL, L"RPCSS\0", NULL, NULL);
                     if (hService != NULL)
                     {
                         SERVICE_DESCRIPTIONW sd;
@@ -527,9 +538,9 @@ public:
 #ifdef WITH_WATCHER
         , m_fHasClients(false)
 #endif
-        , m_cMsShutdownTimeOut(cMsShutdownTimeout)
         , m_hEventShutdown(INVALID_HANDLE_VALUE)
         , m_dwMainThreadID(~(DWORD)42)
+        , m_cMsShutdownTimeOut(cMsShutdownTimeout)
     {
     }
 
@@ -543,12 +554,12 @@ public:
         if (SUCCEEDED(hrc))
         {
             // copy service name
-            int rc = ::RTUtf16Copy(m_wszServiceName, sizeof(m_wszServiceName), p_wszServiceName);
-            AssertRCReturn(rc, E_NOT_SUFFICIENT_BUFFER);
-            rc = ::RTUtf16Copy(m_wszServiceDisplayName, sizeof(m_wszServiceDisplayName), p_wszDisplayName);
-            AssertRCReturn(rc, E_NOT_SUFFICIENT_BUFFER);
-            rc = ::RTUtf16Copy(m_wszServiceDescription, sizeof(m_wszServiceDescription), p_wszDescription);
-            AssertRCReturn(rc, E_NOT_SUFFICIENT_BUFFER);
+            int vrc = ::RTUtf16Copy(m_wszServiceName, sizeof(m_wszServiceName), p_wszServiceName);
+            AssertRCReturn(vrc, E_NOT_SUFFICIENT_BUFFER);
+            vrc = ::RTUtf16Copy(m_wszServiceDisplayName, sizeof(m_wszServiceDisplayName), p_wszDisplayName);
+            AssertRCReturn(vrc, E_NOT_SUFFICIENT_BUFFER);
+            vrc = ::RTUtf16Copy(m_wszServiceDescription, sizeof(m_wszServiceDescription), p_wszDescription);
+            AssertRCReturn(vrc, E_NOT_SUFFICIENT_BUFFER);
 
             m_fInitialized = true;
         }
@@ -641,14 +652,14 @@ protected:
                 if (pReleaseLogger)
                 {
                     char szDest[1024];
-                    int rc = ::RTLogGetDestinations(pReleaseLogger, szDest, sizeof(szDest));
-                    if (RT_SUCCESS(rc))
+                    int vrc = ::RTLogQueryDestinations(pReleaseLogger, szDest, sizeof(szDest));
+                    if (RT_SUCCESS(vrc))
                     {
-                        rc = ::RTStrCat(szDest, sizeof(szDest), " nohistory");
-                        if (RT_SUCCESS(rc))
+                        vrc = ::RTStrCat(szDest, sizeof(szDest), " nohistory");
+                        if (RT_SUCCESS(vrc))
                         {
-                            rc = ::RTLogDestinations(pReleaseLogger, szDest);
-                            AssertRC(rc);
+                            vrc = ::RTLogDestinations(pReleaseLogger, szDest);
+                            AssertRC(vrc);
                         }
                     }
                 }
@@ -691,7 +702,7 @@ protected:
         }
     }
 
-    virtual HRESULT preMessageLoop(int nShowCmd)
+    virtual HRESULT preMessageLoop(int nShowCmd) throw()
     {
         Assert(m_fInitialized);
         LogFunc(("Enter\n"));
@@ -901,10 +912,14 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
     }
 
     /*
-     * Default log location is %ProgramData%\VirtualBox\VBoxSDS.log, falling back
-     * on %_CWD%\VBoxSDS.log (where _CWD typicaly is 'C:\Windows\System32').
+     * Default log location (LOGDIR) is %APPDATA%\VirtualBox\VBoxSDS.log.
      *
-     * We change the current directory to %ProgramData%\VirtualBox\ if possible.
+     * When running VBoxSDS as a regular user, LOGDIR typically will be 'C:\Users\<User>\AppData\Roaming\VirtualBox\'.
+     * When running VBoxSDS as a service (via SCM), LOGDIR typically will be 'C:\Windows\System32\config\systemprofile\AppData\Roaming\VirtualBox\'.
+     *
+     * We change the current directory to LOGDIR if possible.
+     *
+     * See @bugref{10632}.
      *
      * We only create the log file when running VBoxSDS normally, but not
      * when registering/unregistering, at least for now.
@@ -914,14 +929,19 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
         char szLogFile[RTPATH_MAX];
         if (!pszLogFile || !*pszLogFile)
         {
-            WCHAR wszAppData[MAX_PATH + 16];
-            if (SHGetSpecialFolderPathW(NULL, wszAppData, CSIDL_COMMON_APPDATA, TRUE /*fCreate*/))
+            WCHAR wszSpecialFolder[MAX_PATH + 16];
+            if (SHGetSpecialFolderPathW(NULL, wszSpecialFolder, CSIDL_APPDATA, TRUE /*fCreate*/))
             {
                 char *pszConv = szLogFile;
-                vrc = RTUtf16ToUtf8Ex(wszAppData, RTSTR_MAX, &pszConv, sizeof(szLogFile) - 12, NULL);
+                vrc = RTUtf16ToUtf8Ex(wszSpecialFolder, RTSTR_MAX, &pszConv, sizeof(szLogFile) - 12, NULL);
             }
-            else
-                vrc = RTEnvGetUtf8("ProgramData", szLogFile, sizeof(szLogFile) - sizeof("VBoxSDS.log"), NULL);
+            else if (SHGetSpecialFolderPathW(NULL, wszSpecialFolder, CSIDL_SYSTEM, TRUE /*fCreate*/))
+            {
+                char *pszConv = szLogFile;
+                vrc = RTUtf16ToUtf8Ex(wszSpecialFolder, RTSTR_MAX, &pszConv, sizeof(szLogFile) - 12, NULL);
+            }
+            else /* Note! No fallback to environment variables or such. See @bugref{10632}. */
+                vrc = VERR_PATH_NOT_FOUND;
             if (RT_SUCCESS(vrc))
             {
                 vrc = RTPathAppend(szLogFile, sizeof(szLogFile), "VirtualBox\\");
@@ -932,14 +952,14 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
                         vrc = RTDirCreate(szLogFile, 0755, RTDIRCREATE_FLAGS_NOT_CONTENT_INDEXED_DONT_SET);
                     if (RT_SUCCESS(vrc))
                     {
-                        /* Change into it. */
-                        RTPathSetCurrent(szLogFile);
+                        /* Change into it.
+                         * If this fails, better don't continue, as there might be something fishy. */
+                        vrc = RTPathSetCurrent(szLogFile);
+                        if (RT_SUCCESS(vrc))
+                            vrc = RTStrCat(szLogFile, sizeof(szLogFile), "VBoxSDS.log");
                     }
                 }
             }
-            if (RT_FAILURE(vrc))     /* ignore any failure above */
-                szLogFile[0] = '\0';
-            vrc = RTStrCat(szLogFile, sizeof(szLogFile), "VBoxSDS.log");
             if (RT_FAILURE(vrc))
                 return RTMsgErrorExit(RTEXITCODE_FAILURE, "Failed to construct release log filename: %Rrc", vrc);
             pszLogFile = szLogFile;
@@ -1020,7 +1040,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine
             else
                 LogRelFunc(("VBoxSDS: new CComServiceModule::Init failed: %Rhrc\n", hrcExit));
 
-            LogRelFunc(("VBoxSDS: deleting pServiceModule (%p)\n", pServiceModule));
+            LogRelFunc(("VBoxSDS: deleting pServiceModule\n"));
             delete pServiceModule;
             pServiceModule = NULL;
         }
