@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #define LOG_GROUP LOG_GROUP_MAIN
@@ -862,7 +872,7 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
                 cMillies = s_aUpdateTimeoutSteps[uOld];
             }
 
-            int rc = RTSemEventWait(that->mUpdateReq, cMillies);
+            int vrc = RTSemEventWait(that->mUpdateReq, cMillies);
 
             /*
              *  Restore the caller before using VirtualBox. If it fails, this
@@ -872,9 +882,9 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
             if (!autoCaller.isOk())
                 break;
 
-            if (RT_SUCCESS(rc) || update || updateSpawned)
+            if (RT_SUCCESS(vrc) || update || updateSpawned)
             {
-                /* RT_SUCCESS(rc) means an update event is signaled */
+                /* RT_SUCCESS(vrc) means an update event is signaled */
 
                 // get reference to the machines list in VirtualBox
                 VirtualBox::MachinesOList &allMachines = that->mVirtualBox->i_getMachinesList();
@@ -882,7 +892,7 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
                 // lock the machines list for reading
                 AutoReadLock thatLock(allMachines.getLockHandle() COMMA_LOCKVAL_SRC_POS);
 
-                if (RT_SUCCESS(rc) || update)
+                if (RT_SUCCESS(vrc) || update)
                 {
                     /* obtain a new set of opened machines */
                     machines.clear();
@@ -900,7 +910,7 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
                     LogFlowFunc(("UPDATE: direct session count = %d\n", cnt));
                 }
 
-                if (RT_SUCCESS(rc) || updateSpawned)
+                if (RT_SUCCESS(vrc) || updateSpawned)
                 {
                     /* obtain a new set of spawned machines */
                     spawnedMachines.clear();
@@ -971,7 +981,7 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
                 cMillies = s_aUpdateTimeoutSteps[uOld];
             }
 
-            int rc = RTSemEventWait(that->mUpdateReq, cMillies);
+            int vrc = RTSemEventWait(that->mUpdateReq, cMillies);
 
             /*
              *  Restore the caller before using VirtualBox. If it fails, this
@@ -987,9 +997,9 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
              * reaping is made smarter, having cross-reference information
              * from the pid to the corresponding machine object. Both cases do
              * more or less the same thing anyway. */
-            if (RT_SUCCESS(rc) || updateSpawned)
+            if (RT_SUCCESS(vrc) || updateSpawned)
             {
-                /* RT_SUCCESS(rc) means an update event is signaled */
+                /* RT_SUCCESS(vrc) means an update event is signaled */
 
                 // get reference to the machines list in VirtualBox
                 VirtualBox::MachinesOList &allMachines = that->mVirtualBox->i_getMachinesList();
@@ -997,7 +1007,7 @@ DECLCALLBACK(int) VirtualBox::ClientWatcher::worker(RTTHREAD hThreadSelf, void *
                 // lock the machines list for reading
                 AutoReadLock thatLock(allMachines.getLockHandle() COMMA_LOCKVAL_SRC_POS);
 
-                if (RT_SUCCESS(rc) || updateSpawned)
+                if (RT_SUCCESS(vrc) || updateSpawned)
                 {
                     /* obtain a new set of spawned machines */
                     spawnedMachines.clear();
