@@ -4,17 +4,28 @@
  */
 
 /*
- * Copyright (C) 2010-2017 Oracle Corporation
+ * Copyright (C) 2010-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN_PROGRESS
 #include <iprt/types.h>
 
 #include "ProgressProxyImpl.h"
@@ -22,12 +33,11 @@
 #include "VirtualBoxImpl.h"
 #include "VirtualBoxErrorInfoImpl.h"
 
-#include "Logging.h"
+#include "LoggingNew.h"
 
 #include <iprt/time.h>
 #include <iprt/semaphore.h>
-
-#include <VBox/err.h>
+#include <iprt/errcore.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 // ProgressProxy class
@@ -44,8 +54,7 @@ HRESULT ProgressProxy::FinalConstruct()
     muOtherProgressWeight = 0;
     muOtherProgressStartOperation = 0;
 
-    HRESULT rc = Progress::FinalConstruct();
-    return rc;
+    return Progress::FinalConstruct();
 }
 
 /**
@@ -58,7 +67,7 @@ HRESULT ProgressProxy::init(
                             VirtualBox *pParent,
 #endif
                             IUnknown *pInitiator,
-                            CBSTR bstrDescription,
+                            Utf8Str strDescription,
                             BOOL fCancelable)
 {
     mfMultiOperation = false;
@@ -71,11 +80,11 @@ HRESULT ProgressProxy::init(
                           pParent,
 #endif
                           pInitiator,
-                          bstrDescription,
+                          strDescription,
                           fCancelable,
                           1 /* cOperations */,
                           1 /* ulTotalOperationsWeight */,
-                          bstrDescription /* bstrFirstOperationDescription */,
+                          strDescription /* strFirstOperationDescription */,
                           1 /* ulFirstOperationWeight */);
 }
 
@@ -94,10 +103,10 @@ HRESULT ProgressProxy::init(
                             VirtualBox *pParent,
 #endif
                             IUnknown *pInitiator,
-                            CBSTR bstrDescription,
+                            Utf8Str strDescription,
                             BOOL fCancelable,
                             ULONG uTotalOperationsWeight,
-                            CBSTR bstrFirstOperationDescription,
+                            Utf8Str strFirstOperationDescription,
                             ULONG uFirstOperationWeight,
                             ULONG cOtherProgressObjectOperations)
 {
@@ -111,11 +120,11 @@ HRESULT ProgressProxy::init(
                           pParent,
 #endif
                           pInitiator,
-                          bstrDescription,
+                          strDescription,
                           fCancelable,
                           1 + cOtherProgressObjectOperations /* cOperations */,
                           uTotalOperationsWeight,
-                          bstrFirstOperationDescription,
+                          strFirstOperationDescription,
                           uFirstOperationWeight);
 }
 
@@ -337,11 +346,11 @@ void ProgressProxy::copyProgressInfo(IProgress *pOtherProgress, bool fEarly)
             if (fCompleted)
             {
                 /* Check the result. */
-                LONG hrcResult;
-                hrc = pOtherProgress->COMGETTER(ResultCode)(&hrcResult);
+                LONG lResult;
+                hrc = pOtherProgress->COMGETTER(ResultCode)(&lResult);
                 if (FAILED(hrc))
-                    hrcResult = hrc;
-                if (SUCCEEDED((HRESULT)hrcResult))
+                    lResult = (LONG)hrc;
+                if (SUCCEEDED((HRESULT)lResult))
                     LogFlowThisFunc(("Succeeded\n"));
                 else
                 {
@@ -366,16 +375,16 @@ void ProgressProxy::copyProgressInfo(IProgress *pOtherProgress, bool fEarly)
                             bstrText = "<failed>";
 
                         Utf8Str strText(bstrText);
-                        LogFlowThisFunc(("Got ErrorInfo(%s); hrcResult=%Rhrc\n", strText.c_str(), hrcResult));
-                        Progress::i_notifyComplete((HRESULT)hrcResult,
+                        LogFlowThisFunc(("Got ErrorInfo(%s); hrcResult=%Rhrc\n", strText.c_str(), (HRESULT)lResult));
+                        Progress::i_notifyComplete((HRESULT)lResult,
                                                    Guid(bstrIID).ref(),
                                                    Utf8Str(bstrComponent).c_str(),
                                                    "%s", strText.c_str());
                     }
                     else
                     {
-                        LogFlowThisFunc(("ErrorInfo failed with hrc=%Rhrc; hrcResult=%Rhrc\n", hrc, hrcResult));
-                        Progress::i_notifyComplete((HRESULT)hrcResult,
+                        LogFlowThisFunc(("ErrorInfo failed with hrc=%Rhrc; hrcResult=%Rhrc\n", hrc, (HRESULT)lResult));
+                        Progress::i_notifyComplete((HRESULT)lResult,
                                                    COM_IIDOF(IProgress),
                                                    "ProgressProxy",
                                                    tr("No error info"));
@@ -413,7 +422,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(Cancelable)(BOOL *aCancelable)
     CheckComArgOutPointerValid(aCancelable);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -439,7 +448,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(Percent)(ULONG *aPercent)
     CheckComArgOutPointerValid(aPercent);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -471,7 +480,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(TimeRemaining)(LONG *aTimeRemaining)
     CheckComArgOutPointerValid(aTimeRemaining);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -495,7 +504,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(Canceled)(BOOL *aCanceled)
     CheckComArgOutPointerValid(aCanceled);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         /* Check the local data first, then the other object. */
@@ -532,7 +541,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(Operation)(ULONG *aOperation)
     CheckComArgOutPointerValid(aOperation);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -554,7 +563,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(OperationDescription)(BSTR *aOperationDesc
     CheckComArgOutPointerValid(aOperationDescription);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -571,7 +580,7 @@ STDMETHODIMP ProgressProxy::COMGETTER(OperationPercent)(ULONG *aOperationPercent
     CheckComArgOutPointerValid(aOperationPercent);
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -623,7 +632,7 @@ STDMETHODIMP ProgressProxy::WaitForOperationCompletion(ULONG aOperation, LONG aT
     LogFlowThisFunc(("aOperation=%d aTimeout=%d\n", aOperation, aTimeout));
 
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -658,7 +667,7 @@ STDMETHODIMP ProgressProxy::Cancel()
 {
     LogFlowThisFunc(("\n"));
     AutoCaller autoCaller(this);
-    HRESULT hrc = autoCaller.rc();
+    HRESULT hrc = autoCaller.hrc();
     if (SUCCEEDED(hrc))
     {
         AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
