@@ -5,15 +5,25 @@
  */
 
 /*
- * Copyright (C) 2018-2020 Oracle Corporation
+ * Copyright (C) 2018-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #define LOG_GROUP LOG_GROUP_MAIN_RECORDINGSCREENSETTINGS
@@ -143,7 +153,7 @@ HRESULT RecordingScreenSettings::init(RecordingSettings *aParent, RecordingScree
     unconst(m->pPeer)   = aThat;
 
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnRC(thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.hrc());
 
     AutoReadLock thatlock(aThat COMMA_LOCKVAL_SRC_POS);
 
@@ -188,7 +198,7 @@ HRESULT RecordingScreenSettings::initCopy(RecordingSettings *aParent, RecordingS
     /* mPeer is left null. */
 
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnRC(thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.hrc());
 
     AutoReadLock thatlock(aThat COMMA_LOCKVAL_SRC_POS);
 
@@ -242,7 +252,7 @@ void RecordingScreenSettings::uninit()
 HRESULT RecordingScreenSettings::isFeatureEnabled(RecordingFeature_T aFeature, BOOL *aEnabled)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -257,7 +267,7 @@ HRESULT RecordingScreenSettings::isFeatureEnabled(RecordingFeature_T aFeature, B
 HRESULT RecordingScreenSettings::getId(ULONG *id)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -269,7 +279,7 @@ HRESULT RecordingScreenSettings::getId(ULONG *id)
 HRESULT RecordingScreenSettings::getEnabled(BOOL *enabled)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -281,7 +291,7 @@ HRESULT RecordingScreenSettings::getEnabled(BOOL *enabled)
 HRESULT RecordingScreenSettings::setEnabled(BOOL enabled)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     LogFlowThisFunc(("Screen %RU32\n", m->uScreenId));
 
@@ -303,20 +313,20 @@ HRESULT RecordingScreenSettings::setEnabled(BOOL enabled)
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::getFeatures(ULONG *aFeatures)
+HRESULT RecordingScreenSettings::getFeatures(std::vector<RecordingFeature_T> &aFeatures)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aFeatures = 0;
+    aFeatures.clear();
 
     settings::RecordingFeatureMap::const_iterator itFeature = m->bd->featureMap.begin();
     while (itFeature != m->bd->featureMap.end())
     {
         if (itFeature->second) /* Is feature enable? */
-            *aFeatures |= (ULONG)itFeature->first;
+            aFeatures.push_back(itFeature->first);
 
         ++itFeature;
     }
@@ -324,10 +334,10 @@ HRESULT RecordingScreenSettings::getFeatures(ULONG *aFeatures)
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::setFeatures(ULONG aFeatures)
+HRESULT RecordingScreenSettings::setFeatures(const std::vector<RecordingFeature_T> &aFeatures)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change features while recording is enabled"));
@@ -339,10 +349,22 @@ HRESULT RecordingScreenSettings::setFeatures(ULONG aFeatures)
     settings::RecordingFeatureMap featureMapOld = m->bd->featureMap;
     m->bd->featureMap.clear();
 
-    if (aFeatures & RecordingFeature_Audio)
-        m->bd->featureMap[RecordingFeature_Audio] = true;
-    if (aFeatures & RecordingFeature_Video)
-        m->bd->featureMap[RecordingFeature_Video] = true;
+    for (size_t i = 0; i < aFeatures.size(); i++)
+    {
+        switch (aFeatures[i])
+        {
+            case RecordingFeature_Audio:
+                m->bd->featureMap[RecordingFeature_Audio] = true;
+                break;
+
+            case RecordingFeature_Video:
+                m->bd->featureMap[RecordingFeature_Video] = true;
+                break;
+
+            default:
+                break;
+        }
+    }
 
     if (m->bd->featureMap != featureMapOld)
     {
@@ -357,7 +379,7 @@ HRESULT RecordingScreenSettings::setFeatures(ULONG aFeatures)
 HRESULT RecordingScreenSettings::getDestination(RecordingDestination_T *aDestination)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -369,7 +391,7 @@ HRESULT RecordingScreenSettings::getDestination(RecordingDestination_T *aDestina
 HRESULT RecordingScreenSettings::setDestination(RecordingDestination_T aDestination)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change destination type while recording is enabled"));
@@ -393,7 +415,7 @@ HRESULT RecordingScreenSettings::setDestination(RecordingDestination_T aDestinat
 HRESULT RecordingScreenSettings::getFilename(com::Utf8Str &aFilename)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -417,7 +439,7 @@ HRESULT RecordingScreenSettings::getFilename(com::Utf8Str &aFilename)
 HRESULT RecordingScreenSettings::setFilename(const com::Utf8Str &aFilename)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change file name while recording is enabled"));
@@ -456,7 +478,7 @@ HRESULT RecordingScreenSettings::setFilename(const com::Utf8Str &aFilename)
 HRESULT RecordingScreenSettings::getMaxTime(ULONG *aMaxTimeS)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -468,7 +490,7 @@ HRESULT RecordingScreenSettings::getMaxTime(ULONG *aMaxTimeS)
 HRESULT RecordingScreenSettings::setMaxTime(ULONG aMaxTimeS)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change maximum time while recording is enabled"));
@@ -491,7 +513,7 @@ HRESULT RecordingScreenSettings::setMaxTime(ULONG aMaxTimeS)
 HRESULT RecordingScreenSettings::getMaxFileSize(ULONG *aMaxFileSizeMB)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -503,7 +525,7 @@ HRESULT RecordingScreenSettings::getMaxFileSize(ULONG *aMaxFileSizeMB)
 HRESULT RecordingScreenSettings::setMaxFileSize(ULONG aMaxFileSize)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change maximum file size while recording is enabled"));
@@ -526,7 +548,7 @@ HRESULT RecordingScreenSettings::setMaxFileSize(ULONG aMaxFileSize)
 HRESULT RecordingScreenSettings::getOptions(com::Utf8Str &aOptions)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -538,16 +560,14 @@ HRESULT RecordingScreenSettings::getOptions(com::Utf8Str &aOptions)
 HRESULT RecordingScreenSettings::setOptions(const com::Utf8Str &aOptions)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change options while recording is enabled"));
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    int vrc = RecordingScreenSettings::i_parseOptionsString(aOptions, *m->bd.data());
-    if (RT_FAILURE(vrc))
-        return setError(E_INVALIDARG, tr("Invalid option specified"));
+    /* Note: Parsing and validation is done at codec level. */
 
     m->bd.backup();
     m->bd->strOptions = aOptions;
@@ -562,11 +582,11 @@ HRESULT RecordingScreenSettings::setOptions(const com::Utf8Str &aOptions)
 HRESULT RecordingScreenSettings::getAudioCodec(RecordingAudioCodec_T *aCodec)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aCodec = m->bd->Audio.enmAudioCodec;
+    *aCodec = m->bd->Audio.enmCodec;
 
     return S_OK;
 }
@@ -574,20 +594,20 @@ HRESULT RecordingScreenSettings::getAudioCodec(RecordingAudioCodec_T *aCodec)
 HRESULT RecordingScreenSettings::setAudioCodec(RecordingAudioCodec_T aCodec)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change audio codec while recording is enabled"));
 
-    if (aCodec != RecordingAudioCodec_Opus)
+    if (aCodec != RecordingAudioCodec_OggVorbis)
         return setError(E_INVALIDARG, tr("Audio codec not supported"));
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (m->bd->Audio.enmAudioCodec != aCodec)
+    if (m->bd->Audio.enmCodec != aCodec)
     {
         m->bd.backup();
-        m->bd->Audio.enmAudioCodec = aCodec;
+        m->bd->Audio.enmCodec = aCodec;
 
         alock.release();
 
@@ -597,10 +617,73 @@ HRESULT RecordingScreenSettings::setAudioCodec(RecordingAudioCodec_T aCodec)
     return S_OK;
 }
 
+HRESULT RecordingScreenSettings::getAudioDeadline(RecordingCodecDeadline_T *aDeadline)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aDeadline = m->bd->Audio.enmDeadline;
+
+    return S_OK;
+}
+
+HRESULT RecordingScreenSettings::setAudioDeadline(RecordingCodecDeadline_T aDeadline)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    if (!m->pParent->i_canChangeSettings())
+        return setError(E_INVALIDARG, tr("Cannot change audio deadline while recording is enabled"));
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (m->bd->Audio.enmDeadline != aDeadline)
+    {
+        m->bd.backup();
+        m->bd->Audio.enmDeadline = aDeadline;
+
+        alock.release();
+
+        m->pParent->i_onSettingsChanged();
+    }
+
+    return S_OK;
+}
+
+HRESULT RecordingScreenSettings::getAudioRateControlMode(RecordingRateControlMode_T *aMode)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aMode = RecordingRateControlMode_VBR; /** @todo Implement CBR. */
+
+    return S_OK;
+}
+
+HRESULT RecordingScreenSettings::setAudioRateControlMode(RecordingRateControlMode_T aMode)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    if (!m->pParent->i_canChangeSettings())
+        return setError(E_INVALIDARG, tr("Cannot change audio rate control mode while recording is enabled"));
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    /** @todo Implement this. */
+    RT_NOREF(aMode);
+
+    return E_NOTIMPL;
+}
+
 HRESULT RecordingScreenSettings::getAudioHz(ULONG *aHz)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -612,7 +695,7 @@ HRESULT RecordingScreenSettings::getAudioHz(ULONG *aHz)
 HRESULT RecordingScreenSettings::setAudioHz(ULONG aHz)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change audio Hertz rate while recording is enabled"));
@@ -635,7 +718,7 @@ HRESULT RecordingScreenSettings::setAudioHz(ULONG aHz)
 HRESULT RecordingScreenSettings::getAudioBits(ULONG *aBits)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -647,7 +730,7 @@ HRESULT RecordingScreenSettings::getAudioBits(ULONG *aBits)
 HRESULT RecordingScreenSettings::setAudioBits(ULONG aBits)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change audio bits while recording is enabled"));
@@ -670,7 +753,7 @@ HRESULT RecordingScreenSettings::setAudioBits(ULONG aBits)
 HRESULT RecordingScreenSettings::getAudioChannels(ULONG *aChannels)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -682,7 +765,7 @@ HRESULT RecordingScreenSettings::getAudioChannels(ULONG *aChannels)
 HRESULT RecordingScreenSettings::setAudioChannels(ULONG aChannels)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change audio channels while recording is enabled"));
@@ -705,7 +788,7 @@ HRESULT RecordingScreenSettings::setAudioChannels(ULONG aChannels)
 HRESULT RecordingScreenSettings::getVideoCodec(RecordingVideoCodec_T *aCodec)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -717,7 +800,7 @@ HRESULT RecordingScreenSettings::getVideoCodec(RecordingVideoCodec_T *aCodec)
 HRESULT RecordingScreenSettings::setVideoCodec(RecordingVideoCodec_T aCodec)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video codec while recording is enabled"));
@@ -740,10 +823,45 @@ HRESULT RecordingScreenSettings::setVideoCodec(RecordingVideoCodec_T aCodec)
     return S_OK;
 }
 
+HRESULT RecordingScreenSettings::getVideoDeadline(RecordingCodecDeadline_T *aDeadline)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    *aDeadline = m->bd->Video.enmDeadline;
+
+    return S_OK;
+}
+
+HRESULT RecordingScreenSettings::setVideoDeadline(RecordingCodecDeadline_T aDeadline)
+{
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
+
+    if (!m->pParent->i_canChangeSettings())
+        return setError(E_INVALIDARG, tr("Cannot change video deadline while recording is enabled"));
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (m->bd->Video.enmDeadline != aDeadline)
+    {
+        m->bd.backup();
+        m->bd->Video.enmDeadline = aDeadline;
+
+        alock.release();
+
+        m->pParent->i_onSettingsChanged();
+    }
+
+    return S_OK;
+}
+
 HRESULT RecordingScreenSettings::getVideoWidth(ULONG *aVideoWidth)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -755,7 +873,7 @@ HRESULT RecordingScreenSettings::getVideoWidth(ULONG *aVideoWidth)
 HRESULT RecordingScreenSettings::setVideoWidth(ULONG aVideoWidth)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video width while recording is enabled"));
@@ -778,7 +896,7 @@ HRESULT RecordingScreenSettings::setVideoWidth(ULONG aVideoWidth)
 HRESULT RecordingScreenSettings::getVideoHeight(ULONG *aVideoHeight)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -790,7 +908,7 @@ HRESULT RecordingScreenSettings::getVideoHeight(ULONG *aVideoHeight)
 HRESULT RecordingScreenSettings::setVideoHeight(ULONG aVideoHeight)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video height while recording is enabled"));
@@ -813,7 +931,7 @@ HRESULT RecordingScreenSettings::setVideoHeight(ULONG aVideoHeight)
 HRESULT RecordingScreenSettings::getVideoRate(ULONG *aVideoRate)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -825,7 +943,7 @@ HRESULT RecordingScreenSettings::getVideoRate(ULONG *aVideoRate)
 HRESULT RecordingScreenSettings::setVideoRate(ULONG aVideoRate)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video rate while recording is enabled"));
@@ -845,22 +963,22 @@ HRESULT RecordingScreenSettings::setVideoRate(ULONG aVideoRate)
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::getVideoRateControlMode(RecordingVideoRateControlMode_T *aMode)
+HRESULT RecordingScreenSettings::getVideoRateControlMode(RecordingRateControlMode_T *aMode)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aMode = RecordingVideoRateControlMode_CBR; /** @todo Implement VBR. */
+    *aMode = RecordingRateControlMode_VBR; /** @todo Implement CBR. */
 
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::setVideoRateControlMode(RecordingVideoRateControlMode_T aMode)
+HRESULT RecordingScreenSettings::setVideoRateControlMode(RecordingRateControlMode_T aMode)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video rate control mode while recording is enabled"));
@@ -876,7 +994,7 @@ HRESULT RecordingScreenSettings::setVideoRateControlMode(RecordingVideoRateContr
 HRESULT RecordingScreenSettings::getVideoFPS(ULONG *aVideoFPS)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -888,7 +1006,7 @@ HRESULT RecordingScreenSettings::getVideoFPS(ULONG *aVideoFPS)
 HRESULT RecordingScreenSettings::setVideoFPS(ULONG aVideoFPS)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
         return setError(E_INVALIDARG, tr("Cannot change video FPS while recording is enabled"));
@@ -908,25 +1026,25 @@ HRESULT RecordingScreenSettings::setVideoFPS(ULONG aVideoFPS)
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::getVideoScalingMethod(RecordingVideoScalingMethod_T *aMode)
+HRESULT RecordingScreenSettings::getVideoScalingMode(RecordingVideoScalingMode_T *aMode)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    *aMode = RecordingVideoScalingMethod_None; /** @todo Implement this. */
+    *aMode = RecordingVideoScalingMode_None; /** @todo Implement this. */
 
     return S_OK;
 }
 
-HRESULT RecordingScreenSettings::setVideoScalingMethod(RecordingVideoScalingMethod_T aMode)
+HRESULT RecordingScreenSettings::setVideoScalingMode(RecordingVideoScalingMode_T aMode)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     if (!m->pParent->i_canChangeSettings())
-        return setError(E_INVALIDARG, tr("Cannot change video rate scaling method while recording is enabled"));
+        return setError(E_INVALIDARG, tr("Cannot change video scaling mode while recording is enabled"));
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -947,10 +1065,6 @@ int RecordingScreenSettings::i_initInternal(void)
 
     i_reference();
 
-    int vrc = i_parseOptionsString(m->bd->strOptions, *m->bd.data());
-    if (RT_FAILURE(vrc))
-        return vrc;
-
     switch (m->bd->enmDest)
     {
         case RecordingDestination_File:
@@ -964,7 +1078,7 @@ int RecordingScreenSettings::i_initInternal(void)
             break;
     }
 
-    return vrc;
+    return VINF_SUCCESS;
 }
 
 
@@ -981,7 +1095,7 @@ int RecordingScreenSettings::i_initInternal(void)
 HRESULT RecordingScreenSettings::i_loadSettings(const settings::RecordingScreenSettings &data)
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     AutoReadLock mlock(m->pParent COMMA_LOCKVAL_SRC_POS);
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
@@ -1003,7 +1117,7 @@ HRESULT RecordingScreenSettings::i_saveSettings(settings::RecordingScreenSetting
 
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -1022,11 +1136,11 @@ void RecordingScreenSettings::i_commit(void)
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller peerCaller(m->pPeer);
-    AssertComRCReturnVoid(peerCaller.rc());
+    AssertComRCReturnVoid(peerCaller.hrc());
 
     /* lock both for writing since we modify both (mPeer is "master" so locked
      * first) */
@@ -1050,11 +1164,11 @@ void RecordingScreenSettings::i_copyFrom(RecordingScreenSettings *aThat)
 
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnVoid(thatCaller.rc());
+    AssertComRCReturnVoid(thatCaller.hrc());
 
     /* peer is not modified, lock it for reading (aThat is "master" so locked
      * first) */
@@ -1074,7 +1188,7 @@ void RecordingScreenSettings::i_applyDefaults(void)
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -1085,7 +1199,7 @@ settings::RecordingScreenSettings &RecordingScreenSettings::i_getData(void)
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRC(autoCaller.rc());
+    AssertComRC(autoCaller.hrc());
 
     AssertPtr(m);
     return *m->bd.data();
@@ -1134,81 +1248,3 @@ int32_t RecordingScreenSettings::i_getReferences(void)
 {
     return ASMAtomicReadS32(&m->cRefs);
 }
-
-/**
- * Parses a recording screen options string and stores the parsed result in the specified screen settings.
- *
- * @returns IPRT status code.
- * @param   strOptions          Options string to parse.
- * @param   screenSettings      Where to store the parsed result into.
- */
-/* static */
-int RecordingScreenSettings::i_parseOptionsString(const com::Utf8Str &strOptions,
-                                                  settings::RecordingScreenSettings &screenSettings)
-{
-    /*
-     * Parse options string.
-     */
-    size_t pos = 0;
-    com::Utf8Str key, value;
-    while ((pos = strOptions.parseKeyValue(key, value, pos)) != com::Utf8Str::npos)
-    {
-        if (key.compare("vc_quality", Utf8Str::CaseInsensitive) == 0)
-        {
-#ifdef VBOX_WITH_LIBVPX
-            if (value.compare("realtime", Utf8Str::CaseInsensitive) == 0)
-                mVideoRecCfg.Video.Codec.VPX.uEncoderDeadline = VPX_DL_REALTIME;
-            else if (value.compare("good", Utf8Str::CaseInsensitive) == 0)
-                mVideoRecCfg.Video.Codec.VPX.uEncoderDeadline = 1000000 / mVideoRecCfg.Video.uFPS;
-            else if (value.compare("best", Utf8Str::CaseInsensitive) == 0)
-                mVideoRecCfg.Video.Codec.VPX.uEncoderDeadline = VPX_DL_BEST_QUALITY;
-            else
-            {
-                mVideoRecCfg.Video.Codec.VPX.uEncoderDeadline = value.toUInt32();
-            }
-#endif
-        }
-        else if (key.compare("vc_enabled", Utf8Str::CaseInsensitive) == 0)
-        {
-            if (value.compare("false", Utf8Str::CaseInsensitive) == 0)
-            {
-                screenSettings.featureMap[RecordingFeature_Video] = false;
-            }
-        }
-        else if (key.compare("ac_enabled", Utf8Str::CaseInsensitive) == 0)
-        {
-#ifdef VBOX_WITH_AUDIO_RECORDING
-            if (value.compare("true", Utf8Str::CaseInsensitive) == 0)
-            {
-                screenSettings.featureMap[RecordingFeature_Audio] = true;
-            }
-#endif
-        }
-        else if (key.compare("ac_profile", Utf8Str::CaseInsensitive) == 0)
-        {
-#ifdef VBOX_WITH_AUDIO_RECORDING
-            if (value.compare("low", Utf8Str::CaseInsensitive) == 0)
-            {
-                screenSettings.Audio.uHz       = 8000;
-                screenSettings.Audio.cBits     = 16;
-                screenSettings.Audio.cChannels = 1;
-            }
-            else if (value.startsWith("med" /* "med[ium]" */, Utf8Str::CaseInsensitive) == 0)
-            {
-                /* Stay with the default set above. */
-            }
-            else if (value.compare("high", Utf8Str::CaseInsensitive) == 0)
-            {
-                screenSettings.Audio.uHz       = 48000;
-                screenSettings.Audio.cBits     = 16;
-                screenSettings.Audio.cChannels = 2;
-            }
-#endif
-        }
-        /* else just ignore. */
-
-    } /* while */
-
-    return VINF_SUCCESS;
-}
-
