@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2005-2020 Oracle Corporation
+ * Copyright (C) 2005-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #define LOG_GROUP LOG_GROUP_MAIN_USBDEVICEFILTERS
@@ -264,7 +274,7 @@ public:
         VBOX_TWEAK_INTERFACE_ENTRY(IUSBDeviceFilter)
     END_COM_MAP()
 
-    DECLARE_EMPTY_CTOR_DTOR(USBDeviceFilter)
+    DECLARE_COMMON_CLASS_METHODS(USBDeviceFilter)
 
     // IUSBDeviceFilter properties
     STDMETHOD(COMGETTER(Name))(BSTR *aName);
@@ -298,11 +308,8 @@ HRESULT USBDeviceFilters::getDeviceFilters(std::vector<ComPtr<IUSBDeviceFilter>
 #ifdef VBOX_WITH_USB
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    SafeIfaceArray<IUSBDeviceFilter> collection(*m->llDeviceFilters.data());
-    aDeviceFilters.resize(collection.size());
-    if (collection.size())
-        for (size_t i = 0; i < collection.size(); ++i)
-            aDeviceFilters[i] = collection[i];
+    aDeviceFilters.resize(m->llDeviceFilters.data()->size());
+    std::copy(m->llDeviceFilters.data()->begin(), m->llDeviceFilters.data()->end(), aDeviceFilters.begin());
 
     return S_OK;
 #else
@@ -325,16 +332,16 @@ HRESULT USBDeviceFilters::createDeviceFilter(const com::Utf8Str &aName,
 
     /* the machine needs to be mutable */
     AutoMutableOrSavedOrRunningStateDependency adep(m->pParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     ComObjPtr<USBDeviceFilter> pFilter;
     pFilter.createObject();
-    HRESULT rc = pFilter->init(this, Bstr(aName).raw());
-    ComAssertComRCRetRC(rc);
-    rc = pFilter.queryInterfaceTo(aFilter.asOutParam());
-    AssertComRCReturnRC(rc);
+    HRESULT hrc = pFilter->init(this, Bstr(aName).raw());
+    ComAssertComRCRetRC(hrc);
+    hrc = pFilter.queryInterfaceTo(aFilter.asOutParam());
+    AssertComRCReturnRC(hrc);
 
     return S_OK;
 #else
@@ -352,7 +359,7 @@ HRESULT USBDeviceFilters::insertDeviceFilter(ULONG aPosition,
 
     /* the machine needs to be mutable */
     AutoMutableOrSavedOrRunningStateDependency adep(m->pParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -412,7 +419,7 @@ HRESULT USBDeviceFilters::removeDeviceFilter(ULONG aPosition,
 #ifdef VBOX_WITH_USB
     /* the machine needs to be mutable */
     AutoMutableOrSavedOrRunningStateDependency adep(m->pParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -487,7 +494,7 @@ HRESULT USBDeviceFilters::removeDeviceFilter(ULONG aPosition,
 HRESULT USBDeviceFilters::i_loadSettings(const settings::USB &data)
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     /* Note: we assume that the default values for attributes of optional
      * nodes are assigned in the Data::Data() constructor and don't do it
@@ -508,9 +515,8 @@ HRESULT USBDeviceFilters::i_loadSettings(const settings::USB &data)
         const settings::USBDeviceFilter &f = *it;
         ComObjPtr<USBDeviceFilter> pFilter;
         pFilter.createObject();
-        HRESULT rc = pFilter->init(this,        // parent
-                                   f);
-        if (FAILED(rc)) return rc;
+        HRESULT hrc = pFilter->init(this /*aParent*/, f);
+        if (FAILED(hrc)) return hrc;
 
         m->llDeviceFilters->push_back(pFilter);
         pFilter->mInList = true;
@@ -532,7 +538,7 @@ HRESULT USBDeviceFilters::i_loadSettings(const settings::USB &data)
 HRESULT USBDeviceFilters::i_saveSettings(settings::USB &data)
 {
     AutoCaller autoCaller(this);
-    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    if (FAILED(autoCaller.hrc())) return autoCaller.hrc();
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -581,11 +587,11 @@ HRESULT USBDeviceFilters::i_saveSettings(settings::USB &data)
 void USBDeviceFilters::i_rollback()
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* we need the machine state */
     AutoAnyStateDependency adep(m->pParent);
-    AssertComRCReturnVoid(adep.rc());
+    AssertComRCReturnVoid(adep.hrc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -674,11 +680,11 @@ void USBDeviceFilters::i_commit()
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller peerCaller(m->pPeer);
-    AssertComRCReturnVoid(peerCaller.rc());
+    AssertComRCReturnVoid(peerCaller.hrc());
 
     /* lock both for writing since we modify both (mPeer is "master" so locked
      * first) */
@@ -771,15 +777,15 @@ void USBDeviceFilters::i_copyFrom(USBDeviceFilters *aThat)
 
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnVoid(thatCaller.rc());
+    AssertComRCReturnVoid(thatCaller.hrc());
 
     /* even more sanity */
     AutoAnyStateDependency adep(m->pParent);
-    AssertComRCReturnVoid(adep.rc());
+    AssertComRCReturnVoid(adep.hrc());
     /* Machine::copyFrom() may not be called when the VM is running */
     AssertReturnVoid(!Global::IsOnline(adep.machineState()));
 
@@ -820,11 +826,11 @@ HRESULT USBDeviceFilters::i_onDeviceFilterChange(USBDeviceFilter *aFilter,
                                                  BOOL aActiveChanged /* = FALSE */)
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     /* we need the machine state */
     AutoAnyStateDependency adep(m->pParent);
-    AssertComRCReturnRC(adep.rc());
+    AssertComRCReturnRC(adep.hrc());
 
     /* nothing to do if the machine isn't running */
     if (!Global::IsOnline(adep.machineState()))
@@ -885,7 +891,7 @@ HRESULT USBDeviceFilters::i_onDeviceFilterChange(USBDeviceFilter *aFilter,
 bool USBDeviceFilters::i_hasMatchingFilter(const ComObjPtr<HostUSBDevice> &aDevice, ULONG *aMaskedIfs)
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturn(autoCaller.rc(), false);
+    AssertComRCReturn(autoCaller.hrc(), false);
 
     /* It is not possible to work with USB device if there is no USB controller present. */
     if (!m->pParent->i_isUSBControllerPresent())
@@ -926,7 +932,7 @@ bool USBDeviceFilters::i_hasMatchingFilter(IUSBDevice *aUSBDevice, ULONG *aMaske
     LogFlowThisFuncEnter();
 
     AutoCaller autoCaller(this);
-    AssertComRCReturn(autoCaller.rc(), false);
+    AssertComRCReturn(autoCaller.hrc(), false);
 
     /* It is not possible to work with USB device if there is no USB controller present. */
     if (!m->pParent->i_isUSBControllerPresent())
@@ -934,61 +940,59 @@ bool USBDeviceFilters::i_hasMatchingFilter(IUSBDevice *aUSBDevice, ULONG *aMaske
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    HRESULT rc = S_OK;
-
     /* query fields */
     USBFILTER dev;
     USBFilterInit(&dev, USBFILTERTYPE_CAPTURE);
 
     USHORT vendorId = 0;
-    rc = aUSBDevice->COMGETTER(VendorId)(&vendorId);
-    ComAssertComRCRet(rc, false);
+    HRESULT hrc = aUSBDevice->COMGETTER(VendorId)(&vendorId);
+    ComAssertComRCRet(hrc, false);
     ComAssertRet(vendorId, false);
     int vrc = USBFilterSetNumExact(&dev, USBFILTERIDX_VENDOR_ID, vendorId, true); AssertRC(vrc);
 
     USHORT productId = 0;
-    rc = aUSBDevice->COMGETTER(ProductId)(&productId);
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(ProductId)(&productId);
+    ComAssertComRCRet(hrc, false);
     vrc = USBFilterSetNumExact(&dev, USBFILTERIDX_PRODUCT_ID, productId, true); AssertRC(vrc);
 
     USHORT revision;
-    rc = aUSBDevice->COMGETTER(Revision)(&revision);
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Revision)(&revision);
+    ComAssertComRCRet(hrc, false);
     vrc = USBFilterSetNumExact(&dev, USBFILTERIDX_DEVICE, revision, true); AssertRC(vrc);
 
     Bstr manufacturer;
-    rc = aUSBDevice->COMGETTER(Manufacturer)(manufacturer.asOutParam());
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Manufacturer)(manufacturer.asOutParam());
+    ComAssertComRCRet(hrc, false);
     if (!manufacturer.isEmpty())
         USBFilterSetStringExact(&dev, USBFILTERIDX_MANUFACTURER_STR, Utf8Str(manufacturer).c_str(),
                                 true /*fMustBePresent*/, false /*fPurge*/);
 
     Bstr product;
-    rc = aUSBDevice->COMGETTER(Product)(product.asOutParam());
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Product)(product.asOutParam());
+    ComAssertComRCRet(hrc, false);
     if (!product.isEmpty())
         USBFilterSetStringExact(&dev, USBFILTERIDX_PRODUCT_STR, Utf8Str(product).c_str(),
                                 true /*fMustBePresent*/, false /*fPurge*/);
 
     Bstr serialNumber;
-    rc = aUSBDevice->COMGETTER(SerialNumber)(serialNumber.asOutParam());
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(SerialNumber)(serialNumber.asOutParam());
+    ComAssertComRCRet(hrc, false);
     if (!serialNumber.isEmpty())
         USBFilterSetStringExact(&dev, USBFILTERIDX_SERIAL_NUMBER_STR, Utf8Str(serialNumber).c_str(),
                                 true /*fMustBePresent*/, false /*fPurge*/);
 
     Bstr address;
-    rc = aUSBDevice->COMGETTER(Address)(address.asOutParam());
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Address)(address.asOutParam());
+    ComAssertComRCRet(hrc, false);
 
     USHORT port = 0;
-    rc = aUSBDevice->COMGETTER(Port)(&port);
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Port)(&port);
+    ComAssertComRCRet(hrc, false);
     USBFilterSetNumExact(&dev, USBFILTERIDX_PORT, port, true);
 
     BOOL remote = FALSE;
-    rc = aUSBDevice->COMGETTER(Remote)(&remote);
-    ComAssertComRCRet(rc, false);
+    hrc = aUSBDevice->COMGETTER(Remote)(&remote);
+    ComAssertComRCRet(hrc, false);
     ComAssertRet(remote == TRUE, false);
 
     bool match = false;
@@ -1032,7 +1036,7 @@ HRESULT USBDeviceFilters::i_notifyProxy(bool aInsertFilters)
     LogFlowThisFunc(("aInsertFilters=%RTbool\n", aInsertFilters));
 
     AutoCaller autoCaller(this);
-    AssertComRCReturn(autoCaller.rc(), false);
+    AssertComRCReturn(autoCaller.hrc(), false);
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
