@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2022 Oracle and/or its affiliates.
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -131,7 +131,7 @@ HRESULT VRDEServer::init(Machine *aParent, VRDEServer *aThat)
     unconst(mPeer) = aThat;
 
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnRC(thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.hrc());
 
     AutoReadLock thatLock(aThat COMMA_LOCKVAL_SRC_POS);
     mData.share(aThat->mData);
@@ -163,7 +163,7 @@ HRESULT VRDEServer::initCopy(Machine *aParent, VRDEServer *aThat)
     /* mPeer is left null */
 
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnRC(thatCaller.rc());
+    AssertComRCReturnRC(thatCaller.hrc());
 
     AutoReadLock thatLock(aThat COMMA_LOCKVAL_SRC_POS);
     mData.attachCopy(aThat->mData);
@@ -206,7 +206,7 @@ HRESULT VRDEServer::i_loadSettings(const settings::VRDESettings &data)
     using namespace settings;
 
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
     mData.assignCopy(&data);
@@ -224,7 +224,7 @@ HRESULT VRDEServer::i_loadSettings(const settings::VRDESettings &data)
 HRESULT VRDEServer::i_saveSettings(settings::VRDESettings &data)
 {
     AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    AssertComRCReturnRC(autoCaller.hrc());
 
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
     data = *mData.data();
@@ -248,11 +248,11 @@ HRESULT VRDEServer::setEnabled(BOOL aEnabled)
 {
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    HRESULT rc = S_OK;
+    HRESULT hrc = S_OK;
 
     if (mData->fEnabled != RT_BOOL(aEnabled))
     {
@@ -269,12 +269,12 @@ HRESULT VRDEServer::setEnabled(BOOL aEnabled)
         /* Avoid deadlock when i_onVRDEServerChange eventually calls SetExtraData. */
         adep.release();
 
-        rc = mParent->i_onVRDEServerChange(/* aRestart */ TRUE);
-        if (FAILED(rc))
+        hrc = mParent->i_onVRDEServerChange(/* aRestart */ TRUE);
+        if (FAILED(hrc))
         {
             /* Failed to enable/disable the server. Revert the internal state. */
             adep.add();
-            if (SUCCEEDED(adep.rc()))
+            if (SUCCEEDED(adep.hrc()))
             {
                 alock.acquire();
                 mData->fEnabled = !RT_BOOL(aEnabled);
@@ -285,7 +285,7 @@ HRESULT VRDEServer::setEnabled(BOOL aEnabled)
         }
     }
 
-    return rc;
+    return hrc;
 }
 
 static int i_portParseNumber(uint16_t *pu16Port, const char *pszStart, const char *pszEnd)
@@ -353,18 +353,18 @@ static int i_vrdpServerVerifyPortsString(const com::Utf8Str &aPortRange)
         }
 
         /* A probably valid range. Verify and parse it. */
-        int rc;
+        int vrc;
         if (pszDash)
         {
-            rc = i_portParseNumber(NULL, pszStart, pszDash);
-            if (RT_SUCCESS(rc))
-                rc = i_portParseNumber(NULL, pszDash + 1, pszEnd);
+            vrc = i_portParseNumber(NULL, pszStart, pszDash);
+            if (RT_SUCCESS(vrc))
+                vrc = i_portParseNumber(NULL, pszDash + 1, pszEnd);
         }
         else
-            rc = i_portParseNumber(NULL, pszStart, pszEnd);
+            vrc = i_portParseNumber(NULL, pszStart, pszEnd);
 
-        if (RT_FAILURE(rc))
-            return rc;
+        if (RT_FAILURE(vrc))
+            return vrc;
     }
 
     return VINF_SUCCESS;
@@ -376,7 +376,7 @@ HRESULT VRDEServer::setVRDEProperty(const com::Utf8Str &aKey, const com::Utf8Str
 
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -475,37 +475,35 @@ static int loadVRDELibrary(const char *pszLibraryName, RTLDRMOD *phmod, void *pp
 static int loadVRDELibrary(const char *pszLibraryName, RTLDRMOD *phmod, PFNVRDESUPPORTEDPROPERTIES *ppfn)
 #endif
 {
-    int rc = VINF_SUCCESS;
+    int vrc = VINF_SUCCESS;
 
     RTLDRMOD hmod = NIL_RTLDRMOD;
 
     RTERRINFOSTATIC ErrInfo;
     RTErrInfoInitStatic(&ErrInfo);
     if (RTPathHavePath(pszLibraryName))
-        rc = SUPR3HardenedLdrLoadPlugIn(pszLibraryName, &hmod, &ErrInfo.Core);
+        vrc = SUPR3HardenedLdrLoadPlugIn(pszLibraryName, &hmod, &ErrInfo.Core);
     else
-        rc = SUPR3HardenedLdrLoadAppPriv(pszLibraryName, &hmod, RTLDRLOAD_FLAGS_LOCAL, &ErrInfo.Core);
-    if (RT_SUCCESS(rc))
+        vrc = SUPR3HardenedLdrLoadAppPriv(pszLibraryName, &hmod, RTLDRLOAD_FLAGS_LOCAL, &ErrInfo.Core);
+    if (RT_SUCCESS(vrc))
     {
-        rc = RTLdrGetSymbol(hmod, "VRDESupportedProperties", (void **)ppfn);
+        vrc = RTLdrGetSymbol(hmod, "VRDESupportedProperties", (void **)ppfn);
 
-        if (RT_FAILURE(rc) && rc != VERR_SYMBOL_NOT_FOUND)
-            LogRel(("VRDE: Error resolving symbol '%s', rc %Rrc.\n", "VRDESupportedProperties", rc));
+        if (RT_FAILURE(vrc) && vrc != VERR_SYMBOL_NOT_FOUND)
+            LogRel(("VRDE: Error resolving symbol '%s', vrc %Rrc.\n", "VRDESupportedProperties", vrc));
     }
     else
     {
         if (RTErrInfoIsSet(&ErrInfo.Core))
-            LogRel(("VRDE: Error loading the library '%s': %s (%Rrc)\n", pszLibraryName, ErrInfo.Core.pszMsg, rc));
+            LogRel(("VRDE: Error loading the library '%s': %s (%Rrc)\n", pszLibraryName, ErrInfo.Core.pszMsg, vrc));
         else
-            LogRel(("VRDE: Error loading the library '%s' rc = %Rrc.\n", pszLibraryName, rc));
+            LogRel(("VRDE: Error loading the library '%s' vrc = %Rrc.\n", pszLibraryName, vrc));
 
         hmod = NIL_RTLDRMOD;
     }
 
-    if (RT_SUCCESS(rc))
-    {
+    if (RT_SUCCESS(vrc))
         *phmod = hmod;
-    }
     else
     {
         if (hmod != NIL_RTLDRMOD)
@@ -515,7 +513,7 @@ static int loadVRDELibrary(const char *pszLibraryName, RTLDRMOD *phmod, PFNVRDES
         }
     }
 
-    return rc;
+    return vrc;
 }
 
 HRESULT VRDEServer::getVRDEProperties(std::vector<com::Utf8Str> &aProperties)
@@ -556,7 +554,7 @@ HRESULT VRDEServer::getVRDEProperties(std::vector<com::Utf8Str> &aProperties)
         vrc = VERR_FILE_NOT_FOUND;
 #endif
     }
-    Log(("VRDEPROP: library get rc %Rrc\n", vrc));
+    Log(("VRDEPROP: library get vrc %Rrc\n", vrc));
 
     if (RT_SUCCESS(vrc))
     {
@@ -570,7 +568,7 @@ HRESULT VRDEServer::getVRDEProperties(std::vector<com::Utf8Str> &aProperties)
 #else
         vrc = loadVRDELibrary(strVrdeLibrary.c_str(), &hmod, &pfn);
 #endif
-        Log(("VRDEPROP: load library [%s] rc %Rrc\n", strVrdeLibrary.c_str(), vrc));
+        Log(("VRDEPROP: load library [%s] vrc %Rrc\n", strVrdeLibrary.c_str(), vrc));
         if (RT_SUCCESS(vrc))
         {
             const char * const *papszNames = pfn();
@@ -622,7 +620,7 @@ HRESULT VRDEServer::setAuthType(AuthType_T aType)
 {
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -658,7 +656,7 @@ HRESULT VRDEServer::setAuthTimeout(ULONG aTimeout)
 {
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -715,7 +713,7 @@ HRESULT VRDEServer::setAuthLibrary(const com::Utf8Str &aLibrary)
 {
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -752,7 +750,7 @@ HRESULT VRDEServer::setAllowMultiConnection(BOOL aAllowMultiConnection)
 {
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -787,7 +785,7 @@ HRESULT VRDEServer::getReuseSingleConnection(BOOL *aReuseSingleConnection)
 HRESULT VRDEServer::setReuseSingleConnection(BOOL aReuseSingleConnection)
 {
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    if (FAILED(adep.rc())) return adep.rc();
+    if (FAILED(adep.hrc())) return adep.hrc();
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -855,7 +853,7 @@ HRESULT VRDEServer::setVRDEExtPack(const com::Utf8Str &aExtPack)
     HRESULT hrc = S_OK;
     /* the machine can also be in saved state for this property to change */
     AutoMutableOrSavedOrRunningStateDependency adep(mParent);
-    hrc = adep.rc();
+    hrc = adep.hrc();
     if (SUCCEEDED(hrc))
     {
         /*
@@ -912,7 +910,7 @@ void VRDEServer::i_rollback()
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
@@ -927,11 +925,11 @@ void VRDEServer::i_commit()
 {
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller peerCaller(mPeer);
-    AssertComRCReturnVoid(peerCaller.rc());
+    AssertComRCReturnVoid(peerCaller.hrc());
 
     /* lock both for writing since we modify both (mPeer is "master" so locked
      * first) */
@@ -958,11 +956,11 @@ void VRDEServer::i_copyFrom(VRDEServer *aThat)
 
     /* sanity */
     AutoCaller autoCaller(this);
-    AssertComRCReturnVoid(autoCaller.rc());
+    AssertComRCReturnVoid(autoCaller.hrc());
 
     /* sanity too */
     AutoCaller thatCaller(aThat);
-    AssertComRCReturnVoid(thatCaller.rc());
+    AssertComRCReturnVoid(thatCaller.hrc());
 
     /* peer is not modified, lock it for reading (aThat is "master" so locked
      * first) */
