@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2008-2020 Oracle Corporation
+ * Copyright (C) 2008-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /*
@@ -113,7 +123,7 @@ static int getDefaultIfaceIndex(unsigned short *pu16Index, int family)
         {
             Log(("getDefaultIfaceIndex: Got message %u while expecting %u.\n",
                  pRtMsg->rtm_type, RTM_GET));
-            //rc = VERR_INTERNAL_ERROR;
+            //vrc = VERR_INTERNAL_ERROR;
             continue;
         }
         if ((char*)(pRtMsg + 1) < pEnd)
@@ -201,7 +211,7 @@ static bool isWireless(const char *pszName)
 
 int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
 {
-    int rc = VINF_SUCCESS;
+    int vrc = VINF_SUCCESS;
     size_t cbNeeded;
     char *pBuf, *pNext;
     int aiMib[6];
@@ -209,11 +219,11 @@ int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
     bool fDefaultIfaceExistent = true;
 
     /* Get the index of the interface associated with default route. */
-    rc = getDefaultIfaceIndex(&u16DefaultIface, PF_INET);
-    if (RT_FAILURE(rc))
+    vrc = getDefaultIfaceIndex(&u16DefaultIface, PF_INET);
+    if (RT_FAILURE(vrc))
     {
         fDefaultIfaceExistent = false;
-        rc = VINF_SUCCESS;
+        vrc = VINF_SUCCESS;
     }
 
     aiMib[0] = CTL_NET;
@@ -254,7 +264,7 @@ int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
         {
             Log(("NetIfList: Got message %u while expecting %u.\n",
                  pIfMsg->ifm_type, RTM_IFINFO));
-            rc = VERR_INTERNAL_ERROR;
+            vrc = VERR_INTERNAL_ERROR;
             break;
         }
         struct sockaddr_dl *pSdl = (struct sockaddr_dl *)(pIfMsg + 1);
@@ -263,7 +273,7 @@ int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
         PNETIFINFO pNew = (PNETIFINFO)RTMemAllocZ(RT_UOFFSETOF_DYN(NETIFINFO, szName[cbNameLen]));
         if (!pNew)
         {
-            rc = VERR_NO_MEMORY;
+            vrc = VERR_NO_MEMORY;
             break;
         }
         memcpy(pNew->MACAddress.au8, LLADDR(pSdl), sizeof(pNew->MACAddress.au8));
@@ -331,14 +341,12 @@ int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
 
     close(sock);
     free(pBuf);
-    return rc;
-
-
+    return vrc;
 }
 
 int NetIfGetConfigByName(PNETIFINFO pInfo)
 {
-    int rc = VINF_SUCCESS;
+    int vrc = VINF_SUCCESS;
     size_t cbNeeded;
     char *pBuf, *pNext;
     int aiMib[6];
@@ -381,7 +389,7 @@ int NetIfGetConfigByName(PNETIFINFO pInfo)
         {
             Log(("NetIfList: Got message %u while expecting %u.\n",
                  pIfMsg->ifm_type, RTM_IFINFO));
-            rc = VERR_INTERNAL_ERROR;
+            vrc = VERR_INTERNAL_ERROR;
             break;
         }
         struct sockaddr_dl *pSdl = (struct sockaddr_dl *)(pIfMsg + 1);
@@ -432,7 +440,7 @@ int NetIfGetConfigByName(PNETIFINFO pInfo)
     }
     close(sock);
     free(pBuf);
-    return rc;
+    return vrc;
 }
 
 /**
