@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -168,7 +178,11 @@ static DECLCALLBACK(int) rtCrPkixSignatureRsa_Init(PCRTCRPKIXSIGNATUREDESC pDesc
 {
     RT_NOREF_PV(pDesc); RT_NOREF_PV(pvState); RT_NOREF_PV(pvOpaque);
 
-    if (pParams)
+    if (   !pParams
+        || pParams->enmType == RTASN1TYPE_NULL
+        || pParams->enmType == RTASN1TYPE_NOT_PRESENT)
+    { /* likely */ }
+    else
         return VERR_CR_PKIX_SIGNATURE_TAKES_NO_PARAMETERS;
 
     RTCRKEYTYPE enmKeyType = RTCrKeyGetType(hKey);
@@ -468,6 +482,10 @@ static const char * const g_apszHashWithRsaAliases[] =
     RTCR_PKCS1_SHA224_WITH_RSA_OID,
     RTCR_PKCS1_SHA512T224_WITH_RSA_OID,
     RTCR_PKCS1_SHA512T256_WITH_RSA_OID,
+    RTCR_NIST_SHA3_224_WITH_RSA_OID,
+    RTCR_NIST_SHA3_256_WITH_RSA_OID,
+    RTCR_NIST_SHA3_384_WITH_RSA_OID,
+    RTCR_NIST_SHA3_512_WITH_RSA_OID,
     /* Note: Note quite sure about these OIW oddballs. */
     "1.3.14.3.2.11" /* OIW rsaSignature */,
     "1.3.14.3.2.14" /* OIW mdc2WithRSASignature */,
