@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2013-2017 Oracle Corporation
+ * Copyright (C) 2013-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -132,25 +142,6 @@ typedef struct RTDBGCFGU64MNEMONIC
 typedef RTDBGCFGU64MNEMONIC const *PCRTDBGCFGU64MNEMONIC;
 
 
-/** @name Open flags.
- * @{ */
-/** The operative system mask.  The values are RT_OPSYS_XXX. */
-#define RTDBGCFG_O_OPSYS_MASK           UINT32_C(0x000000ff)
-/** The files may be compressed MS styled. */
-#define RTDBGCFG_O_MAYBE_COMPRESSED_MS  RT_BIT_32(26)
-/** Whether to make a recursive search. */
-#define RTDBGCFG_O_RECURSIVE            RT_BIT_32(27)
-/** We're looking for a separate debug file. */
-#define RTDBGCFG_O_EXT_DEBUG_FILE       RT_BIT_32(28)
-/** We're looking for an executable image. */
-#define RTDBGCFG_O_EXECUTABLE_IMAGE     RT_BIT_32(29)
-/** The file search should be done in an case insensitive fashion. */
-#define RTDBGCFG_O_CASE_INSENSITIVE     RT_BIT_32(30)
-/** Use Windbg style symbol servers when encountered in the path. */
-#define RTDBGCFG_O_SYMSRV               RT_BIT_32(31)
-/** @} */
-
-
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
@@ -630,6 +621,7 @@ static int rtDbgCfgTryOpenDir(PRTDBGCFGINT pThis, char *pszPath, PRTPATHSPLIT pS
      * Do a recursive search if requested.
      */
     if (   (fFlags & RTDBGCFG_O_RECURSIVE)
+        && pThis
         && !(pThis->fFlags & RTDBGCFG_FLAGS_NO_RECURSIV_SEARCH) )
     {
         /** @todo Recursive searching will be done later. */
@@ -738,7 +730,6 @@ static int rtDbgCfgTryDownloadAndOpen(PRTDBGCFGINT pThis, const char *pszServer,
                                       PRTPATHSPLIT pSplitFn, const char *pszCacheSuffix, uint32_t fFlags,
                                       PFNRTDBGCFGOPEN pfnCallback, void *pvUser1, void *pvUser2)
 {
-#ifdef IPRT_WITH_HTTP
     RT_NOREF_PV(pszUuidMappingSubDir); /** @todo do we bother trying pszUuidMappingSubDir? */
     RT_NOREF_PV(pszCacheSuffix); /** @todo do we bother trying pszUuidMappingSubDir? */
     RT_NOREF_PV(fFlags);
@@ -747,6 +738,9 @@ static int rtDbgCfgTryDownloadAndOpen(PRTDBGCFGINT pThis, const char *pszServer,
         return VWRN_NOT_FOUND;
     if (!pszCacheSubDir || !*pszCacheSubDir)
         return VWRN_NOT_FOUND;
+    if (   !(fFlags & RTDBGCFG_O_SYMSRV)
+        && !(fFlags & RTDBGCFG_O_DEBUGINFOD))
+        return VWRN_NOT_FOUND;
 
     /*
      * Create the path.
@@ -794,56 +788,115 @@ static int rtDbgCfgTryDownloadAndOpen(PRTDBGCFGINT pThis, const char *pszServer,
         return rc;
 
     /*
-     * Download the file.
+     * Download/copy the file.
      */
-    RTHTTP hHttp;
-    rc = RTHttpCreate(&hHttp);
-    if (RT_FAILURE(rc))
-        return rc;
-    RTHttpUseSystemProxySettings(hHttp);
-
-    static const char * const s_apszHeaders[] =
+    char szUrl[_2K];
+    /* Download URL? */
+    if (   RTStrIStartsWith(pszServer, "http://")
+        || RTStrIStartsWith(pszServer, "https://")
+        || RTStrIStartsWith(pszServer, "ftp://") )
     {
-        "User-Agent: Microsoft-Symbol-Server/6.6.0999.9",
-        "Pragma: no-cache",
-    };
+#ifdef IPRT_WITH_HTTP
+        RTHTTP hHttp;
+        rc = RTHttpCreate(&hHttp);
+        if (RT_SUCCESS(rc))
+        {
+            RTHttpUseSystemProxySettings(hHttp);
             RTHttpSetFollowRedirects(hHttp, 8);
 
-    rc = RTHttpSetHeaders(hHttp, RT_ELEMENTS(s_apszHeaders), s_apszHeaders);
-    if (RT_SUCCESS(rc))
+            static const char * const s_apszHeadersMsSymSrv[] =
+            {
+                "User-Agent: Microsoft-Symbol-Server/6.6.0999.9",
+                "Pragma: no-cache",
+            };
+
+            static const char * const s_apszHeadersDebuginfod[] =
+            {
+                "User-Agent: IPRT DbgCfg 1.0",
+                "Pragma: no-cache",
+            };
+
+            if (fFlags & RTDBGCFG_O_SYMSRV)
+                rc = RTHttpSetHeaders(hHttp, RT_ELEMENTS(s_apszHeadersMsSymSrv), s_apszHeadersMsSymSrv);
+            else /* Must be debuginfod. */
+                rc = RTHttpSetHeaders(hHttp, RT_ELEMENTS(s_apszHeadersDebuginfod), s_apszHeadersDebuginfod);
+            if (RT_SUCCESS(rc))
+            {
+                if (fFlags & RTDBGCFG_O_SYMSRV)
+                    RTStrPrintf(szUrl, sizeof(szUrl), "%s/%s/%s/%s", pszServer, pszFilename, pszCacheSubDir, pszFilename);
+                else
+                    RTStrPrintf(szUrl, sizeof(szUrl), "%s/buildid/%s/debuginfo", pszServer, pszCacheSubDir);
+
+                /** @todo Use some temporary file name and rename it after the operation
+                 *        since not all systems support read-deny file sharing
+                 *        settings. */
+                rtDbgCfgLog2(pThis, "Downloading '%s' to '%s'...\n", szUrl, pszPath);
+                rc = RTHttpGetFile(hHttp, szUrl, pszPath);
+                if (RT_FAILURE(rc))
+                {
+                    RTFileDelete(pszPath);
+                    rtDbgCfgLog1(pThis, "%Rrc on URL '%s'\n", rc, szUrl);
+                }
+                if (   rc == VERR_HTTP_NOT_FOUND
+                    && (fFlags & RTDBGCFG_O_SYMSRV))
+                {
+                    /* Try the compressed version of the file. */
+                    pszPath[strlen(pszPath) - 1] = '_';
+                    szUrl[strlen(szUrl)     - 1] = '_';
+                    rtDbgCfgLog2(pThis, "Downloading '%s' to '%s'...\n", szUrl, pszPath);
+                    rc = RTHttpGetFile(hHttp, szUrl, pszPath);
+                    if (RT_SUCCESS(rc))
+                        rc = rtDbgCfgUnpackMsCacheFile(pThis, pszPath, pszFilename);
+                    else
+                    {
+                        rtDbgCfgLog1(pThis, "%Rrc on URL '%s'\n", rc, pszPath);
+                        RTFileDelete(pszPath);
+                    }
+                }
+            }
+
+            RTHttpDestroy(hHttp);
+        }
+#else
+        rc = VWRN_NOT_FOUND;
+#endif
+    }
+    /* No download, assume dir on server share. */
+    else
     {
-        char szUrl[_2K];
-        RTStrPrintf(szUrl, sizeof(szUrl), "%s/%s/%s/%s", pszServer, pszFilename, pszCacheSubDir, pszFilename);
+        if (RTStrIStartsWith(pszServer, "file:///"))
+            pszServer += 4 + 1 + 3 - 1;
 
-        /** @todo Use some temporary file name and rename it after the operation
-         *        since not all systems support read-deny file sharing
-         *        settings. */
-        rtDbgCfgLog2(pThis, "Downloading '%s' to '%s'...\n", szUrl, pszPath);
-        rc = RTHttpGetFile(hHttp, szUrl, pszPath);
-        if (RT_FAILURE(rc))
-        {
-            RTFileDelete(pszPath);
-            rtDbgCfgLog1(pThis, "%Rrc on URL '%s'\n", rc, szUrl);
-        }
-        if (rc == VERR_HTTP_NOT_FOUND)
+        /* Compose the path to the uncompressed file on the server. */
+        rc = RTPathJoin(szUrl, sizeof(szUrl), pszServer, pszFilename);
+        if (RT_SUCCESS(rc))
+            rc = RTPathAppend(szUrl, sizeof(szUrl), pszCacheSubDir);
+        if (RT_SUCCESS(rc))
+            rc = RTPathAppend(szUrl, sizeof(szUrl), pszFilename);
+        if (RT_SUCCESS(rc))
         {
-            /* Try the compressed version of the file. */
-            pszPath[strlen(pszPath) - 1] = '_';
-            szUrl[strlen(szUrl)     - 1] = '_';
-            rtDbgCfgLog2(pThis, "Downloading '%s' to '%s'...\n", szUrl, pszPath);
-            rc = RTHttpGetFile(hHttp, szUrl, pszPath);
-            if (RT_SUCCESS(rc))
-                rc = rtDbgCfgUnpackMsCacheFile(pThis, pszPath, pszFilename);
-            else
+            rtDbgCfgLog2(pThis, "Copying '%s' to '%s'...\n", szUrl, pszPath);
+            rc = RTFileCopy(szUrl, pszPath);
+            if (RT_FAILURE(rc))
             {
-                rtDbgCfgLog1(pThis, "%Rrc on URL '%s'\n", rc, pszPath);
                 RTFileDelete(pszPath);
+                rtDbgCfgLog1(pThis, "%Rrc on '%s'\n", rc, szUrl);
+
+                /* Try the compressed version. */
+                pszPath[strlen(pszPath) - 1] = '_';
+                szUrl[strlen(szUrl)     - 1] = '_';
+                rtDbgCfgLog2(pThis, "Copying '%s' to '%s'...\n", szUrl, pszPath);
+                rc = RTFileCopy(szUrl, pszPath);
+                if (RT_SUCCESS(rc))
+                    rc = rtDbgCfgUnpackMsCacheFile(pThis, pszPath, pszFilename);
+                else
+                {
+                    rtDbgCfgLog1(pThis, "%Rrc on '%s'\n", rc, pszPath);
+                    RTFileDelete(pszPath);
+                }
             }
         }
     }
-
-    RTHttpDestroy(hHttp);
-
     if (RT_SUCCESS(rc))
     {
         /*
@@ -869,13 +922,6 @@ static int rtDbgCfgTryDownloadAndOpen(PRTDBGCFGINT pThis, const char *pszServer,
     }
 
     return rc;
-
-#else  /* !IPRT_WITH_HTTP */
-    RT_NOREF_PV(pThis); RT_NOREF_PV(pszServer); RT_NOREF_PV(pszPath); RT_NOREF_PV(pszCacheSubDir);
-    RT_NOREF_PV(pszUuidMappingSubDir); RT_NOREF_PV(pSplitFn); RT_NOREF_PV(pszCacheSuffix); RT_NOREF_PV(fFlags);
-    RT_NOREF_PV(pfnCallback); RT_NOREF_PV(pvUser1); RT_NOREF_PV(pvUser2);
-    return VWRN_NOT_FOUND;
-#endif /* !IPRT_WITH_HTTP */
 }
 
 
@@ -1168,15 +1214,16 @@ static int rtDbgCfgOpenWithSubDir(RTDBGCFG hDbgCfg, const char *pszFilename, con
     AssertPtrReturn(pszFilename, VERR_INVALID_POINTER);
     AssertPtrReturn(pszCacheSubDir, VERR_INVALID_POINTER);
     AssertPtrReturn(pfnCallback, VERR_INVALID_POINTER);
+    AssertReturn(!(fFlags & ~RTDBGCFG_O_VALID_MASK), VERR_INVALID_FLAGS);
 
     /*
      * Do some guessing as to the way we should parse the filename and whether
      * it's case exact or not.
      */
-    bool fDosPath = strchr(pszFilename, ':')  != NULL
-                 || strchr(pszFilename, '\\') != NULL
-                 || RT_OPSYS_USES_DOS_PATHS(fFlags & RTDBGCFG_O_OPSYS_MASK)
-                 || (fFlags & RTDBGCFG_O_CASE_INSENSITIVE);
+    bool fDosPath = RT_OPSYS_USES_DOS_PATHS(fFlags & RTDBGCFG_O_OPSYS_MASK)
+                 || (fFlags & RTDBGCFG_O_CASE_INSENSITIVE)
+                 || strchr(pszFilename, ':')  != NULL
+                 || strchr(pszFilename, '\\') != NULL;
     if (fDosPath)
         fFlags |= RTDBGCFG_O_CASE_INSENSITIVE;
 
@@ -1243,6 +1290,7 @@ static int rtDbgCfgOpenWithSubDir(RTDBGCFG hDbgCfg, const char *pszFilename, con
                 if (   rc2 != VINF_CALLBACK_RETURN
                     && rc2 != VERR_CALLBACK_RETURN
                     && (fFlags & RTDBGCFG_O_EXECUTABLE_IMAGE)
+                    && !(fFlags & RTDBGCFG_O_NO_SYSTEM_PATHS)
                     && !(pThis->fFlags & RTDBGCFG_FLAGS_NO_SYSTEM_PATHS) )
                 {
                     rc2 = rtDbgCfgTryOpenList(pThis, &pThis->NtExecutablePathList, pSplitFn, pszCacheSubDir,
@@ -1253,6 +1301,7 @@ static int rtDbgCfgOpenWithSubDir(RTDBGCFG hDbgCfg, const char *pszFilename, con
 
                 if (   rc2 != VINF_CALLBACK_RETURN
                     && rc2 != VERR_CALLBACK_RETURN
+                    && !(fFlags & RTDBGCFG_O_NO_SYSTEM_PATHS)
                     && !(pThis->fFlags & RTDBGCFG_FLAGS_NO_SYSTEM_PATHS) )
                 {
                     rc2 = rtDbgCfgTryOpenList(pThis, &pThis->NtSymbolPathList, pSplitFn, pszCacheSubDir,
@@ -1278,6 +1327,17 @@ static int rtDbgCfgOpenWithSubDir(RTDBGCFG hDbgCfg, const char *pszFilename, con
 }
 
 
+RTDECL(int) RTDbgCfgOpenEx(RTDBGCFG hDbgCfg, const char *pszFilename, const char *pszCacheSubDir,
+                           const char *pszUuidMappingSubDir, uint32_t fFlags,
+                           PFNRTDBGCFGOPEN pfnCallback, void *pvUser1, void *pvUser2)
+{
+    return rtDbgCfgOpenWithSubDir(hDbgCfg, pszFilename, pszCacheSubDir, pszUuidMappingSubDir, fFlags,
+                                  pfnCallback, pvUser1, pvUser2);
+}
+
+
+
+
 RTDECL(int) RTDbgCfgOpenPeImage(RTDBGCFG hDbgCfg, const char *pszFilename, uint32_t cbImage, uint32_t uTimestamp,
                                 PFNRTDBGCFGOPEN pfnCallback, void *pvUser1, void *pvUser2)
 {
@@ -1356,6 +1416,23 @@ RTDECL(int) RTDbgCfgOpenDwo(RTDBGCFG hDbgCfg, const char *pszFilename, uint32_t
 }
 
 
+RTDECL(int) RTDbgCfgOpenDwoBuildId(RTDBGCFG hDbgCfg, const char *pszFilename, const uint8_t *pbBuildId,
+                                   size_t cbBuildId, PFNRTDBGCFGOPEN pfnCallback, void *pvUser1, void *pvUser2)
+{
+    char *pszSubDir = NULL;
+    int rc = RTStrAPrintf(&pszSubDir, "%#.*Rhxs", cbBuildId, pbBuildId);
+    if (RT_SUCCESS(rc))
+    {
+        rc = rtDbgCfgOpenWithSubDir(hDbgCfg, pszFilename, pszSubDir, NULL,
+                                    RTDBGCFG_O_DEBUGINFOD | RT_OPSYS_UNKNOWN | RTDBGCFG_O_EXT_DEBUG_FILE,
+                                    pfnCallback, pvUser1, pvUser2);
+        RTStrFree(pszSubDir);
+    }
+
+    return rc;
+}
+
+
 
 /*
  *
@@ -1878,6 +1955,7 @@ static int rtDbgCfgChangeStringList(PRTDBGCFGINT pThis, RTDBGCFGOP enmOp, const
     if (enmOp == RTDBGCFGOP_SET)
         rtDbgCfgFreeStrList(pList);
 
+    PRTLISTNODE pPrependTo = pList;
     while (*pszValue)
     {
         /* Skip separators. */
@@ -1926,7 +2004,10 @@ static int rtDbgCfgChangeStringList(PRTDBGCFGINT pThis, RTDBGCFGOP enmOp, const
             pNew->sz[cchPath] = '\0';
 
             if (enmOp == RTDBGCFGOP_PREPEND)
-                RTListPrepend(pList, &pNew->ListEntry);
+            {
+                RTListNodeInsertAfter(pPrependTo, &pNew->ListEntry);
+                pPrependTo = &pNew->ListEntry;
+            }
             else
                 RTListAppend(pList, &pNew->ListEntry);
         }
