@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -83,7 +93,7 @@ static void *rtSgBufGet(PRTSGBUF pSgBuf, size_t *pcbData)
 RTDECL(void) RTSgBufInit(PRTSGBUF pSgBuf, PCRTSGSEG paSegs, size_t cSegs)
 {
     AssertPtr(pSgBuf);
-    Assert(   (cSegs > 0 && VALID_PTR(paSegs))
+    Assert(   (cSegs > 0 && RT_VALID_PTR(paSegs))
            || (!cSegs && !paSegs));
     Assert(cSegs < (~(unsigned)0 >> 1));
 
@@ -346,6 +356,56 @@ RTDECL(size_t) RTSgBufCopyFromBuf(PRTSGBUF pSgBuf, const void *pvBuf, size_t cbC
 }
 
 
+RTDECL(size_t) RTSgBufCopyToFn(PRTSGBUF pSgBuf, size_t cbCopy, PFNRTSGBUFCOPYTO pfnCopyTo, void *pvUser)
+{
+    AssertPtrReturn(pSgBuf, 0);
+    AssertPtrReturn(pfnCopyTo, 0);
+
+    size_t cbLeft = cbCopy;
+
+    while (cbLeft)
+    {
+        size_t cbThisCopy = cbLeft;
+        void *pvSrc = rtSgBufGet(pSgBuf, &cbThisCopy);
+
+        if (!cbThisCopy)
+            break;
+
+        size_t cbThisCopied = pfnCopyTo(pSgBuf, pvSrc, cbThisCopy, pvUser);
+        cbLeft -= cbThisCopied;
+        if (cbThisCopied < cbThisCopy)
+            break;
+    }
+
+    return cbCopy - cbLeft;
+}
+
+
+RTDECL(size_t) RTSgBufCopyFromFn(PRTSGBUF pSgBuf, size_t cbCopy, PFNRTSGBUFCOPYFROM pfnCopyFrom, void *pvUser)
+{
+    AssertPtrReturn(pSgBuf, 0);
+    AssertPtrReturn(pfnCopyFrom, 0);
+
+    size_t cbLeft = cbCopy;
+
+    while (cbLeft)
+    {
+        size_t cbThisCopy = cbLeft;
+        void *pvDst = rtSgBufGet(pSgBuf, &cbThisCopy);
+
+        if (!cbThisCopy)
+            break;
+
+        size_t cbThisCopied = pfnCopyFrom(pSgBuf, pvDst, cbThisCopy, pvUser);
+        cbLeft -= cbThisCopied;
+        if (cbThisCopied < cbThisCopy)
+            break;
+    }
+
+    return cbCopy - cbLeft;
+}
+
+
 RTDECL(size_t) RTSgBufAdvance(PRTSGBUF pSgBuf, size_t cbAdvance)
 {
     AssertPtrReturn(pSgBuf, 0);
