@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -72,19 +82,21 @@
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
-/** The AVL thread containing the threads. */
-static PAVLPVNODECORE       g_ThreadTree;
-/** The number of threads in the tree (for ring-0 termination kludge). */
-static uint32_t volatile    g_cThreadInTree;
+/** Indicates whether we've been initialized or not. */
+static bool                     g_frtThreadInitialized;
 #ifdef IN_RING3
 /** The RW lock protecting the tree. */
-static RTSEMRW          g_ThreadRWSem = NIL_RTSEMRW;
+static RTSEMRW                  g_ThreadRWSem = NIL_RTSEMRW;
 #else
 /** The spinlocks protecting the tree. */
-static RTSPINLOCK       g_ThreadSpinlock = NIL_RTSPINLOCK;
+static RTSPINLOCK               g_ThreadSpinlock = NIL_RTSPINLOCK;
 #endif
-/** Indicates whether we've been initialized or not. */
-static bool             g_frtThreadInitialized;
+/** The AVL thread containing the threads. */
+static PAVLPVNODECORE           g_ThreadTree;
+/** The number of threads in the tree (for ring-0 termination kludge). */
+static uint32_t volatile        g_cThreadInTree;
+/** Counters for each thread type. */
+DECL_HIDDEN_DATA(uint32_t volatile) g_acRTThreadTypeStats[RTTHREADTYPE_END];
 
 
 /*********************************************************************************************************************************
@@ -287,23 +299,15 @@ static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntFla
     return rc;
 }
 
-/**
- * Adopts a non-IPRT thread.
- *
- * @returns IPRT status code.
- * @param   enmType         The thread type.
- * @param   fFlags          The thread flags. RTTHREADFLAGS_WAITABLE is not currently allowed.
- * @param   pszName         The thread name. Optional.
- * @param   pThread         Where to store the thread handle. Optional.
- */
+
 RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *pszName, PRTTHREAD pThread)
 {
     int      rc;
     RTTHREAD Thread;
 
-    AssertReturn(!(fFlags & RTTHREADFLAGS_WAITABLE), VERR_INVALID_PARAMETER);
-    AssertReturn(!pszName || VALID_PTR(pszName), VERR_INVALID_POINTER);
-    AssertReturn(!pThread || VALID_PTR(pThread), VERR_INVALID_POINTER);
+    AssertReturn(!(fFlags & RTTHREADFLAGS_WAITABLE), VERR_INVALID_FLAGS);
+    AssertPtrNullReturn(pszName, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pThread, VERR_INVALID_POINTER);
 
     rc = VINF_SUCCESS;
     Thread = RTThreadSelf();
@@ -322,8 +326,11 @@ RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *psz
         /* try adopt it */
         rc = rtThreadAdopt(enmType, fFlags, 0, pszName);
         Thread = RTThreadSelf();
-        Log(("RTThreadAdopt: %RTthrd %RTnthrd '%s' enmType=%d fFlags=%#x rc=%Rrc\n",
-             Thread, RTThreadNativeSelf(), pszName, enmType, fFlags, rc));
+
+        /* Don't too early during init, as rtLogLock may end up here and cause endless recursion. */
+        if (rc != VERR_FAILED_TO_SET_SELF_TLS)
+            Log(("RTThreadAdopt: %RTthrd %RTnthrd '%s' enmType=%d fFlags=%#x rc=%Rrc\n",
+                 Thread, RTThreadNativeSelf(), pszName, enmType, fFlags, rc));
     }
     else
         Log(("RTThreadAdopt: %RTthrd %RTnthrd '%s' enmType=%d fFlags=%#x - already adopted!\n",
@@ -336,12 +343,6 @@ RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *psz
 RT_EXPORT_SYMBOL(RTThreadAdopt);
 
 
-/**
- * Get the thread handle of the current thread, automatically adopting alien
- * threads.
- *
- * @returns Thread handle.
- */
 RTDECL(RTTHREAD) RTThreadSelfAutoAdopt(void)
 {
     RTTHREAD hSelf = RTThreadSelf();
@@ -391,6 +392,10 @@ PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntF
 #endif
 #ifdef RT_WITH_ICONV_CACHE
         rtStrIconvCacheInit(pThread);
+#endif
+#if defined(IPRT_NO_CRT) && defined(IN_RING3)
+        pThread->NoCrt.enmAllocType = RTNOCRTTHREADDATA::kAllocType_Embedded;
+        RTListInit(&pThread->NoCrt.ListEntry);
 #endif
         rc = RTSemEventMultiCreate(&pThread->EventUser);
         if (RT_SUCCESS(rc))
@@ -459,7 +464,17 @@ DECLHIDDEN(void) rtThreadInsert(PRTTHREADINT pThread, RTNATIVETHREAD NativeThrea
                 fRc = RTAvlPVInsert(&g_ThreadTree, &pThread->Core);
                 ASMAtomicOrU32(&pThread->fIntFlags, RTTHREADINT_FLAG_IN_TREE);
                 if (fRc)
+                {
                     ASMAtomicIncU32(&g_cThreadInTree);
+                    ASMAtomicIncU32(&g_acRTThreadTypeStats[pThread->enmType]);
+
+#if defined(IPRT_NO_CRT) && defined(IN_RING3)
+                    RTTLS const iTlsPerThread = g_iTlsRtNoCrtPerThread;
+                    if (   iTlsPerThread != NIL_RTTLS
+                        && RTTlsGet(iTlsPerThread) == NULL)
+                        RTTlsSet(iTlsPerThread, &pThread->NoCrt);
+#endif
+                }
 
                 AssertReleaseMsg(fRc, ("Lock problem? %p (%RTnthrd) %s\n", pThread, NativeThread, pThread->szName));
                 NOREF(fRc);
@@ -480,12 +495,13 @@ DECLHIDDEN(void) rtThreadInsert(PRTTHREADINT pThread, RTNATIVETHREAD NativeThrea
 static void rtThreadRemoveLocked(PRTTHREADINT pThread)
 {
     PRTTHREADINT pThread2 = (PRTTHREADINT)RTAvlPVRemove(&g_ThreadTree, pThread->Core.Key);
-#if !defined(RT_OS_OS2) /** @todo this asserts for threads created by NSPR */
     AssertMsg(pThread2 == pThread, ("%p(%s) != %p (%p/%s)\n", pThread2, pThread2  ? pThread2->szName : "<null>",
                                     pThread, pThread->Core.Key, pThread->szName));
-#endif
     if (pThread2)
+    {
         ASMAtomicDecU32(&g_cThreadInTree);
+        ASMAtomicDecU32(&g_acRTThreadTypeStats[pThread->enmType]);
+    }
 }
 
 
@@ -545,8 +561,8 @@ DECLHIDDEN(PRTTHREADINT) rtThreadGetByNative(RTNATIVETHREAD NativeThread)
  */
 DECLHIDDEN(PRTTHREADINT) rtThreadGet(RTTHREAD Thread)
 {
-    if (    Thread != NIL_RTTHREAD
-        &&  VALID_PTR(Thread))
+    if (   Thread != NIL_RTTHREAD
+        && RT_VALID_PTR(Thread))
     {
         PRTTHREADINT pThread = (PRTTHREADINT)Thread;
         if (    pThread->u32Magic == RTTHREADINT_MAGIC
@@ -680,6 +696,14 @@ DECLHIDDEN(void) rtThreadTerminate(PRTTHREADINT pThread, int rc)
      * key clashes in the AVL tree and release our reference to ourself.
      */
     rtThreadRemove(pThread);
+
+#if defined(IPRT_NO_CRT) && defined(IN_RING3)
+    RTTLS const iTlsPerThread = g_iTlsRtNoCrtPerThread;
+    if (   iTlsPerThread != NIL_RTTLS
+        && RTTlsGet(iTlsPerThread) == &pThread->NoCrt)
+        RTTlsSet(iTlsPerThread, &g_RtNoCrtPerThreadDummy);
+#endif
+
     rtThreadRelease(pThread);
 }
 
@@ -694,7 +718,7 @@ DECLHIDDEN(void) rtThreadTerminate(PRTTHREADINT pThread, int rc)
  * @param   NativeThread    The native thread id.
  * @param   pszThreadName   The name of the thread (purely a dummy for backtrace).
  */
-DECLCALLBACK(DECLHIDDEN(int)) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName)
+DECL_HIDDEN_CALLBACK(int) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *pszThreadName)
 {
     int rc;
     NOREF(pszThreadName);
@@ -737,20 +761,6 @@ DECLCALLBACK(DECLHIDDEN(int)) rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD
 }
 
 
-/**
- * Create a new thread.
- *
- * @returns iprt status code.
- * @param   pThread     Where to store the thread handle to the new thread. (optional)
- * @param   pfnThread   The thread function.
- * @param   pvUser      User argument.
- * @param   cbStack     The size of the stack for the new thread.
- *                      Use 0 for the default stack size.
- * @param   enmType     The thread type. Used for deciding scheduling attributes
- *                      of the thread.
- * @param   fFlags      Flags of the RTTHREADFLAGS type (ORed together).
- * @param   pszName     Thread name.
- */
 RTDECL(int) RTThreadCreate(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUser, size_t cbStack,
                            RTTHREADTYPE enmType, unsigned fFlags, const char *pszName)
 {
@@ -763,26 +773,12 @@ RTDECL(int) RTThreadCreate(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUse
     /*
      * Validate input.
      */
-    if (!VALID_PTR(pThread) && pThread)
-    {
-        Assert(VALID_PTR(pThread));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (!VALID_PTR(pfnThread))
-    {
-        Assert(VALID_PTR(pfnThread));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (!pszName || !*pszName || strlen(pszName) >= RTTHREAD_NAME_LEN)
-    {
-        AssertMsgFailed(("pszName=%s (max len is %d because of logging)\n", pszName, RTTHREAD_NAME_LEN - 1));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (fFlags & ~RTTHREADFLAGS_MASK)
-    {
-        AssertMsgFailed(("fFlags=%#x\n", fFlags));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertPtrNullReturn(pThread, VERR_INVALID_POINTER);
+    AssertPtrReturn(pfnThread, VERR_INVALID_POINTER);
+    AssertMsgReturn(pszName && *pszName != '\0' && strlen(pszName) < RTTHREAD_NAME_LEN,
+                    ("pszName=%s (max len is %d because of logging)\n", pszName, RTTHREAD_NAME_LEN - 1),
+                    VERR_INVALID_PARAMETER);
+    AssertMsgReturn(!(fFlags & ~RTTHREADFLAGS_MASK), ("fFlags=%#x\n", fFlags), VERR_INVALID_FLAGS);
 
     /*
      * Allocate thread argument.
@@ -819,21 +815,6 @@ RTDECL(int) RTThreadCreate(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUse
 RT_EXPORT_SYMBOL(RTThreadCreate);
 
 
-/**
- * Create a new thread.
- *
- * Same as RTThreadCreate except the name is given in the RTStrPrintfV form.
- *
- * @returns iprt status code.
- * @param   pThread     See RTThreadCreate.
- * @param   pfnThread   See RTThreadCreate.
- * @param   pvUser      See RTThreadCreate.
- * @param   cbStack     See RTThreadCreate.
- * @param   enmType     See RTThreadCreate.
- * @param   fFlags      See RTThreadCreate.
- * @param   pszNameFmt  Thread name format.
- * @param   va          Format arguments.
- */
 RTDECL(int) RTThreadCreateV(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUser, size_t cbStack,
                             RTTHREADTYPE enmType, uint32_t fFlags, const char *pszNameFmt, va_list va)
 {
@@ -844,21 +825,6 @@ RTDECL(int) RTThreadCreateV(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUs
 RT_EXPORT_SYMBOL(RTThreadCreateV);
 
 
-/**
- * Create a new thread.
- *
- * Same as RTThreadCreate except the name is given in the RTStrPrintf form.
- *
- * @returns iprt status code.
- * @param   pThread     See RTThreadCreate.
- * @param   pfnThread   See RTThreadCreate.
- * @param   pvUser      See RTThreadCreate.
- * @param   cbStack     See RTThreadCreate.
- * @param   enmType     See RTThreadCreate.
- * @param   fFlags      See RTThreadCreate.
- * @param   pszNameFmt  Thread name format.
- * @param   ...         Format arguments.
- */
 RTDECL(int) RTThreadCreateF(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUser, size_t cbStack,
                             RTTHREADTYPE enmType, uint32_t fFlags, const char *pszNameFmt, ...)
 {
@@ -872,12 +838,6 @@ RTDECL(int) RTThreadCreateF(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUs
 RT_EXPORT_SYMBOL(RTThreadCreateF);
 
 
-/**
- * Gets the native thread id of a IPRT thread.
- *
- * @returns The native thread id.
- * @param   Thread      The IPRT thread.
- */
 RTDECL(RTNATIVETHREAD) RTThreadGetNative(RTTHREAD Thread)
 {
     PRTTHREADINT pThread = rtThreadGet(Thread);
@@ -892,13 +852,6 @@ RTDECL(RTNATIVETHREAD) RTThreadGetNative(RTTHREAD Thread)
 RT_EXPORT_SYMBOL(RTThreadGetNative);
 
 
-/**
- * Gets the IPRT thread of a native thread.
- *
- * @returns The IPRT thread handle
- * @returns NIL_RTTHREAD if not a thread known to IPRT.
- * @param   NativeThread        The native thread handle/id.
- */
 RTDECL(RTTHREAD) RTThreadFromNative(RTNATIVETHREAD NativeThread)
 {
     PRTTHREADINT pThread = rtThreadGetByNative(NativeThread);
@@ -909,12 +862,6 @@ RTDECL(RTTHREAD) RTThreadFromNative(RTNATIVETHREAD NativeThread)
 RT_EXPORT_SYMBOL(RTThreadFromNative);
 
 
-/**
- * Gets the name of the current thread thread.
- *
- * @returns Pointer to readonly name string.
- * @returns NULL on failure.
- */
 RTDECL(const char *) RTThreadSelfName(void)
 {
     RTTHREAD Thread = RTThreadSelf();
@@ -923,9 +870,9 @@ RTDECL(const char *) RTThreadSelfName(void)
         PRTTHREADINT pThread = rtThreadGet(Thread);
         if (pThread)
         {
-            const char *szName = pThread->szName;
+            const char *pszName = pThread->szName;
             rtThreadRelease(pThread);
-            return szName;
+            return pszName;
         }
     }
     return NULL;
@@ -933,13 +880,6 @@ RTDECL(const char *) RTThreadSelfName(void)
 RT_EXPORT_SYMBOL(RTThreadSelfName);
 
 
-/**
- * Gets the name of a thread.
- *
- * @returns Pointer to readonly name string.
- * @returns NULL on failure.
- * @param   Thread      Thread handle of the thread to query the name of.
- */
 RTDECL(const char *) RTThreadGetName(RTTHREAD Thread)
 {
     PRTTHREADINT pThread;
@@ -957,13 +897,6 @@ RTDECL(const char *) RTThreadGetName(RTTHREAD Thread)
 RT_EXPORT_SYMBOL(RTThreadGetName);
 
 
-/**
- * Sets the name of a thread.
- *
- * @returns iprt status code.
- * @param   Thread      Thread handle of the thread to query the name of.
- * @param   pszName     The thread name.
- */
 RTDECL(int) RTThreadSetName(RTTHREAD Thread, const char *pszName)
 {
     /*
@@ -991,26 +924,17 @@ RTDECL(int) RTThreadSetName(RTTHREAD Thread, const char *pszName)
 RT_EXPORT_SYMBOL(RTThreadSetName);
 
 
-/**
- * Checks if the specified thread is the main thread.
- *
- * @returns true if it is, false if it isn't.
- *
- * @param   hThread     The thread handle.
- *
- * @remarks This function may not return the correct value when rtR3Init was
- *          called on a thread of the than the main one.  This could for
- *          instance happen when the DLL/DYLIB/SO containing IPRT is dynamically
- *          loaded at run time by a different thread.
- */
 RTDECL(bool) RTThreadIsMain(RTTHREAD hThread)
 {
-    PRTTHREADINT pThread = rtThreadGet(hThread);
-    if (pThread)
+    if (hThread != NIL_RTTHREAD)
     {
-        bool fRc = !!(pThread->fIntFlags & RTTHREADINT_FLAGS_MAIN);
-        rtThreadRelease(pThread);
-        return fRc;
+        PRTTHREADINT pThread = rtThreadGet(hThread);
+        if (pThread)
+        {
+            bool fRc = !!(pThread->fIntFlags & RTTHREADINT_FLAGS_MAIN);
+            rtThreadRelease(pThread);
+            return fRc;
+        }
     }
     return false;
 }
@@ -1058,11 +982,6 @@ RTDECL(bool) RTThreadIsInitialized(void)
 RT_EXPORT_SYMBOL(RTThreadIsInitialized);
 
 
-/**
- * Signal the user event.
- *
- * @returns     iprt status code.
- */
 RTDECL(int) RTThreadUserSignal(RTTHREAD Thread)
 {
     int             rc;
@@ -1079,14 +998,6 @@ RTDECL(int) RTThreadUserSignal(RTTHREAD Thread)
 RT_EXPORT_SYMBOL(RTThreadUserSignal);
 
 
-/**
- * Wait for the user event, resume on interruption.
- *
- * @returns     iprt status code.
- * @param       Thread          The thread to wait for.
- * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
- *                              an indefinite wait.
- */
 RTDECL(int) RTThreadUserWait(RTTHREAD Thread, RTMSINTERVAL cMillies)
 {
     int             rc;
@@ -1103,14 +1014,6 @@ RTDECL(int) RTThreadUserWait(RTTHREAD Thread, RTMSINTERVAL cMillies)
 RT_EXPORT_SYMBOL(RTThreadUserWait);
 
 
-/**
- * Wait for the user event, return on interruption.
- *
- * @returns     iprt status code.
- * @param       Thread          The thread to wait for.
- * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
- *                              an indefinite wait.
- */
 RTDECL(int) RTThreadUserWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies)
 {
     int             rc;
@@ -1127,12 +1030,6 @@ RTDECL(int) RTThreadUserWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies)
 RT_EXPORT_SYMBOL(RTThreadUserWaitNoResume);
 
 
-/**
- * Reset the user event.
- *
- * @returns     iprt status code.
- * @param       Thread          The thread to reset.
- */
 RTDECL(int) RTThreadUserReset(RTTHREAD Thread)
 {
     int     rc;
@@ -1224,16 +1121,6 @@ static int rtThreadWait(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc, bool f
 }
 
 
-/**
- * Wait for the thread to terminate, resume on interruption.
- *
- * @returns     iprt status code.
- *              Will not return VERR_INTERRUPTED.
- * @param       Thread          The thread to wait for.
- * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
- *                              an indefinite wait.
- * @param       prc             Where to store the return code of the thread. Optional.
- */
 RTDECL(int) RTThreadWait(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc)
 {
     int rc = rtThreadWait(Thread, cMillies, prc, true);
@@ -1243,15 +1130,6 @@ RTDECL(int) RTThreadWait(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc)
 RT_EXPORT_SYMBOL(RTThreadWait);
 
 
-/**
- * Wait for the thread to terminate, return on interruption.
- *
- * @returns     iprt status code.
- * @param       Thread          The thread to wait for.
- * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
- *                              an indefinite wait.
- * @param       prc             Where to store the return code of the thread. Optional.
- */
 RTDECL(int) RTThreadWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc)
 {
     return rtThreadWait(Thread, cMillies, prc, false);
@@ -1259,13 +1137,6 @@ RTDECL(int) RTThreadWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies, int *pr
 RT_EXPORT_SYMBOL(RTThreadWaitNoResume);
 
 
-/**
- * Changes the type of the specified thread.
- *
- * @returns iprt status code.
- * @param   Thread      The thread which type should be changed.
- * @param   enmType     The new thread type.
- */
 RTDECL(int) RTThreadSetType(RTTHREAD Thread, RTTHREADTYPE enmType)
 {
     /*
@@ -1308,13 +1179,6 @@ RTDECL(int) RTThreadSetType(RTTHREAD Thread, RTTHREADTYPE enmType)
 RT_EXPORT_SYMBOL(RTThreadSetType);
 
 
-/**
- * Gets the type of the specified thread.
- *
- * @returns The thread type.
- * @returns RTTHREADTYPE_INVALID if the thread handle is invalid.
- * @param   Thread      The thread in question.
- */
 RTDECL(RTTHREADTYPE) RTThreadGetType(RTTHREAD Thread)
 {
     RTTHREADTYPE enmType = RTTHREADTYPE_INVALID;
@@ -1415,13 +1279,6 @@ DECLHIDDEN(int) rtThreadDoSetProcPriority(RTPROCPRIORITY enmPriority)
 }
 
 
-/**
- * Change the thread state to blocking.
- *
- * @param   hThread         The current thread.
- * @param   enmState        The sleep state.
- * @param   fReallySleeping Really going to sleep now.
- */
 RTDECL(void) RTThreadBlocking(RTTHREAD hThread, RTTHREADSTATE enmState, bool fReallySleeping)
 {
     Assert(RTTHREAD_IS_SLEEPING(enmState));
@@ -1437,15 +1294,6 @@ RTDECL(void) RTThreadBlocking(RTTHREAD hThread, RTTHREADSTATE enmState, bool fRe
 RT_EXPORT_SYMBOL(RTThreadBlocking);
 
 
-/**
- * Unblocks a thread.
- *
- * This function is paired with rtThreadBlocking.
- *
- * @param   hThread     The current thread.
- * @param   enmCurState The current state, used to check for nested blocking.
- *                      The new state will be running.
- */
 RTDECL(void) RTThreadUnblocked(RTTHREAD hThread, RTTHREADSTATE enmCurState)
 {
     PRTTHREADINT pThread = hThread;
@@ -1474,12 +1322,6 @@ RTDECL(void) RTThreadUnblocked(RTTHREAD hThread, RTTHREADSTATE enmCurState)
 RT_EXPORT_SYMBOL(RTThreadUnblocked);
 
 
-/**
- * Get the current thread state.
- *
- * @returns The thread state.
- * @param   hThread         The thread.
- */
 RTDECL(RTTHREADSTATE) RTThreadGetState(RTTHREAD hThread)
 {
     RTTHREADSTATE   enmState = RTTHREADSTATE_INVALID;
