@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -1031,6 +1041,9 @@ int PrintHeader(const char *argv0, const char *pszBaseDir)
         pszBaseDir = szBuf;
     }
 
+    const char *pszYear = __DATE__;
+    pszYear += strlen(pszYear) - 4;
+
     Stream1Printf("/* $" "Id" "$ */\n"
                   "/** @file\n"
                   " * IPRT - Unicode Tables.\n"
@@ -1040,29 +1053,39 @@ int PrintHeader(const char *argv0, const char *pszBaseDir)
                   " */\n"
                   "\n"
                   "/*\n"
-                  " * Copyright (C) 2006-2017 Oracle Corporation \n"
+                  " * Copyright (C) 2006-%s Oracle and/or its affiliates.\n"
                   " *\n"
-                  " * This file is part of VirtualBox Open Source Edition (OSE), as\n"
-                  " * available from http://www.virtualbox.org. This file is free software;\n"
-                  " * you can redistribute it and/or modify it under the terms of the GNU\n"
-                  " * General Public License (GPL) as published by the Free Software\n"
-                  " * Foundation, in version 2 as it comes in the \"COPYING\" file of the\n"
-                  " * VirtualBox OSE distribution. VirtualBox OSE is distributed in the\n"
-                  " * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.\n"
+                  " * This file is part of VirtualBox base platform packages, as\n"
+                  " * available from https://www.virtualbox.org.\n"
+                  " *\n"
+                  " * This program is free software; you can redistribute it and/or\n"
+                  " * modify it under the terms of the GNU General Public License\n"
+                  " * as published by the Free Software Foundation, in version 3 of the\n"
+                  " * License.\n"
+                  " *\n"
+                  " * This program is distributed in the hope that it will be useful, but\n"
+                  " * WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+                  " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
+                  " * General Public License for more details.\n"
+                  " *\n"
+                  " * You should have received a copy of the GNU General Public License\n"
+                  " * along with this program; if not, see <https://www.gnu.org/licenses>.\n"
                   " *\n"
                   " * The contents of this file may alternatively be used under the terms\n"
                   " * of the Common Development and Distribution License Version 1.0\n"
-                  " * (CDDL) only, as it comes in the \"COPYING.CDDL\" file of the\n"
-                  " * VirtualBox OSE distribution, in which case the provisions of the\n"
+                  " * (CDDL), a copy of it is provided in the \"COPYING.CDDL\" file included\n"
+                  " * in the VirtualBox distribution, in which case the provisions of the\n"
                   " * CDDL are applicable instead of those of the GPL.\n"
                   " *\n"
                   " * You may elect to license modified versions of this file under the\n"
                   " * terms and conditions of either the GPL or the CDDL or both.\n"
+                  " *\n"
+                  " * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0\n"
                   " */\n"
                   "\n"
                   "#include <iprt/uni.h>\n"
                   "\n",
-                  pszBaseDir, argv0);
+                  pszBaseDir, argv0, pszYear);
     return 0;
 }
 
