@@ -1,18 +1,35 @@
 /* $Id: QED.cpp $ */
 /** @file
  * QED - QED Disk image.
+ *
+ * The QED backend implements support for the qemu enhanced disk format (short QED)
+ * The specification for the format is available under http://wiki.qemu.org/Features/QED/Specification
+ *
+ * Missing things to implement:
+ *    - compaction
+ *    - resizing which requires block relocation (very rare case)
  */
 
 /*
- * Copyright (C) 2011-2020 Oracle Corporation
+ * Copyright (C) 2011-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -34,15 +51,6 @@
 #include "VDBackends.h"
 #include "VDBackendsInline.h"
 
-/**
- * The QED backend implements support for the qemu enhanced disk format (short QED)
- * The specification for the format is available under http://wiki.qemu.org/Features/QED/Specification
- *
- * Missing things to implement:
- *    - compaction
- *    - resizing which requires block relocation (very rare case)
- */
-
 
 /*********************************************************************************************************************************
 *   Structures in a QED image, little endian                                                                                     *
@@ -319,7 +327,6 @@ static bool qedHdrConvertToHostEndianess(PQedHeader pHeader)
 /**
  * Creates a QED header from the given image state.
  *
- * @returns nothing.
  * @param   pImage     Image instance data.
  * @param   pHeader    Pointer to the header to convert.
  */
@@ -341,13 +348,12 @@ static void qedHdrConvertFromHostEndianess(PQEDIMAGE pImage, PQedHeader pHeader)
 /**
  * Convert table entries from little endian to host endianess.
  *
- * @returns nothing.
  * @param   paTbl       Pointer to the table.
  * @param   cEntries    Number of entries in the table.
  */
 static void qedTableConvertToHostEndianess(uint64_t *paTbl, uint32_t cEntries)
 {
-    while(cEntries-- > 0)
+    while (cEntries-- > 0)
     {
         *paTbl = RT_LE2H_U64(*paTbl);
         paTbl++;
@@ -358,15 +364,14 @@ static void qedTableConvertToHostEndianess(uint64_t *paTbl, uint32_t cEntries)
 /**
  * Convert table entries from host to little endian format.
  *
- * @returns nothing.
  * @param   paTblImg    Pointer to the table which will store the little endian table.
  * @param   paTbl       The source table to convert.
  * @param   cEntries    Number of entries in the table.
  */
-static void qedTableConvertFromHostEndianess(uint64_t *paTblImg, uint64_t *paTbl,
+static void qedTableConvertFromHostEndianess(uint64_t *paTblImg, uint64_t const *paTbl,
                                              uint32_t cEntries)
 {
-    while(cEntries-- > 0)
+    while (cEntries-- > 0)
     {
         *paTblImg = RT_H2LE_U64(*paTbl);
         paTbl++;
@@ -393,7 +398,6 @@ static int qedL2TblCacheCreate(PQEDIMAGE pImage)
 /**
  * Destroys the L2 table cache.
  *
- * @returns nothing.
  * @param   pImage    The image instance data.
  */
 static void qedL2TblCacheDestroy(PQEDIMAGE pImage)
@@ -452,7 +456,6 @@ static PQEDL2CACHEENTRY qedL2TblCacheRetain(PQEDIMAGE pImage, uint64_t offL2Tbl)
 /**
  * Releases a L2 table cache entry.
  *
- * @returns nothing.
  * @param   pL2Entry    The L2 cache entry.
  */
 static void qedL2TblCacheEntryRelease(PQEDL2CACHEENTRY pL2Entry)
@@ -518,7 +521,6 @@ static PQEDL2CACHEENTRY qedL2TblCacheEntryAlloc(PQEDIMAGE pImage)
 /**
  * Frees a L2 table cache entry.
  *
- * @returns nothing.
  * @param   pImage    The image instance data.
  * @param   pL2Entry  The L2 cache entry to free.
  */
@@ -534,7 +536,6 @@ static void qedL2TblCacheEntryFree(PQEDIMAGE pImage, PQEDL2CACHEENTRY pL2Entry)
 /**
  * Inserts an entry in the L2 table cache.
  *
- * @returns nothing.
  * @param   pImage    The image instance data.
  * @param   pL2Entry  The L2 cache entry to insert.
  */
@@ -652,7 +653,6 @@ static uint32_t qedGetPowerOfTwo(uint32_t u32)
 /**
  * Sets the L1, L2 and offset bitmasks and L1 and L2 bit shift members.
  *
- * @returns nothing.
  * @param   pImage    The image instance data.
  */
 static void qedTableMasksInit(PQEDIMAGE pImage)
@@ -674,7 +674,6 @@ static void qedTableMasksInit(PQEDIMAGE pImage)
 /**
  * Converts a given logical offset into the
  *
- * @returns nothing.
  * @param   pImage         The image instance data.
  * @param   off            The logical offset to convert.
  * @param   pidxL1         Where to store the index in the L1 table on success.
@@ -1512,7 +1511,9 @@ static DECLCALLBACK(int) qedProbe(const char *pszFilename, PVDINTERFACE pVDIfsDi
     /* Get I/O interface. */
     PVDINTERFACEIOINT pIfIo = VDIfIoIntGet(pVDIfsImage);
     AssertPtrReturn(pIfIo, VERR_INVALID_PARAMETER);
-    AssertReturn((VALID_PTR(pszFilename) && *pszFilename), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pszFilename, VERR_INVALID_POINTER);
+    AssertReturn(*pszFilename != '\0', VERR_INVALID_PARAMETER);
+
 
     /*
      * Open the file and read the footer.
@@ -1562,7 +1563,9 @@ static DECLCALLBACK(int) qedOpen(const char *pszFilename, unsigned uOpenFlags,
 
     /* Check open flags. All valid flags are supported. */
     AssertReturn(!(uOpenFlags & ~VD_OPEN_FLAGS_MASK), VERR_INVALID_PARAMETER);
-    AssertReturn((VALID_PTR(pszFilename) && *pszFilename), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pszFilename, VERR_INVALID_POINTER);
+    AssertReturn(*pszFilename != '\0', VERR_INVALID_PARAMETER);
+
 
     PQEDIMAGE pImage = (PQEDIMAGE)RTMemAllocZ(RT_UOFFSETOF(QEDIMAGE, RegionList.aRegions[1]));
     if (RT_LIKELY(pImage))
@@ -1606,10 +1609,10 @@ static DECLCALLBACK(int) qedCreate(const char *pszFilename, uint64_t cbSize,
 
     /* Check open flags. All valid flags are supported. */
     AssertReturn(!(uOpenFlags & ~VD_OPEN_FLAGS_MASK), VERR_INVALID_PARAMETER);
-    AssertReturn(   VALID_PTR(pszFilename)
-                 && *pszFilename
-                 && VALID_PTR(pPCHSGeometry)
-                 && VALID_PTR(pLCHSGeometry), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pszFilename, VERR_INVALID_POINTER);
+    AssertReturn(*pszFilename != '\0', VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pPCHSGeometry, VERR_INVALID_POINTER);
+    AssertPtrReturn(pLCHSGeometry, VERR_INVALID_POINTER);
 
     PQEDIMAGE pImage = (PQEDIMAGE)RTMemAllocZ(RT_UOFFSETOF(QEDIMAGE, RegionList.aRegions[1]));
     if (RT_LIKELY(pImage))
@@ -1713,7 +1716,8 @@ static DECLCALLBACK(int) qedRead(void *pBackendData, uint64_t uOffset, size_t cb
     AssertPtr(pImage);
     Assert(uOffset % 512 == 0);
     Assert(cbToRead % 512 == 0);
-    AssertReturn((VALID_PTR(pIoCtx) && cbToRead), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pIoCtx, VERR_INVALID_POINTER);
+    AssertReturn(cbToRead, VERR_INVALID_PARAMETER);
     AssertReturn(uOffset + cbToRead <= pImage->cbSize, VERR_INVALID_PARAMETER);
 
     qedConvertLogicalOffset(pImage, uOffset, &idxL1, &idxL2, &offCluster);
@@ -1754,7 +1758,8 @@ static DECLCALLBACK(int) qedWrite(void *pBackendData, uint64_t uOffset, size_t c
     AssertPtr(pImage);
     Assert(!(uOffset % 512));
     Assert(!(cbToWrite % 512));
-    AssertReturn((VALID_PTR(pIoCtx) && cbToWrite), VERR_INVALID_PARAMETER);
+    AssertPtrReturn(pIoCtx, VERR_INVALID_POINTER);
+    AssertReturn(cbToWrite, VERR_INVALID_PARAMETER);
     AssertReturn(uOffset + cbToWrite <= pImage->cbSize, VERR_INVALID_PARAMETER);
 
     if (!(pImage->uOpenFlags & VD_OPEN_FLAGS_READONLY))
