@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2011-2020 Oracle Corporation
+ * Copyright (C) 2011-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -137,20 +147,17 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repe_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint8
      */
     ADDR_TYPE       uCounterReg  = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_SREG_FROM_IDX(iEffSeg) | CPUMCTX_EXTRN_ES);
 
     PCCPUMSELREGHID pSrc1Hid     = iemSRegGetHid(pVCpu, iEffSeg);
-    uint64_t        uSrc1Base;
+    uint64_t        uSrc1Base    = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC    rcStrict     = iemMemSegCheckReadAccessEx(pVCpu, pSrc1Hid, iEffSeg, &uSrc1Base);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
 
-    uint64_t        uSrc2Base;
+    uint64_t        uSrc2Base    = 0; /* gcc may not be used uninitialized */
     rcStrict = iemMemSegCheckReadAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uSrc2Base);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -170,10 +177,10 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repe_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint8
          */
         ADDR2_TYPE  uVirtSrc1Addr = uSrc1AddrReg + (ADDR2_TYPE)uSrc1Base;
         ADDR2_TYPE  uVirtSrc2Addr = uSrc2AddrReg + (ADDR2_TYPE)uSrc2Base;
-        uint32_t    cLeftSrc1Page = (PAGE_SIZE - (uVirtSrc1Addr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftSrc1Page = (GUEST_PAGE_SIZE - (uVirtSrc1Addr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftSrc1Page > uCounterReg)
             cLeftSrc1Page = uCounterReg;
-        uint32_t    cLeftSrc2Page = (PAGE_SIZE - (uVirtSrc2Addr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftSrc2Page = (GUEST_PAGE_SIZE - (uVirtSrc2Addr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         uint32_t    cLeftPage     = RT_MIN(cLeftSrc1Page, cLeftSrc2Page);
 
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -187,12 +194,12 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repe_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint8
            )
         {
             RTGCPHYS GCPhysSrc1Mem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc1Addr, IEM_ACCESS_DATA_R, &GCPhysSrc1Mem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc1Addr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysSrc1Mem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
             RTGCPHYS GCPhysSrc2Mem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc2Addr, IEM_ACCESS_DATA_R, &GCPhysSrc2Mem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc2Addr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysSrc2Mem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -290,8 +297,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repe_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint8
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -307,20 +313,17 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repne_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_SREG_FROM_IDX(iEffSeg) | CPUMCTX_EXTRN_ES);
 
-    PCCPUMSELREGHID pSrc1Hid = iemSRegGetHid(pVCpu, iEffSeg);
-    uint64_t        uSrc1Base;
+    PCCPUMSELREGHID pSrc1Hid     = iemSRegGetHid(pVCpu, iEffSeg);
+    uint64_t        uSrc1Base    = 0; /* gcc may not be used uninitialized */;
     VBOXSTRICTRC rcStrict = iemMemSegCheckReadAccessEx(pVCpu, pSrc1Hid, iEffSeg, &uSrc1Base);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
 
-    uint64_t        uSrc2Base;
+    uint64_t        uSrc2Base    = 0; /* gcc may not be used uninitialized */
     rcStrict = iemMemSegCheckReadAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uSrc2Base);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -340,10 +343,10 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repne_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint
          */
         ADDR2_TYPE  uVirtSrc1Addr = uSrc1AddrReg + (ADDR2_TYPE)uSrc1Base;
         ADDR2_TYPE  uVirtSrc2Addr = uSrc2AddrReg + (ADDR2_TYPE)uSrc2Base;
-        uint32_t    cLeftSrc1Page = (PAGE_SIZE - (uVirtSrc1Addr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftSrc1Page = (GUEST_PAGE_SIZE - (uVirtSrc1Addr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftSrc1Page > uCounterReg)
             cLeftSrc1Page = uCounterReg;
-        uint32_t    cLeftSrc2Page = (PAGE_SIZE - (uVirtSrc2Addr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftSrc2Page = (GUEST_PAGE_SIZE - (uVirtSrc2Addr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         uint32_t    cLeftPage = RT_MIN(cLeftSrc1Page, cLeftSrc2Page);
 
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -357,12 +360,12 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repne_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint
            )
         {
             RTGCPHYS GCPhysSrc1Mem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc1Addr, IEM_ACCESS_DATA_R, &GCPhysSrc1Mem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc1Addr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysSrc1Mem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
             RTGCPHYS GCPhysSrc2Mem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc2Addr, IEM_ACCESS_DATA_R, &GCPhysSrc2Mem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrc2Addr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysSrc2Mem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -460,8 +463,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_repne_cmps_op,OP_SIZE,_addr,ADDR_SIZE), uint
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -477,13 +479,10 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repe_scas_,OP_rAX,_m,ADDR_SIZE))
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_ES);
-    uint64_t        uBaseAddr;
+    uint64_t        uBaseAddr   = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC rcStrict = iemMemSegCheckReadAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -502,7 +501,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repe_scas_,OP_rAX,_m,ADDR_SIZE))
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -514,7 +513,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repe_scas_,OP_rAX,_m,ADDR_SIZE))
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_R, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -592,8 +591,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repe_scas_,OP_rAX,_m,ADDR_SIZE))
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -609,13 +607,10 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repne_scas_,OP_rAX,_m,ADDR_SIZE))
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_ES);
-    uint64_t        uBaseAddr;
+    uint64_t        uBaseAddr   = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC rcStrict = iemMemSegCheckReadAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -634,7 +629,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repne_scas_,OP_rAX,_m,ADDR_SIZE))
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -646,7 +641,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repne_scas_,OP_rAX,_m,ADDR_SIZE))
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_R, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -723,8 +718,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_repne_scas_,OP_rAX,_m,ADDR_SIZE))
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -742,20 +736,17 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_movs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_SREG_FROM_IDX(iEffSeg) | CPUMCTX_EXTRN_ES);
 
-    PCCPUMSELREGHID pSrcHid = iemSRegGetHid(pVCpu, iEffSeg);
-    uint64_t        uSrcBase;
+    PCCPUMSELREGHID pSrcHid   = iemSRegGetHid(pVCpu, iEffSeg);
+    uint64_t        uSrcBase  = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC rcStrict = iemMemSegCheckReadAccessEx(pVCpu, pSrcHid, iEffSeg, &uSrcBase);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
 
-    uint64_t        uDstBase;
+    uint64_t        uDstBase  = 0; /* gcc may not be used uninitialized */
     rcStrict = iemMemSegCheckWriteAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uDstBase);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -783,10 +774,10 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_movs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
          */
         ADDR2_TYPE  uVirtSrcAddr = uSrcAddrReg + (ADDR2_TYPE)uSrcBase;
         ADDR2_TYPE  uVirtDstAddr = uDstAddrReg + (ADDR2_TYPE)uDstBase;
-        uint32_t    cLeftSrcPage = (PAGE_SIZE - (uVirtSrcAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftSrcPage = (GUEST_PAGE_SIZE - (uVirtSrcAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftSrcPage > uCounterReg)
             cLeftSrcPage = uCounterReg;
-        uint32_t    cLeftDstPage = (PAGE_SIZE - (uVirtDstAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftDstPage = (GUEST_PAGE_SIZE - (uVirtDstAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         uint32_t    cLeftPage = RT_MIN(cLeftSrcPage, cLeftDstPage);
 
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -800,12 +791,12 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_movs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
            )
         {
             RTGCPHYS GCPhysSrcMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrcAddr, IEM_ACCESS_DATA_R, &GCPhysSrcMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtSrcAddr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysSrcMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
             RTGCPHYS GCPhysDstMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtDstAddr, IEM_ACCESS_DATA_W, &GCPhysDstMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtDstAddr, OP_SIZE / 8, IEM_ACCESS_DATA_W, &GCPhysDstMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -823,8 +814,8 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_movs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
                 rcStrict = iemMemPageMap(pVCpu, GCPhysSrcMem, IEM_ACCESS_DATA_R, (void **)&puSrcMem, &PgLockSrcMem);
                 if (rcStrict == VINF_SUCCESS)
                 {
-                    Assert(   (GCPhysSrcMem         >> PAGE_SHIFT) != (GCPhysDstMem         >> PAGE_SHIFT)
-                           || ((uintptr_t)puSrcMem  >> PAGE_SHIFT) == ((uintptr_t)puDstMem  >> PAGE_SHIFT));
+                    Assert(   (GCPhysSrcMem         >> GUEST_PAGE_SHIFT) != (GCPhysDstMem         >> GUEST_PAGE_SHIFT)
+                           || ((uintptr_t)puSrcMem  >> GUEST_PAGE_SHIFT) == ((uintptr_t)puDstMem  >> GUEST_PAGE_SHIFT));
 
                     /* Perform the operation exactly (don't use memcpy to avoid
                        having to consider how its implementation would affect
@@ -885,8 +876,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_movs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -902,14 +892,11 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_stos_,OP_rAX,_m,ADDR_SIZE))
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_ES);
 
-    uint64_t        uBaseAddr;
+    uint64_t        uBaseAddr   = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC rcStrict = iemMemSegCheckWriteAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -937,7 +924,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_stos_,OP_rAX,_m,ADDR_SIZE))
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -949,7 +936,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_stos_,OP_rAX,_m,ADDR_SIZE))
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_W, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_W, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -1034,8 +1021,7 @@ IEM_CIMPL_DEF_0(RT_CONCAT4(iemCImpl_stos_,OP_rAX,_m,ADDR_SIZE))
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -1051,14 +1037,11 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_lods_,OP_rAX,_m,ADDR_SIZE), int8_t, iEffSeg)
      */
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
     IEM_CTX_IMPORT_RET(pVCpu, CPUMCTX_EXTRN_SREG_FROM_IDX(iEffSeg));
-    PCCPUMSELREGHID pSrcHid = iemSRegGetHid(pVCpu, iEffSeg);
-    uint64_t        uBaseAddr;
+    PCCPUMSELREGHID pSrcHid   = iemSRegGetHid(pVCpu, iEffSeg);
+    uint64_t        uBaseAddr = 0; /* gcc may not be used uninitialized */
     VBOXSTRICTRC rcStrict = iemMemSegCheckReadAccessEx(pVCpu, pSrcHid, iEffSeg, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -1075,7 +1058,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_lods_,OP_rAX,_m,ADDR_SIZE), int8_t, iEffSeg)
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -1087,7 +1070,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_lods_,OP_rAX,_m,ADDR_SIZE), int8_t, iEffSeg)
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_R, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -1160,8 +1143,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_lods_,OP_rAX,_m,ADDR_SIZE), int8_t, iEffSeg)
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -1230,7 +1212,8 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, fIoCh
 #endif
 
     OP_TYPE        *puMem;
-    rcStrict = iemMemMap(pVCpu, (void **)&puMem, OP_SIZE / 8, X86_SREG_ES, pVCpu->cpum.GstCtx.ADDR_rDI, IEM_ACCESS_DATA_W);
+    rcStrict = iemMemMap(pVCpu, (void **)&puMem, OP_SIZE / 8, X86_SREG_ES, pVCpu->cpum.GstCtx.ADDR_rDI,
+                         IEM_ACCESS_DATA_W, OP_SIZE / 8 - 1);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
 
@@ -1238,6 +1221,9 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, fIoCh
     rcStrict = IOMIOPortRead(pVM, pVCpu, pVCpu->cpum.GstCtx.dx, &u32Value, OP_SIZE / 8);
     if (IOM_SUCCESS(rcStrict))
     {
+        /**
+         * @todo I/O breakpoint support for INS
+         */
         *puMem = (OP_TYPE)u32Value;
 # ifdef IN_RING3
         VBOXSTRICTRC rcStrict2 = iemMemCommitAndUnmap(pVCpu, puMem, IEM_ACCESS_DATA_W);
@@ -1250,7 +1236,17 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, fIoCh
                 pVCpu->cpum.GstCtx.ADDR_rDI += OP_SIZE / 8;
             else
                 pVCpu->cpum.GstCtx.ADDR_rDI -= OP_SIZE / 8;
-            iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+
+            /** @todo finish: work out how this should work wrt status codes. Not sure we
+             * can use iemSetPassUpStatus here, but it depends on what
+             * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
+            rcStrict2 = iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+            if (rcStrict2 != VINF_SUCCESS)
+            {
+                iemSetPassUpStatus(pVCpu, rcStrict);
+                rcStrict = rcStrict2;
+            }
+            pVCpu->iem.s.cPotentialExits++;
         }
         else
             AssertLogRelMsgFailedReturn(("rcStrict2=%Rrc\n", VBOXSTRICTRC_VAL(rcStrict2)), RT_FAILURE_NP(rcStrict2) ? rcStrict2 : VERR_IEM_IPE_1);
@@ -1316,12 +1312,9 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
 
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
-    uint64_t        uBaseAddr;
+    uint64_t        uBaseAddr   = 0; /* gcc may not be used uninitialized */
     rcStrict = iemMemSegCheckWriteAccessEx(pVCpu, iemSRegUpdateHid(pVCpu, &pVCpu->cpum.GstCtx.es), X86_SREG_ES, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -1347,7 +1340,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -1359,7 +1352,7 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_W, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_W, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -1387,9 +1380,13 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
                 {
                     if (IOM_SUCCESS(rcStrict))
                     {
+                        /** @todo finish: work out how this should work wrt status codes. Not sure we
+                         * can use iemSetPassUpStatus here, but it depends on what
+                         * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
                         rcStrict = iemSetPassUpStatus(pVCpu, rcStrict);
                         if (uCounterReg == 0)
-                            iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+                            rcStrict = iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+                        pVCpu->iem.s.cPotentialExits++;
                     }
                     return rcStrict;
                 }
@@ -1420,7 +1417,8 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
         do
         {
             OP_TYPE *puMem;
-            rcStrict = iemMemMap(pVCpu, (void **)&puMem, OP_SIZE / 8, X86_SREG_ES, uAddrReg, IEM_ACCESS_DATA_W);
+            rcStrict = iemMemMap(pVCpu, (void **)&puMem, OP_SIZE / 8, X86_SREG_ES, uAddrReg,
+                                 IEM_ACCESS_DATA_W, OP_SIZE / 8 - 1);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -1450,9 +1448,13 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
             cLeftPage--;
             if (rcStrict != VINF_SUCCESS)
             {
-                if (uCounterReg == 0)
-                    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+                /** @todo finish: work out how this should work wrt status codes. Not sure we
+                 * can use iemSetPassUpStatus here, but it depends on what
+                 * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
                 rcStrict = iemSetPassUpStatus(pVCpu, rcStrict);
+                if (uCounterReg == 0)
+                    rcStrict = iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+                pVCpu->iem.s.cPotentialExits++;
                 return rcStrict;
             }
 
@@ -1471,8 +1473,8 @@ IEM_CIMPL_DEF_1(RT_CONCAT4(iemCImpl_rep_ins_op,OP_SIZE,_addr,ADDR_SIZE), bool, f
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    pVCpu->iem.s.cPotentialExits++;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 
@@ -1540,9 +1542,13 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_t, i
                 pVCpu->cpum.GstCtx.ADDR_rSI += OP_SIZE / 8;
             else
                 pVCpu->cpum.GstCtx.ADDR_rSI -= OP_SIZE / 8;
-            iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+            /** @todo finish: work out how this should work wrt status codes. Not sure we
+             * can use iemSetPassUpStatus here, but it depends on what
+             * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
             if (rcStrict != VINF_SUCCESS)
                 rcStrict = iemSetPassUpStatus(pVCpu, rcStrict);
+            rcStrict = iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+            pVCpu->iem.s.cPotentialExits++;
         }
     }
     return rcStrict;
@@ -1604,13 +1610,10 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
 
     ADDR_TYPE       uCounterReg = pVCpu->cpum.GstCtx.ADDR_rCX;
     if (uCounterReg == 0)
-    {
-        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-        return VINF_SUCCESS;
-    }
+        return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 
-    PCCPUMSELREGHID pHid = iemSRegGetHid(pVCpu, iEffSeg);
-    uint64_t        uBaseAddr;
+    PCCPUMSELREGHID pHid      = iemSRegGetHid(pVCpu, iEffSeg);
+    uint64_t        uBaseAddr = 0; /* gcc may not be used uninitialized */
     rcStrict = iemMemSegCheckReadAccessEx(pVCpu, pHid, iEffSeg, &uBaseAddr);
     if (rcStrict != VINF_SUCCESS)
         return rcStrict;
@@ -1627,7 +1630,7 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
          * Do segmentation and virtual page stuff.
          */
         ADDR2_TYPE  uVirtAddr = uAddrReg + (ADDR2_TYPE)uBaseAddr;
-        uint32_t    cLeftPage = (PAGE_SIZE - (uVirtAddr & PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
+        uint32_t    cLeftPage = (GUEST_PAGE_SIZE - (uVirtAddr & GUEST_PAGE_OFFSET_MASK)) / (OP_SIZE / 8);
         if (cLeftPage > uCounterReg)
             cLeftPage = uCounterReg;
         if (   cLeftPage > 0 /* can be null if unaligned, do one fallback round. */
@@ -1639,7 +1642,7 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
            )
         {
             RTGCPHYS GCPhysMem;
-            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, IEM_ACCESS_DATA_R, &GCPhysMem);
+            rcStrict = iemMemPageTranslateAndCheckAccess(pVCpu, uVirtAddr, OP_SIZE / 8, IEM_ACCESS_DATA_R, &GCPhysMem);
             if (rcStrict != VINF_SUCCESS)
                 return rcStrict;
 
@@ -1667,9 +1670,13 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
                 {
                     if (IOM_SUCCESS(rcStrict))
                     {
+                        /** @todo finish: work out how this should work wrt status codes. Not sure we
+                         * can use iemSetPassUpStatus here, but it depends on what
+                         * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
                         rcStrict = iemSetPassUpStatus(pVCpu, rcStrict);
                         if (uCounterReg == 0)
-                            iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+                            rcStrict = iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+                        pVCpu->iem.s.cPotentialExits++;
                     }
                     return rcStrict;
                 }
@@ -1716,9 +1723,13 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
             {
                 if (IOM_SUCCESS(rcStrict))
                 {
-                    if (uCounterReg == 0)
-                        iemRegAddToRipAndClearRF(pVCpu, cbInstr);
+                    /** @todo finish: work out how this should work wrt status codes. Not sure we
+                     * can use iemSetPassUpStatus here, but it depends on what
+                     * iemRegAddToRipAndFinishingClearingRF may eventually return (if anything)... */
                     rcStrict = iemSetPassUpStatus(pVCpu, rcStrict);
+                    if (uCounterReg == 0)
+                        iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
+                    pVCpu->iem.s.cPotentialExits++;
                 }
                 return rcStrict;
             }
@@ -1737,8 +1748,8 @@ IEM_CIMPL_DEF_2(RT_CONCAT4(iemCImpl_rep_outs_op,OP_SIZE,_addr,ADDR_SIZE), uint8_
     /*
      * Done.
      */
-    iemRegAddToRipAndClearRF(pVCpu, cbInstr);
-    return VINF_SUCCESS;
+    pVCpu->iem.s.cPotentialExits++;
+    return iemRegAddToRipAndFinishingClearingRF(pVCpu, cbInstr);
 }
 
 #endif /* OP_SIZE != 64-bit */
