@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2015-2017 Oracle Corporation
+ * Copyright (C) 2015-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -25,18 +35,18 @@
 #include <VBox/vmm/hm.h>
 #include <VBox/vmm/pdmapi.h>
 #include <VBox/vmm/ssm.h>
+#include <VBox/vmm/em.h>
 #include "GIMInternal.h"
 #include <VBox/vmm/vm.h>
 
 #include <VBox/disopcode.h>
+#include <VBox/err.h>
 #include <VBox/version.h>
 
 #include <iprt/asm-math.h>
 #include <iprt/assert.h>
-#include <iprt/err.h>
 #include <iprt/string.h>
 #include <iprt/mem.h>
-#include <iprt/spinlock.h>
 
 
 
@@ -78,12 +88,137 @@ typedef KVMWALLCLOCKINFO *PKVMWALLCLOCKINFO;
  */
 static CPUMMSRRANGE const g_aMsrRanges_Kvm[] =
 {
-    GIMKVM_MSRRANGE(MSR_GIM_KVM_RANGE0_START, MSR_GIM_KVM_RANGE0_END, "KVM range 0"),
-    GIMKVM_MSRRANGE(MSR_GIM_KVM_RANGE1_START, MSR_GIM_KVM_RANGE1_END, "KVM range 1")
+    GIMKVM_MSRRANGE(MSR_GIM_KVM_RANGE0_FIRST, MSR_GIM_KVM_RANGE0_LAST, "KVM range 0"),
+    GIMKVM_MSRRANGE(MSR_GIM_KVM_RANGE1_FIRST, MSR_GIM_KVM_RANGE1_LAST, "KVM range 1")
 };
 #undef GIMKVM_MSRRANGE
 
 
+/**
+ * Updates the KVM VCPU system-time structure in guest memory.
+ *
+ * @returns VBox status code.
+ * @param   pVM     The cross context VM structure.
+ * @param   pVCpu   The cross context virtual CPU structure.
+ *
+ * @remarks This must be called after the system time MSR value has been updated.
+ */
+static int gimR3KvmUpdateSystemTime(PVM pVM, PVMCPU pVCpu)
+{
+    PGIMKVM    pKvm    = &pVM->gim.s.u.Kvm;
+    PGIMKVMCPU pKvmCpu = &pVCpu->gim.s.u.KvmCpu;
+
+    /*
+     * Validate the MSR has the enable bit and the guest's system time struct. address.
+     */
+    MSR_GIM_KVM_SYSTEM_TIME_IS_ENABLED(pKvmCpu->u64SystemTimeMsr);
+    if (!PGMPhysIsGCPhysNormal(pVM, pKvmCpu->GCPhysSystemTime))
+    {
+        LogRel(("GIM: KVM: VCPU%3d: Invalid physical addr requested for mapping system-time struct. GCPhysSystemTime=%#RGp\n",
+               pVCpu->idCpu, pKvmCpu->GCPhysSystemTime));
+        return VERR_GIM_OPERATION_FAILED;
+    }
+
+    VMSTATE const enmVMState = pVM->enmVMState;
+    bool const fRunning = VMSTATE_IS_RUNNING(enmVMState);
+    Assert(!(pKvmCpu->u32SystemTimeVersion & UINT32_C(1)));
+
+    /*
+     * Construct a system-time struct.
+     */
+    GIMKVMSYSTEMTIME SystemTime;
+    RT_ZERO(SystemTime);
+    SystemTime.u32Version  = pKvmCpu->u32SystemTimeVersion + !!fRunning;
+    SystemTime.u64NanoTS   = pKvmCpu->uVirtNanoTS;
+    SystemTime.u64Tsc      = pKvmCpu->uTsc;
+    SystemTime.fFlags      = pKvmCpu->fSystemTimeFlags | GIM_KVM_SYSTEM_TIME_FLAGS_TSC_STABLE;
+
+    /*
+     * How the guest calculates the system time (nanoseconds):
+     *
+     * tsc = rdtsc - SysTime.u64Tsc
+     * if (SysTime.i8TscShift >= 0)
+     *     tsc <<= i8TscShift;
+     * else
+     *     tsc >>= -i8TscShift;
+     * time = ((tsc * SysTime.u32TscScale) >> 32) + SysTime.u64NanoTS
+     */
+    uint64_t u64TscFreq   = pKvm->cTscTicksPerSecond;
+    SystemTime.i8TscShift = 0;
+    while (u64TscFreq > 2 * RT_NS_1SEC_64)
+    {
+        u64TscFreq >>= 1;
+        SystemTime.i8TscShift--;
+    }
+    uint32_t uTscFreqLo = (uint32_t)u64TscFreq;
+    while (uTscFreqLo <= RT_NS_1SEC)
+    {
+        uTscFreqLo <<= 1;
+        SystemTime.i8TscShift++;
+    }
+    SystemTime.u32TscScale = ASMDivU64ByU32RetU32(RT_NS_1SEC_64 << 32, uTscFreqLo);
+
+    /*
+     * For informational purposes, back-calculate the exact TSC frequency the guest will see.
+     * Note that the frequency is in kHz, not Hz, since that's what Linux uses.
+     */
+    uint64_t uTscKHz = (RT_NS_1MS_64 << 32) / SystemTime.u32TscScale;
+    if (SystemTime.i8TscShift < 0)
+        uTscKHz <<= -SystemTime.i8TscShift;
+    else
+        uTscKHz >>=  SystemTime.i8TscShift;
+
+    /*
+     * Update guest memory with the system-time struct.
+     *
+     * We update the struct with an incremented, odd version field to indicate to the guest
+     * that the memory is being updated concurrently by the host and it should discard any
+     * data from this struct when it reads an odd version.
+     *
+     * When the VM is not running, we don't need to do this two step update for obvious
+     * reasons and so we skip it.
+     */
+    if (fRunning)
+        Assert(SystemTime.u32Version & UINT32_C(1));
+    else
+        Assert(!(SystemTime.u32Version & UINT32_C(1)));
+
+    int rc = PGMPhysSimpleWriteGCPhys(pVM, pKvmCpu->GCPhysSystemTime, &SystemTime, sizeof(GIMKVMSYSTEMTIME));
+    if (RT_SUCCESS(rc))
+    {
+        LogRel(("GIM: KVM: VCPU%3d: Enabled system-time struct. at %#RGp - u32TscScale=%#RX32 i8TscShift=%d uVersion=%#RU32 "
+                "fFlags=%#x uTsc=%#RX64 uVirtNanoTS=%#RX64 TscKHz=%RU64\n", pVCpu->idCpu, pKvmCpu->GCPhysSystemTime,
+                SystemTime.u32TscScale, SystemTime.i8TscShift, SystemTime.u32Version + !!fRunning, SystemTime.fFlags,
+                pKvmCpu->uTsc, pKvmCpu->uVirtNanoTS, uTscKHz));
+        TMR3CpuTickParavirtEnable(pVM);
+    }
+    else
+    {
+        LogRel(("GIM: KVM: VCPU%3d: Failed to write system-time struct. at %#RGp. rc=%Rrc\n", pVCpu->idCpu,
+                pKvmCpu->GCPhysSystemTime, rc));
+    }
+
+    if (fRunning)
+    {
+        ++SystemTime.u32Version;
+        Assert(!(SystemTime.u32Version & UINT32_C(1)));
+        rc = PGMPhysSimpleWriteGCPhys(pVM, pKvmCpu->GCPhysSystemTime + RT_UOFFSETOF(GIMKVMSYSTEMTIME, u32Version),
+                                          &SystemTime.u32Version, sizeof(SystemTime.u32Version));
+        if (RT_FAILURE(rc))
+        {
+            LogRel(("GIM: KVM: VCPU%3d: Failed to write system-time struct. while updating version field at %#RGp. rc=%Rrc\n",
+                    pVCpu->idCpu, pKvmCpu->GCPhysSystemTime, rc));
+            return rc;
+        }
+
+        /* Update the version so our next write will start with an even value. */
+        pKvmCpu->u32SystemTimeVersion += 2;
+    }
+
+    return rc;
+}
+
+
 /**
  * Initializes the KVM GIM provider.
  *
@@ -157,25 +292,16 @@ VMMR3_INT_DECL(int) gimR3KvmInit(PVM pVM)
 
     /*
      * Setup hypercall and #UD handling.
+     * Note! We always need to trap VMCALL/VMMCALL hypercall using #UDs for raw-mode VMs.
      */
-    for (VMCPUID i = 0; i < pVM->cCpus; i++)
-        VMMHypercallsEnable(&pVM->aCpus[i]);
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
+        EMSetHypercallInstructionsEnabled(pVM->apCpusR3[idCpu], true);
 
-    if (ASMIsAmdCpu())
-    {
-        pKvm->fTrapXcptUD   = true;
-        pKvm->uOpCodeNative = OP_VMMCALL;
-    }
-    else
-    {
-        Assert(ASMIsIntelCpu() || ASMIsViaCentaurCpu() || ASMIsShanghaiCpu());
-        pKvm->fTrapXcptUD   = false;
-        pKvm->uOpCodeNative = OP_VMCALL;
-    }
-
-    /* We always need to trap VMCALL/VMMCALL hypercall using #UDs for raw-mode VMs. */
-    if (!HMIsEnabled(pVM))
-        pKvm->fTrapXcptUD = true;
+    size_t cbHypercall = 0;
+    rc = GIMQueryHypercallOpcodeBytes(pVM, pKvm->abOpcodeNative, sizeof(pKvm->abOpcodeNative), &cbHypercall, &pKvm->uOpcodeNative);
+    AssertLogRelRCReturn(rc, rc);
+    AssertLogRelReturn(cbHypercall == sizeof(pKvm->abOpcodeNative), VERR_GIM_IPE_1);
+    pKvm->fTrapXcptUD = pKvm->uOpcodeNative != OP_VMCALL;
 
     return VINF_SUCCESS;
 }
@@ -248,9 +374,9 @@ VMMR3_INT_DECL(void) gimR3KvmReset(PVM pVM)
      */
     PGIMKVM pKvm = &pVM->gim.s.u.Kvm;
     pKvm->u64WallClockMsr = 0;
-    for (VMCPUID iCpu = 0; iCpu < pVM->cCpus; iCpu++)
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
     {
-        PGIMKVMCPU pKvmCpu = &pVM->aCpus[iCpu].gim.s.u.KvmCpu;
+        PGIMKVMCPU pKvmCpu = &pVM->apCpusR3[idCpu]->gim.s.u.KvmCpu;
         pKvmCpu->u64SystemTimeMsr = 0;
         pKvmCpu->u32SystemTimeVersion = 0;
         pKvmCpu->fSystemTimeFlags = 0;
@@ -280,9 +406,9 @@ VMMR3_INT_DECL(int) gimR3KvmSave(PVM pVM, PSSMHANDLE pSSM)
     /*
      * Save per-VCPU data.
      */
-    for (uint32_t i = 0; i < pVM->cCpus; i++)
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
     {
-        PCGIMKVMCPU pKvmCpu = &pVM->aCpus[i].gim.s.u.KvmCpu;
+        PCGIMKVMCPU pKvmCpu = &pVM->apCpusR3[idCpu]->gim.s.u.KvmCpu;
         SSMR3PutU64(pSSM, pKvmCpu->u64SystemTimeMsr);
         SSMR3PutU64(pSSM, pKvmCpu->uTsc);
         SSMR3PutU64(pSSM, pKvmCpu->uVirtNanoTS);
@@ -328,9 +454,9 @@ VMMR3_INT_DECL(int) gimR3KvmLoad(PVM pVM, PSSMHANDLE pSSM)
     /*
      * Load per-VCPU data.
      */
-    for (uint32_t i = 0; i < pVM->cCpus; i++)
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
     {
-        PVMCPU     pVCpu   = &pVM->aCpus[i];
+        PVMCPU     pVCpu   = pVM->apCpusR3[idCpu];
         PGIMKVMCPU pKvmCpu = &pVCpu->gim.s.u.KvmCpu;
 
         SSMR3GetU64(pSSM, &pKvmCpu->u64SystemTimeMsr);
@@ -348,7 +474,7 @@ VMMR3_INT_DECL(int) gimR3KvmLoad(PVM pVM, PSSMHANDLE pSSM)
         {
             Assert(!TMVirtualIsTicking(pVM));       /* paranoia. */
             Assert(!TMCpuTickIsTicking(pVCpu));
-            gimR3KvmEnableSystemTime(pVM, pVCpu);
+            gimR3KvmUpdateSystemTime(pVM, pVCpu);
         }
     }
 
@@ -363,86 +489,6 @@ VMMR3_INT_DECL(int) gimR3KvmLoad(PVM pVM, PSSMHANDLE pSSM)
 }
 
 
-/**
- * Enables the KVM VCPU system-time structure.
- *
- * @returns VBox status code.
- * @param   pVM                The cross context VM structure.
- * @param   pVCpu              The cross context virtual CPU structure.
- *
- * @remarks Don't do any release assertions here, these can be triggered by
- *          guest R0 code.
- */
-VMMR3_INT_DECL(int) gimR3KvmEnableSystemTime(PVM pVM, PVMCPU pVCpu)
-{
-    PGIMKVM    pKvm    = &pVM->gim.s.u.Kvm;
-    PGIMKVMCPU pKvmCpu = &pVCpu->gim.s.u.KvmCpu;
-
-    /*
-     * Validate the mapping address first.
-     */
-    if (!PGMPhysIsGCPhysNormal(pVM, pKvmCpu->GCPhysSystemTime))
-    {
-        LogRel(("GIM: KVM: VCPU%3d: Invalid physical addr requested for mapping system-time struct. GCPhysSystemTime=%#RGp\n",
-               pVCpu->idCpu, pKvmCpu->GCPhysSystemTime));
-        return VERR_GIM_OPERATION_FAILED;
-    }
-
-    /*
-     * Construct the system-time struct.
-     */
-    GIMKVMSYSTEMTIME SystemTime;
-    RT_ZERO(SystemTime);
-    SystemTime.u32Version  = pKvmCpu->u32SystemTimeVersion;
-    SystemTime.u64NanoTS   = pKvmCpu->uVirtNanoTS;
-    SystemTime.u64Tsc      = pKvmCpu->uTsc;
-    SystemTime.fFlags      = pKvmCpu->fSystemTimeFlags | GIM_KVM_SYSTEM_TIME_FLAGS_TSC_STABLE;
-
-    /*
-     * How the guest calculates the system time (nanoseconds):
-     *
-     * tsc = rdtsc - SysTime.u64Tsc
-     * if (SysTime.i8TscShift >= 0)
-     *     tsc <<= i8TscShift;
-     * else
-     *     tsc >>= -i8TscShift;
-     * time = ((tsc * SysTime.u32TscScale) >> 32) + SysTime.u64NanoTS
-     */
-    uint64_t u64TscFreq   = pKvm->cTscTicksPerSecond;
-    SystemTime.i8TscShift = 0;
-    while (u64TscFreq > 2 * RT_NS_1SEC_64)
-    {
-        u64TscFreq >>= 1;
-        SystemTime.i8TscShift--;
-    }
-    uint32_t uTscFreqLo = (uint32_t)u64TscFreq;
-    while (uTscFreqLo <= RT_NS_1SEC)
-    {
-        uTscFreqLo <<= 1;
-        SystemTime.i8TscShift++;
-    }
-    SystemTime.u32TscScale = ASMDivU64ByU32RetU32(RT_NS_1SEC_64 << 32, uTscFreqLo);
-
-    /*
-     * Update guest memory with the system-time struct.
-     */
-    Assert(!(SystemTime.u32Version & UINT32_C(1)));
-    int rc = PGMPhysSimpleWriteGCPhys(pVM, pKvmCpu->GCPhysSystemTime, &SystemTime, sizeof(GIMKVMSYSTEMTIME));
-    if (RT_SUCCESS(rc))
-    {
-        LogRel(("GIM: KVM: VCPU%3d: Enabled system-time struct. at %#RGp - u32TscScale=%#RX32 i8TscShift=%d uVersion=%#RU32 "
-                "fFlags=%#x uTsc=%#RX64 uVirtNanoTS=%#RX64\n", pVCpu->idCpu, pKvmCpu->GCPhysSystemTime, SystemTime.u32TscScale,
-                SystemTime.i8TscShift, SystemTime.u32Version, SystemTime.fFlags, pKvmCpu->uTsc, pKvmCpu->uVirtNanoTS));
-        TMR3CpuTickParavirtEnable(pVM);
-    }
-    else
-        LogRel(("GIM: KVM: VCPU%3d: Failed to write system-time struct. at %#RGp. rc=%Rrc\n",
-                pVCpu->idCpu, pKvmCpu->GCPhysSystemTime, rc));
-
-    return rc;
-}
-
-
 /**
  * Disables the KVM system-time struct.
  *
@@ -542,3 +588,43 @@ VMMR3_INT_DECL(int) gimR3KvmEnableWallClock(PVM pVM, RTGCPHYS GCPhysWallClock)
     return VMMR3EmtRendezvous(pVM, VMMEMTRENDEZVOUS_FLAGS_TYPE_ONCE, gimR3KvmEnableWallClockCallback, &WallClockInfo);
 }
 
+
+/**
+ * Enables the KVM system time structure.
+ *
+ * This can be done concurrently because the guest memory being updated is per-VCPU
+ * and the struct even has a "version" field which needs to be incremented
+ * before/after altering guest memory to allow concurrent updates from the host.
+ * Hence this is not being done in an EMT rendezvous. It -is- done in ring-3 since
+ * we call into ring-3 only TM code in the end.
+ *
+ * @returns VBox status code.
+ * @param   pVM             The cross context VM structure.
+ * @param   pVCpu           The cross context virtual CPU structure.
+ * @param   uMsrSystemTime  The system time MSR value being written.
+ */
+VMMR3_INT_DECL(int) gimR3KvmEnableSystemTime(PVMCC pVM, PVMCPUCC pVCpu, uint64_t uMsrSystemTime)
+{
+    Assert(uMsrSystemTime & MSR_GIM_KVM_SYSTEM_TIME_ENABLE_BIT);
+    PGIMKVM    pKvm    = &pVM->gim.s.u.Kvm;
+    PGIMKVMCPU pKvmCpu = &pVCpu->gim.s.u.KvmCpu;
+
+    /*
+     * Update the system-time struct.
+     * The system-time structs are usually placed at a different guest address for each VCPU.
+     */
+    pKvmCpu->uTsc                  = TMCpuTickGetNoCheck(pVCpu);
+    pKvmCpu->uVirtNanoTS           = ASMMultU64ByU32DivByU32(pKvmCpu->uTsc, RT_NS_1SEC, pKvm->cTscTicksPerSecond);
+    pKvmCpu->u64SystemTimeMsr      = uMsrSystemTime;
+    pKvmCpu->GCPhysSystemTime      = MSR_GIM_KVM_SYSTEM_TIME_GUEST_GPA(uMsrSystemTime);
+
+    int rc = gimR3KvmUpdateSystemTime(pVM, pVCpu);
+    if (RT_FAILURE(rc))
+    {
+        pKvmCpu->u64SystemTimeMsr = 0;
+        /* We shouldn't throw a #GP(0) here for buggy guests (neither does KVM apparently), see @bugref{8627}. */
+    }
+
+    return rc;
+}
+
