@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -776,7 +786,6 @@ bool pdmacEpIsTransferAllowed(PPDMASYNCCOMPLETIONENDPOINT pEndpoint, uint32_t cb
 /**
  * Called by the endpoint if a task has finished.
  *
- * @returns nothing
  * @param   pTask                     Pointer to the finished task.
  * @param   rc                        Status code of the completed request.
  * @param   fCallCompletionHandler    Flag whether the completion handler should be called to
@@ -930,7 +939,6 @@ int pdmR3AsyncCompletionEpClassInit(PVM pVM, PCPDMASYNCCOMPLETIONEPCLASSOPS pEpC
 /**
  * Worker terminating all endpoint classes.
  *
- * @returns nothing
  * @param   pEndpointClass    Pointer to the endpoint class to terminate.
  *
  * @remarks This method ensures that any still open endpoint is closed.
@@ -967,7 +975,6 @@ static void pdmR3AsyncCompletionEpClassTerminate(PPDMASYNCCOMPLETIONEPCLASS pEnd
 /**
  * Records the size of the request in the statistics.
  *
- * @returns nothing.
  * @param   pEndpoint    The endpoint to register the request size for.
  * @param   cbReq        Size of the request.
  */
@@ -1010,7 +1017,6 @@ static void pdmR3AsyncCompletionStatisticsRecordSize(PPDMASYNCCOMPLETIONENDPOINT
 /**
  * Records the required processing time of a request.
  *
- * @returns nothing.
  * @param   pEndpoint    The endpoint.
  * @param   cNsRun       The request time in nanoseconds.
  */
@@ -1223,7 +1229,6 @@ static int pdmR3AsyncCompletionStatisticsRegister(PPDMASYNCCOMPLETIONENDPOINT pE
 /**
  * Deregisters advanced statistics for one endpoint.
  *
- * @returns nothing.
  * @param   pEndpoint    The endpoint to deregister the advanced statistics for.
  */
 static void pdmR3AsyncCompletionStatisticsDeregister(PPDMASYNCCOMPLETIONENDPOINT pEndpoint)
@@ -1276,7 +1281,6 @@ int pdmR3AsyncCompletionTerm(PVM pVM)
 /**
  * Resume worker for the async completion manager.
  *
- * @returns nothing.
  * @param   pVM The cross context VM structure.
  */
 void pdmR3AsyncCompletionResume(PVM pVM)
@@ -1363,7 +1367,6 @@ static PPDMASYNCCOMPLETIONTASK pdmR3AsyncCompletionGetTask(PPDMASYNCCOMPLETIONEN
 /**
  * Puts a task in one of the caches.
  *
- * @returns nothing.
  * @param   pEndpoint    The endpoint the task belongs to.
  * @param   pTask        The task to cache.
  */
@@ -1501,7 +1504,6 @@ VMMR3DECL(int) PDMR3AsyncCompletionEpCreateForFile(PPPDMASYNCCOMPLETIONENDPOINT
 /**
  * Closes a endpoint waiting for any pending tasks to finish.
  *
- * @returns nothing.
  * @param   pEndpoint       Handle of the endpoint.
  */
 VMMR3DECL(void) PDMR3AsyncCompletionEpClose(PPDMASYNCCOMPLETIONENDPOINT pEndpoint)
@@ -1509,7 +1511,7 @@ VMMR3DECL(void) PDMR3AsyncCompletionEpClose(PPDMASYNCCOMPLETIONENDPOINT pEndpoin
     LogFlowFunc((": pEndpoint=%p\n", pEndpoint));
 
     /* Sanity checks. */
-    AssertReturnVoid(VALID_PTR(pEndpoint));
+    AssertReturnVoid(RT_VALID_PTR(pEndpoint));
 
     PPDMASYNCCOMPLETIONEPCLASS pEndpointClass = pEndpoint->pEpClass;
     pEndpointClass->pEndpointOps->pfnEpClose(pEndpoint);
