@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2023 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -96,7 +106,6 @@ static const DBGCCMD g_aCmds[] =
 /**
  * Frees a task.
  *
- * @returns nothing.
  * @param   pEndpoint    Pointer to the endpoint the segment was for.
  * @param   pTask        The task to free.
  */
@@ -147,7 +156,7 @@ PPDMACTASKFILE pdmacFileTaskAlloc(PPDMASYNCCOMPLETIONENDPOINTFILE pEndpoint)
         if (RT_FAILURE(rc))
             pTask = NULL;
 
-        LogFlow(("Allocated task %p\n", pTask));
+        LogFlow(("Allocated task %p -> %Rrc\n", pTask, rc));
     }
     else
     {
@@ -157,10 +166,9 @@ PPDMACTASKFILE pdmacFileTaskAlloc(PPDMASYNCCOMPLETIONENDPOINTFILE pEndpoint)
         pTask = pEndpoint->pTasksFreeHead;
         pEndpoint->pTasksFreeHead = pTask->pNext;
         ASMAtomicDecU32(&pEndpoint->cTasksCached);
+        pTask->pNext = NULL;
     }
 
-    pTask->pNext = NULL;
-
     return pTask;
 }
 
@@ -362,7 +370,7 @@ static DECLCALLBACK(void) pdmacFileEpTaskCompleted(PPDMACTASKFILE pTask, void *p
                 if (tsDelay < pEpClassFile->cMilliesNext)
                 {
                     ASMAtomicWriteU64(&pEpClassFile->cMilliesNext, tsDelay);
-                    TMTimerSetMillies(pEpClassFile->pTimer, tsDelay);
+                    TMTimerSetMillies(pVM, pEpClassFile->hTimer, tsDelay);
                 }
 
                 LogRel(("AIOMgr: Delaying request %#p for %u ms\n", pTaskFile, tsDelay));
@@ -507,7 +515,6 @@ int pdmacFileAioMgrCreate(PPDMASYNCCOMPLETIONEPCLASSFILE pEpClass, PPPDMACEPFILE
 /**
  * Destroys a async I/O manager.
  *
- * @returns nothing.
  * @param   pEpClassFile    Pointer to globals for the file endpoint class.
  * @param   pAioMgr         The async I/O manager to destroy.
  */
@@ -732,8 +739,12 @@ static DECLCALLBACK(int) pdmacEpFileDelayInject(PCDBGCCMD pCmd, PDBGCCMDHLP pCmd
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(void) pdmacR3TimerCallback(PVM pVM, PTMTIMER pTimer, void *pvUser)
+/**
+ * @callback_method_impl{FNTMTIMERINT, }
+ */
+static DECLCALLBACK(void) pdmacR3TimerCallback(PVM pVM, TMTIMERHANDLE hTimer, void *pvUser)
 {
+    Assert(hTimer == pEpClassFile->hTimer);
     uint64_t tsCur = RTTimeProgramMilliTS();
     uint64_t cMilliesNext = UINT64_MAX;
     PPDMASYNCCOMPLETIONEPCLASSFILE pEpClassFile = (PPDMASYNCCOMPLETIONEPCLASSFILE)pvUser;
@@ -783,7 +794,7 @@ static DECLCALLBACK(void) pdmacR3TimerCallback(PVM pVM, PTMTIMER pTimer, void *p
     if (cMilliesNext < pEpClassFile->cMilliesNext)
     {
         ASMAtomicWriteU64(&pEpClassFile->cMilliesNext, cMilliesNext);
-        TMTimerSetMillies(pEpClassFile->pTimer, cMilliesNext);
+        TMTimerSetMillies(pVM, hTimer, cMilliesNext);
     }
 }
 
@@ -865,11 +876,12 @@ static DECLCALLBACK(int) pdmacFileInitialize(PPDMASYNCCOMPLETIONEPCLASS pClassGl
         AssertRC(rc);
     }
 
-#ifdef PDM_ASYNC_COMPLETION_FILE_WITH_DELAY
-    rc = TMR3TimerCreateInternal(pEpClassFile->Core.pVM, TMCLOCK_REAL, pdmacR3TimerCallback, pEpClassFile, "AC Delay", &pEpClassFile->pTimer);
+# ifdef PDM_ASYNC_COMPLETION_FILE_WITH_DELAY
+    rc = TMR3TimerCreate(pEpClassFile->Core.pVM, TMCLOCK_REAL, pdmacR3TimerCallback, pEpClassFile,
+                         TMTIMER_FLAGS_NO_RING0, "AC Delay", &pEpClassFile->hTimer);
     AssertRC(rc);
     pEpClassFile->cMilliesNext = UINT64_MAX;
-#endif
+# endif
 #endif
 
     return rc;
